/*
 * NVIDIA_COPYRIGHT_BEGIN
 *
 * Copyright 1999-2002 by NVIDIA Corporation.  All rights reserved.  All
 * information contained herein is proprietary and confidential to NVIDIA
 * Corporation.  Any use, reproduction, or disclosure without the written
 * permission of NVIDIA Corporation is prohibited.
 *
 * NVIDIA_COPYRIGHT_END
 */

var g_BoardsJsId = "$Id: //sw/nvr40/diag/mods/js/boards.js#90 $";
//------------------------------------------------------------------------------
// BOARDS DESCRIPTION FILE (BOARDS.JS)
//
// This contains a list of board descriptions.  It is designed to be used
// in conjunction with the MemSize class.
//------------------------------------------------------------------------------

/*******************************************************************************

 HOW THIS FILE WORKS:

 The VERY FIRST element in the array is the format version of the table.
 Don't change this value unless you also change the memsize C++ code.

 Each ROW in the array is read to see if it "matches" the current board.
 If colums 5 (ib) through 15 (fbbar) all match, it is a "match"

 Columns 16 and 17 are a little bit special.  A value of 0x0000 is a
 "wildcard" that will match any board... BUT a card matches two descriptions
 and one of those descriptions uses wildcards and the other doesn't, only
 the one that doesn't (i.e., it has an explicit value in column 16 or 17)
 will be used.  In that case, the one with wildcards will be ignored.

 If no "matches" are found, MemSize.Run() will return a pass, but print out
 a warning message that no match was found.

 If a match is found, column 1 contains the "name" of the matched board.

 Once the match is found, the algorithm checks to make sure columns 2 and 3
 (bus and eb) match.  If both match, the test passes.  If either one does
 not match, the returns fail.

 Here is a description of each column

 COL#1 (name):    Name of the board

 // these two columns determine if a "matched" board passes or fails
 COL#2 (bus):     Bus width

 COL#3 (eb):      Number of external banks

 // these columns (col #4 and higher) determine if a boards is a "match"
 COL#4 (req):     Bit mask of required features:
                    Bit 0: Bonded as a non-GL board
                    Bit 1: Bonded as a GL board
                    Bits 31:2: TBD

 COL#5 (ib):      Number of internal banks

 COL#6 (rows):    Number of rows

 COL#7 (columns): Number of columns

 COL#8 (pci):     Board's PCI Device ID.  Legal values are from 0 to F.  Each
                  of the four bits map to the lower 2 bits of the device ID.
                  A value of C (1100b) means that a device ID ending in 2 or 3
                  is a match, but a device ID ending in 0 or 1 is not.

 COL#9 (NV):      Which NVIDIA device this board has on it.

 COL#10 (os):     1=mac; 0=everything else.  Some "apple-specific" boards can
                  not be identified any other way.

 COL#11 (straps): Memory strap values.  Legal values are from 0 to FFFF.  Each
                  of the sixteen bits maps to a value of the four-bit memory
                  strap value.  0x5800 (0101 1000 0000 0000) means that
                  strap values of 11, 12 and 14 are matches, all others are not
 COL#12 (dqs):    bits per DDR DQS signal.  A value of zero means SDR.

 COL#13 (agp):    1=agp, 0=pci

 COL#14 (ad):     1=PCI AD normal, 0=PCI AD reversed

 COL#15 (fbbar):  Framebuffer BAR size.
                     Bit 0:  32 MB allowed
                     Bit 1:  64 MB allowed
                     Bit 2: 128 MB allowed
                     Bit 3: 256 MB allowed
                     Bit 4: 512 MB allowed

 COL#16 (svid):   Subsystem Vendor ID.  0 means always match, any other value
                  must match exactly.

 COL#17 (ssid):   Subsystem ID.  0 means always match, any other value
                  must match exactly.

 // this column is not implemented right now
 COL#18 (desc):   This will eventually contain a handle to per-board layout
                  information, but that feature is not yet implemented.

*******************************************************************************/

var Boards =
[ 5, // table version
// name           bus  eb  reqbits ib   rows    cols   pci     NV  os  straps dqs agp ad fbbar  svid   ssid   desc

// NV10 boards
 ["P03",          128, 2  , 0x0000, 2, 0x0800, 0x100, 0x000F, 0x10, 0, 0x0008,  0, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P10",          128, 2  , 0x0000, 4, 0x0400, 0x100, 0x000F, 0x10, 0, 0x5800, 32, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P11",          128, 1  , 0x0000, 4, 0x1000, 0x100, 0x000F, 0x10, 0, 0x0400,  8, 1, 1, 0xFF, 0x0000, 0x0000,    ],

// NV11 boards
 ["P36",          128, 1  , 0x0000, 4, 0x1000, 0x100, 0x0009, 0x11, 0, 0x0002,  0, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["ASUS-P36",      64, 1  , 0x0000, 4, 0x1000, 0x100, 0x0009, 0x11, 0, 0x0002,  0, 1, 1, 0xFF, 0x1043, 0xa00f,    ],
 ["P38P39P49",    128, 1  , 0x0000, 4, 0x0800, 0x100, 0x000F, 0x11, 0, 0x0001,  0, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["Sony_mb"  ,     64, 1  , 0x0000, 4, 0x0800, 0x100, 0x000F, 0x11, 0, 0x0001,  0, 1, 0, 0xFF, 0x0000, 0x0000,    ],
 ["oldP40P66",    128, 1|2, 0x0000, 4, 0x0800, 0x100, 0x000F, 0x11, 1, 0x0001,  0, 1, 1, 0xFF, 0x0000, 0xA5B8,    ],
 ["P40",          128, 1  , 0x0000, 4, 0x0800, 0x100, 0x000F, 0x11, 1, 0x0001,  0, 1, 1, 0xFF, 0x0000, 0x0001,    ],
 ["Mosaic",        32, 1  , 0x0000, 4, 0x1800, 0x100, 0x0004, 0x11, 0, 0x8200, 32, 1, 1, 0xFF, 0x1028, 0x00f3,    ],
 ["Mosaic_Plus",   64, 1  , 0x0000, 4, 0x1000, 0x100, 0x0004, 0x11, 0, 0x2000,  8, 1, 1, 0xFF, 0x1028, 0x00f3,    ],
 ["P41",           64, 1  , 0x0000, 4, 0x1800, 0x100, 0x000C, 0x11, 0, 0x8300, 32, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P42",           64, 1  , 0x0000, 4, 0x1000, 0x100, 0x000C, 0x11, 0, 0x8000, 32, 1, 0, 0xFF, 0x0000, 0x0000,    ],
 ["P43",           32, 1|2, 0x0000, 4, 0x0800, 0x100, 0x0003, 0x11, 0, 0x0200, 32, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P43",           32, 1  , 0x0000, 4, 0x1000, 0x100, 0x0003, 0x11, 0, 0x8000, 32, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P44",           64, 1  , 0x0000, 4, 0x1000, 0x300, 0x000F, 0x11, 0, 0x0C00,  8, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P45",           64, 1  , 0x0000, 4, 0x1000, 0x100, 0x000F, 0x11, 0, 0x0400,  8, 0, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P48",          128, 1  , 0x0000, 4, 0x0800, 0x100, 0x000F, 0x11, 0, 0x0001,  0, 0, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P60P36-64",     64, 1  , 0x0000, 4, 0x1000, 0x100, 0x0002, 0x11, 0, 0x0002,  0, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P60",           64, 1  , 0x0000, 4, 0x1000, 0x200, 0x0002, 0x11, 0, 0x0060,  0, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P66",          128, 2  , 0x0000, 4, 0x0800, 0x100, 0x000F, 0x11, 1, 0x0001,  0, 1, 1, 0xFF, 0x0000, 0x0002,    ],
 ["AP80-16MB",     64, 1  , 0x0000, 4, 0x0800, 0x100, 0x0001, 0x11, 1, 0x0300, 32, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["AP80-32MB",     64, 1  , 0x0000, 4, 0x1000, 0x100, 0x0001, 0x11, 1, 0x8000, 32, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["pny-ref",      128, 1  , 0x0000, 4, 0x1000, 0x100, 0x0001, 0x11, 0, 0x0002,  0, 0, 1, 0xFF, 0x0000, 0x0000,    ],
 ["lt2853b",       64, 1  , 0x0000, 4, 0x1000, 0x100, 0x0009, 0x11, 0, 0x0002,  0, 1, 1, 0xFF, 0x110A, 0x0086,    ],
 ["FujSieP26",     64, 1  , 0x0000, 4, 0x1000, 0x100, 0x0009, 0x11, 0, 0x0002,  0, 1, 1, 0xFF, 0x110A, 0x0075,    ],
 ["CpqTablet",     32, 1  , 0x0000, 4, 0x1000, 0x100, 0x0004, 0x11, 0, 0x8000, 32, 0, 1, 0xFF, 0x0e11, 0x00b5,    ],
 ["MSI-8839",      64, 1  , 0x0000, 4, 0x1000, 0x300, 0x0001, 0x11, 0, 0x0042,  0, 1, 1, 0xFF, 0x1462, 0x0000,    ],
 ["F22",          128, 1  , 0x0000, 4, 0x1000, 0x100, 0x0008, 0x11, 1, 0x0002,  0, 0, 1, 0xFF, 0x139c, 0x0202,    ],

// NV15 boards
 ["P20",          128, 2  , 0x0000, 4, 0x0400, 0x100, 0x000F, 0x15, 0, 0x5800, 32, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P20P22",       128, 1|2, 0x0000, 4, 0x0800, 0x100, 0x000F, 0x15, 0, 0x0300, 32, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P30",          128, 1  , 0x0000, 4, 0x1000, 0x100, 0x0003, 0x15, 0, 0x0400,  8, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P32",          128, 1  , 0x0000, 4, 0x1000, 0x100, 0x000C, 0x15, 0, 0x0400,  8, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P23",          128, 1  , 0x0000, 4, 0x1000, 0x100, 0x0002, 0x15, 0, 0x8000, 32, 1, 1, 0xFF, 0x0000, 0x0000,    ],

// NV17 Boards
 ["E63",          128, 1  , 0x0000, 4, 0x1000, 0x100, 0x1000, 0x17, 0, 0x2000,  8, 0, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P69",           64, 1  , 0x0000, 4, 0x1000, 0x200, 0x0400, 0x17, 0, 0x0800,  8, 0, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P70",          128, 1  , 0x0000, 4, 0x1800, 0x100, 0x0102, 0x17, 0, 0x8100, 32, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P71",          128, 1  , 0x0000, 4, 0x1000, 0x300, 0x0507, 0x17, 0, 0x0042,  0, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P72",          128, 1  , 0x0000, 4, 0x1000, 0x100, 0x0003, 0x17, 0, 0x8000,  8, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P72DELL",      128, 1  , 0x0000, 4, 0x1800, 0x100, 0x1030, 0x17, 0, 0x2080,  8, 1, 1, 0xFF, 0x1028, 0x00d4,    ],
 ["P95",           64, 1  , 0x0000, 4, 0x1000, 0x100, 0x0010, 0x17, 0, 0x2000,  8, 1, 1, 0xFF, 0x1028, 0x013f,    ],
 ["P73/P150",      64, 1  , 0x0000, 4, 0x1800, 0x200, 0x0107, 0x17, 0, 0x2800,  8, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P73P150",       64, 1  , 0x0000, 4, 0x1800, 0x100, 0x0107, 0x17, 0, 0x0400,  8, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["A74",          128, 1  , 0x0000, 4, 0x1800, 0x100, 0x0007, 0x17, 1, 0xDA00, 32, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P75",          128, 1  , 0x0000, 4, 0x1000, 0x100, 0x0801, 0x17, 0, 0x2000,  8, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P77",           64, 1  , 0x0000, 4, 0x1000, 0x100, 0x0403, 0x17, 0, 0x2000,  8, 0, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P78",          128, 1  , 0x0000, 4, 0x1000, 0x100, 0x0102, 0x17, 0, 0x2000,  8, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P79",          128, 1  , 0x0000, 4, 0x1000, 0x100, 0x0001, 0x17, 0, 0x8000, 32, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P150",          64, 1  , 0x0000, 4, 0x1000, 0x100, 0x0400, 0x17, 0, 0x0400,  8, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P155-SKU0",    128, 1  , 0x0000, 4, 0x1000, 0x100, 0x0008, 0x17, 0, 0x0400,  8, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P155-SKU1",     64, 1  , 0x0000, 4, 0x1000, 0x200, 0x0008, 0x17, 0, 0x0800,  8, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P155-SKU5",     64, 1  , 0x0000, 4, 0x1000, 0x300, 0x0400, 0x17, 0, 0x0800,  8, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["C13",           64, 1  , 0x0001, 4, 0x1000, 0x100, 0x0400, 0x17, 0, 0x0400,  8, 0, 1, 0x04, 0x0000, 0x0000,    ],
 ["C94",          128, 1  , 0x0000, 4, 0x1000, 0x200, 0x0002, 0x17, 0, 0x0800,  8, 1, 1, 0xFF, 0x1462, 0x8680,    ],
 ["C94",          128, 1  , 0x0000, 4, 0x1000, 0x100, 0x0002, 0x17, 0, 0x0400,  8, 1, 1, 0xFF, 0x1b13, 000000,    ],
 ["MAP16",         32, 1  , 0x0000, 4, 0x1000, 0x100, 0x2000, 0x17, 0, 0x2000,  8, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["MAP32",         64, 1  , 0x0000, 4, 0x1000, 0x100, 0x0040, 0x17, 0, 0x2000,  8, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["MAP64",        128, 1  , 0x0000, 4, 0x1000, 0x100, 0x0200, 0x17, 0, 0x2000,  8, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["E134/MAP464",  128, 1  , 0x0001, 4, 0x1000, 0x100, 0x0040, 0x17, 0, 0x8020,  8, 1, 1, 0x0E, 0x0000, 0x0000,    ],
 ["E134/MAP464GL",128, 1  , 0x0002, 4, 0x1000, 0x100, 0x2000, 0x17, 0, 0x8020,  8, 1, 1, 0x0E, 0x0000, 0x0000,    ],
 ["PNY-NV17",     128, 1  , 0x0000, 4, 0x1000, 0x100, 0x0004, 0x17, 0, 0x0002,  0, 0, 1, 0xFF, 0x0000, 0x0000,    ],
 ["FP17-imac",     64, 1  , 0x0000, 4, 0x1000, 0x100, 0x0010, 0x17, 1, 0x8000, 32, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["Sony-NB",       64, 1  , 0x0000, 4, 0x1000, 0x100, 0x0020, 0x17, 0, 0x2000,  8, 1, 1, 0xFF, 0x104d, 0x8110,    ],
 ["Sony-NB_NE5",   32, 1  , 0x0000, 4, 0x1000, 0x100, 0x0020, 0x17, 0, 0x2000,  8, 1, 1, 0xFF, 0x104d, 0x8143,    ],
 ["Toshiba-NB",   128, 1  , 0x0000, 4, 0x0800, 0x100, 0x0010, 0x17, 0, 0x0200, 32, 1, 1, 0xFF, 0x1179, 0xff00,    ],
 ["MSI-8885",      64, 1  , 0x0000, 4, 0x1000, 0x200, 0x0004, 0x17, 0, 0x0800,  8, 0, 1, 0xFF, 0x1462, 0x8853,    ],
 ["MSI-8883",     128, 1  , 0x0000, 4, 0x1000, 0x200, 0x0001, 0x17, 0, 0x0800,  8, 1, 1, 0xFF, 0x1462, 0x8836,    ],
 ["ASUS02DX1043", 128, 1  , 0x0000, 4, 0x1000, 0x200, 0x0002, 0x17, 0, 0x0800,  8, 1, 1, 0xFF, 0x1043, 0x0000,    ],
 ["CV00",         128, 1  , 0x0000, 4, 0x1000, 0x100, 0x1000, 0x17, 1, 0x2000,  8, 0, 1, 0xFF, 0x139c, 0x0204,    ],
 ["CV01",         128, 1  , 0x0000, 4, 0x1000, 0x100, 0x1000, 0x17, 1, 0x2000,  8, 0, 1, 0xFF, 0x139c, 0x0205,    ],
 ["CV02",         128, 1  , 0x0000, 4, 0x1000, 0x100, 0x1000, 0x17, 1, 0x2000,  8, 0, 1, 0xFF, 0x139c, 0x0206,    ],
 ["AP99P84" ,128 | 64, 1  , 0x0000, 4, 0x1000, 0x100, 0x0200, 0x17, 1, 0x2000,  8, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["Raptor2",       64, 1  , 0x0001, 4, 0x1000, 0x100, 0x0040, 0x17, 0, 0x4000,  8, 1, 1, 0x04, 0x103c, 0x08b0,    ],

// NV18 boards
 ["P104/P112",    128, 1  , 0x0000, 4, 0x1000, 0x100, 0x000A, 0x18, 0, 0x2000,  8, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P111/P117/P118",64, 1  , 0x0000, 4, 0x1000, 0x200, 0x000F, 0x18, 0, 0x0A00,  8, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P113/P115",    128, 1  , 0x0000, 4, 0x1000, 0x100, 0x0001, 0x18, 0, 0x2000,  8, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P117/P118",     64, 1  , 0x0001, 4, 0x1000, 0x200, 0x0006, 0x18, 0, 0x0133,  8, 1, 1, 0x04, 0x0000, 0x0000,    ],
 ["P119",          64, 2  , 0x0001, 4, 0x1000, 0x200, 0x0003, 0x18, 0, 0x0080,  8, 1, 1, 0x04, 0x0000, 0x0000,    ],
 ["E133",         128, 1  , 0x0000, 4, 0x1000, 0x100, 0x0004, 0x18, 0, 0x2000,  8, 1, 1, 0x04, 0x0000, 0x0000,    ],
 ["P141/P143",    128, 1  , 0x0001, 4, 0x1000, 0x200, 0x0001, 0x18, 0, 0x0008,  8, 1, 1, 0x04, 0x0000, 0x0000,    ],
 ["P162-1",       128, 1  , 0x0001, 4, 0x1000, 0x200, 0x0002, 0x18, 0, 0x0008,  8, 1, 1, 0x0C, 0x0000, 0x0000,    ],
 ["P162-2",       128, 1  , 0x0001, 4, 0x1000, 0x200, 0x0002, 0x18, 0, 0x0002,  8, 1, 1, 0x08, 0x0000, 0x0000,    ],
 ["MSI-8893",     128, 1  , 0x0000, 4, 0x1000, 0x100, 0x0002, 0x18, 0, 0x0040,  8, 1, 1, 0xFF, 0x1462, 0x8932,    ],
 ["AQ26/27", 128 | 64, 1  , 0x0001, 4, 0x1000, 0x100, 0x0002, 0x18, 1, 0xB000,  8, 1, 1, 0xFF, 0x10de, 0x0010,    ],
 ["A114",         128, 1  , 0x0001, 4, 0x1000, 0x100, 0x0001, 0x18, 1, 0x0020,  8, 1, 1, 0x02, 0x10de, 0x0010,    ],
 ["Trigem-C116",  128, 1  , 0x0000, 4, 0x1000, 0x200, 0x0002, 0x18, 0, 0x0800,  8, 1, 1, 0xFF, 0x16eb, 0x0621,    ],

// NV20 boards
 ["P50",          128, 1  , 0x0000, 4, 0x1000, 0x300, 0x000F, 0x20, 0, 0x4C00,  8, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P51",          128, 1  , 0x0000, 4, 0x1000, 0x100, 0x000F, 0x20, 1, 0x0400,  8, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P53",          128, 1  , 0x0000, 4, 0x1000, 0x100, 0x000F, 0x20, 0, 0x8000, 32, 1, 1, 0xFF, 0x0000, 0x0000,    ],

// NV25 boards
 ["P80P82P83",    128, 1|2, 0x0000, 4, 0x1000, 0x100, 0xFFFF, 0x25, 0, 0x7000,  8, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["A81A84",       128, 1|2, 0x0000, 4, 0x1000, 0x100, 0xFFFF, 0x25, 1, 0x3000,  8, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P85",          128, 1  , 0x0000, 4, 0x1000, 0x300, 0x0008, 0x25, 0, 0x0C00,  8, 1, 1, 0xFF, 0x0000, 0x0000,    ],

// NV28 boards
 ["P138-64",       64, 1  , 0x0001, 4, 0x1000, 0x100, 0x0040, 0x28, 0, 0x0080,  8, 1, 1, 0x02, 0x1028, 0x0000,    ],
 ["P138-128",     128, 1  , 0x0001, 4, 0x1000, 0x100, 0x0040, 0x28, 0, 0x0010,  8, 1, 1, 0x02, 0x1028, 0x0000,    ],
 ["C139",         128, 1  , 0x0002, 4, 0x1000, 0x100, 0x1000, 0x28, 0, 0x1000,  8, 1, 1, 0x02, 0x1028, 0x0000,    ],
 ["P151",         128, 1  , 0x0000, 4, 0x1000, 0x300, 0x0002, 0x28, 0, 0x0068,  8, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["C151",         128, 1  , 0x0000, 4, 0x1000, 0x200, 0x0008, 0x28, 0, 0x0008,  8, 1, 1, 0xFF, 0x1462, 0x8891,    ],
 ["P152-SS",      128, 1  , 0x0000, 4, 0x1000, 0x100, 0x0004, 0x28, 0, 0x00c0,  8, 1, 1, 0x02, 0x0000, 0x0000,    ],
 ["P152-DS",      128, 2  , 0x0000, 4, 0x1000, 0x100, 0x0005, 0x28, 0, 0x00d0,  8, 1, 1, 0x04, 0x0000, 0x0000,    ],
 ["P152-WS",      128, 2  , 0x0000, 4, 0x1000, 0x100, 0x0300, 0x28, 0, 0x0010,  8, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["E153",         128, 1  , 0x0000, 4, 0x1000, 0x100, 0x1000, 0x28, 0, 0x0010,  8, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["C193",         128, 1  , 0x0000, 4, 0x1000, 0x100, 0x1000, 0x28, 0, 0x4000,  8, 1, 1, 0xFF, 0x1028, 0x0000,    ],

// NV30 boards
 ["E121",         128, 1|2, 0x0000, 4, 0x1000, 0x100, 0x0001, 0x30, 0, 0x0001,  0, 1, 1, 0xFF, 0x0000, 0x0000,    ],
 ["P126",         128, 2  , 0x0001, 4, 0x1000, 0x100, 0x0006, 0x30, 0, 0x0001,  0, 1, 1, 0x04, 0x0000, 0x0000,    ],
 ["P128",         128, 2  , 0x0002, 4, 0x1000, 0x100, 0x0300, 0x30, 0, 0x0001,  0, 1, 1, 0x04, 0x0000, 0x0000,    ],

// NV31 boards
 ["E133",         128, 1  , 0x0000, 4, 0x1000, 0x100, 0x0400, 0x31, 0, 0x2020,  0, 1, 1, 0x04, 0x0000, 0x0000,    ],
 ["P136GL",       128, 2  , 0x0002, 4, 0x1000, 0x300, 0x1000, 0x31, 0, 0x0302,  0, 1, 1, 0x08, 0x1028, 0x0000,    ],
 ["P140",         128, 2  , 0x0001, 4, 0x1000, 0x100, 0x0002, 0x31, 0, 0x0048,  0, 1, 1, 0x04, 0x0000, 0x0000,    ],
 ["P141/P143",    128, 1  , 0x0001, 4, 0x3000, 0x200, 0x0005, 0x31, 0, 0x008E,  0, 1, 1, 0x0C, 0x0000, 0x0000,    ],
 ["P141GL",       128, 1  , 0x0002, 4, 0x3000, 0x200, 0x0200, 0x31, 0, 0x008A,  0, 1, 1, 0x0C, 0x0000, 0x0000,    ],
 ["P162",         128, 1  , 0x0001, 4, 0x2000, 0x200, 0x0010, 0x31, 0, 0x0080,  0, 1, 1, 0x08, 0x0000, 0x0000,    ],
 ["E134/MAP464",  128, 1  , 0x0001, 4, 0x1000, 0x100, 0x0840, 0x31, 0, 0x8020,  0, 1, 1, 0x0E, 0x0000, 0x0000,    ],
 ["E134/MAP464GL",128, 1  , 0x0002, 4, 0x1000, 0x100, 0x2000, 0x31, 0, 0x8020,  0, 1, 1, 0x0E, 0x0000, 0x0000,    ],
 ["Dell-P136",    128, 2  , 0x0001, 4, 0x1000, 0x100, 0x0400, 0x31, 0, 0x0002,  0, 1, 1, 0x08, 0x1028, 0x019c,    ],
 ["Sony-Chingis", 128, 1  , 0x0001, 4, 0x1000, 0x100, 0x0400, 0x31, 0, 0x2000,  0, 1, 1, 0x08, 0x104d, 0x814f,    ],
 ["DN95-NV31" ,   128, 1  , 0x0001, 4, 0x1000, 0x100, 0x0400, 0x31, 0, 0x0020,  0, 1, 1, 0x08, 0x1509, 0x3132,    ],


// NV34 boards
 ["P118/P162/P164",64, 1  , 0x0001, 4, 0x3000, 0x200, 0x000C, 0x34, 0, 0x08F2,  8, 1, 1, 0x06, 0x0000, 0x0000,    ],
 ["P135/E133",    128, 1  , 0x0000, 4, 0x1000, 0x100, 0x0010, 0x34, 0, 0x0020,  8, 1, 1, 0x04, 0x0000, 0x0000,    ],
 ["P136",         128, 1  , 0x0001, 4, 0x1000, 0x100, 0x0010, 0x34, 0, 0x0010,  8, 1, 1, 0x08, 0x0000, 0x0000,    ],
 ["P140",         128, 2  , 0x0001, 4, 0x1000, 0x100, 0x0002, 0x34, 0, 0x0068,  8, 1, 1, 0x04, 0x0000, 0x0000,    ],
 ["P143/P162/P164",128,1  , 0x0001, 4, 0x1000, 0x200, 0x0004, 0x34, 0, 0x000D,  8, 1, 1, 0x0C, 0x0000, 0x0000,    ],
 ["P162GL",       128, 1  , 0x0002, 4, 0x1000, 0x200, 0x0800, 0x34, 0, 0x000F,  8, 1, 1, 0x0C, 0x0000, 0x0000,    ],
 ["P162-256MB",   128, 1  , 0x0001, 4, 0x2000, 0x200, 0x0004, 0x34, 0, 0x0080,  8, 1, 1, 0x08, 0x0000, 0x0000,    ],
 ["P162",         128, 1  , 0x0001, 4, 0x1000, 0x100, 0x0004, 0x34, 0, 0x0010,  8, 1, 1, 0x04, 0x0000, 0x0000,    ],
 ["E15/MAP34",    128, 1  , 0x0001, 4, 0x1000, 0x100, 0x0100, 0x34, 0, 0x0020,  8, 1, 1, 0x04, 0x0000, 0x0000,    ],
 ["E133",         128, 1  , 0x0001, 4, 0x1000, 0x100, 0x0010, 0x34, 0, 0x0080,  8, 1, 1, 0x04, 0x0000, 0x0000,    ],
 ["E134",         128, 1  , 0x0001, 4, 0x1000, 0x100, 0x0100, 0x34, 0, 0x0080,  8, 1, 1, 0x04, 0x0000, 0x0000,    ],
 ["E134/MAP464",  128, 1  , 0x0001, 4, 0x1000, 0x100, 0x0040, 0x34, 0, 0x8020,  8, 1, 1, 0x0E, 0x0000, 0x0000,    ],
 ["E134/MAP464GL",128, 1  , 0x0002, 4, 0x1000, 0x100, 0x2000, 0x34, 0, 0x8020,  8, 1, 1, 0x0E, 0x0000, 0x0000,    ],
 ["DN95-NV34",    128, 1  , 0x0001, 4, 0x1000, 0x100, 0x0010, 0x34, 0, 0x0028,  8, 1, 1, 0x04, 0x1509, 0x3130,    ],
 ["AQ27",         128, 1  , 0x0001, 4, 0x1000, 0x100, 0x0100, 0x34, 1, 0x0020,  8, 1, 1, 0xFF, 0x10de, 0x0010,    ],
 ["A146",         128, 1  , 0x0001, 4, 0x1000, 0x100, 0x0002, 0x34, 1, 0x0020,  8, 1, 1, 0x04, 0x10de, 0x0010,    ],
 ["AQ26B",        128, 1  , 0x0001, 4, 0x1000, 0x100, 0x0200, 0x34, 1, 0x0020,  8, 1, 1, 0x04, 0x10de, 0x0010,    ],
 ["Dell-P135",    128, 1  , 0x0001, 4, 0x1000, 0x100, 0x0010, 0x34, 0, 0x0020,  8, 1, 1, 0x08, 0x1028, 0x015f,    ],


// NV35 boards
 ["P170/P172",    256, 2  , 0x0001, 4, 0x1000, 0x100, 0x0009, 0x35, 0, 0x0020,  0, 1, 1, 0x0C, 0x0000, 0x0000,    ],
 ["P170/172/177-SS",256,1 , 0x0001, 4, 0x1000, 0x100, 0x0002, 0x35, 0, 0x0020,  0, 1, 1, 0x04, 0x0000, 0x0000,    ],
 ["P172",         256, 2  , 0x0001, 4, 0x1000, 0x100, 0x0002, 0x35, 0, 0x0020,  0, 1, 1, 0x08, 0x0000, 0x0000,    ],
 ["P171",         256, 2  , 0x0002, 4, 0x1000, 0x100, 0x0100, 0x35, 0, 0x0020,  0, 1, 1, 0x08, 0x0000, 0x0000,    ],
 ["P177-BADSTRAP",200, 1  , 0x0001, 4, 0x1000, 0x100, 0x0004, 0x35, 0, 0x0020,  0, 1, 1, 0x10, 0x0000, 0x0000,    ],
 ["BAD_STRAP",    200, 1  , 0x0003, 4, 0xFF00, 0xF00, 0xFFFF, 0x35, 0, 0xFF00,  0, 1, 1, 0xFF, 0x0000, 0x0000,    ],

// NV36 boards
 ["P190",         128, 2  , 0x0001, 4, 0x1000, 0x100, 0x0002, 0x36, 0, 0x0020,  0, 1, 1, 0x08, 0x0000, 0x0000,    ],
 ["P194",         128, 1  , 0x0001, 4, 0x1000, 0x200, 0x0008, 0x36, 0, 0x0002,  0, 1, 1, 0x04, 0x0000, 0x0000,    ],
];
