// $Id: //sw/nvr40/diag/mods/js/drf.h#1 $
//------------------------------------------------------------------------------
// Device Register Field macros.
//
// Copyright (C) 2000 NVIDIA Corporation. All Rights Reserved.
//------------------------------------------------------------------------------
// 45678901234567890123456789012345678901234567890123456789012345678901234567890

#ifndef INCLUDED_DRF_H
#define INCLUDED_DRF_H

//------------------------------------------------------------------------------
// Register access functions.
//------------------------------------------------------------------------------

#define REG_RD08(a)   ( Gpu.RegRd08(a)   )
#define REG_RD16(a)   ( Gpu.RegRd16(a)   )
#define REG_RD32(a)   ( Gpu.RegRd32(a)   )

#define REG_WR08(a,d) ( Gpu.RegWr08(a,d) )
#define REG_WR16(a,d) ( Gpu.RegWr16(a,d) )
#define REG_WR32(a,d) ( Gpu.RegWr32(a,d) )

#define BIT(b)           (1<<(b))
#define DEVICE_BASE(d)   (0?d)
#define DEVICE_EXTENT(d) (1?d)
#define DRF_SHIFT(drf)   ((0?drf) % 32)
#define DRF_MASK(drf)    (0xFFFFFFFF>>>(31-((1?drf) % 32)+((0?drf) % 32)))
#define DRF_DEF(d,r,f,c) ((NV##d##r##f##c) << DRF_SHIFT(NV##d##r##f))
#define DRF_NUM(d,r,f,n) (((n)&DRF_MASK(NV##d##r##f)) << DRF_SHIFT(NV##d##r##f))
#define DRF_VAL(d,r,f,v) (((v)>>>DRF_SHIFT(NV##d##r##f))&DRF_MASK(NV##d##r##f))
#define REF_VAL(drf,v)   (((v)>>>DRF_SHIFT(drf))&DRF_MASK(drf))
#define REF_NUM(drf,n)   (((n)&DRF_MASK(drf))<<DRF_SHIFT(drf))

#define REG_WR_DRF_NUM(d,r,f,n) \
   REG_WR32( NV##d##r, DRF_NUM(d,r,f,n) )

#define REG_WR_DRF_DEF(d,r,f,c) \
   REG_WR32( NV##d##r, DRF_DEF(d,r,f,c) )

#define FLD_WR_DRF_NUM(d,r,f,n)                                 \
   REG_WR32( NV##d##r,                                          \
      ( REG_RD32(NV##d##r)                                      \
         & ~(DRF_MASK(NV##d##r##f) << DRF_SHIFT(NV##d##r##f)) ) \
      | DRF_NUM(d,r,f,n) )

#define FLD_WR_DRF_DEF(d,r,f,c)                                 \
   REG_WR32( NV##d##r,                                          \
      ( REG_RD32( NV##d##r)                                     \
         & ~(DRF_MASK(NV##d##r##f) << DRF_SHIFT(NV##d##r##f)) ) \
      | DRF_DEF(d,r,f,c) )

#define FLD_WR_DRF_DEF_RAM(i,d,r,f,c)                           \
   REG_WR32( NV ## d ## r(i),                                   \
      ( REG_RD32(NV##d##r(i))                                   \
         & ~(DRF_MASK(NV##d##r##f) << DRF_SHIFT(NV##d##r##f)) ) \
      | DRF_DEF(d,r,f,c) )

#define REG_RD_DRF(d,r,f) \
   ( (REG_RD32(NV##d##r) >>> DRF_SHIFT(NV##d##r##f)) & DRF_MASK(NV##d##r##f) )

#define FLD_DRF_NUM(o,d,r,f,n)                                \
   ( (o & ~(DRF_MASK(NV##d##r##f) << DRF_SHIFT(NV##d##r##f))) \
   | DRF_NUM(d,r,f,n) )

#define FLD_DRF_DEF(o,d,r,f,c)                                \
   ( (o & ~(DRF_MASK(NV##d##r##f) << DRF_SHIFT(NV##d##r##f))) \
   | DRF_DEF(d,r,f,c) )

//------------------------------------------------------------------------------
// Frame buffer access functions.
//------------------------------------------------------------------------------

#define FB_RD08(a)   ( Gpu.FbRd08(a)   )
#define FB_RD16(a)   ( Gpu.FbRd16(a)   )
#define FB_RD32(a)   ( Gpu.FbRd32(a)   )

#define FB_WR08(a,d) ( Gpu.FbWr08(a,d) )
#define FB_WR16(a,d) ( Gpu.FbWr16(a,d) )
#define FB_WR32(a,d) ( Gpu.FbWr32(a,d) )

#endif // !INCLUDED_DRF_H