/*
 * NVIDIA_COPYRIGHT_BEGIN
 *
 * Copyright 1999-2002 by NVIDIA Corporation.  All rights reserved.  All
 * information contained herein is proprietary and confidential to NVIDIA
 * Corporation.  Any use, reproduction, or disclosure without the written
 * permission of NVIDIA Corporation is prohibited.
 *
 * NVIDIA_COPYRIGHT_END
 */

//------------------------------------------------------------------------------
// $Id: //sw/nvr40/diag/mods/js/gpu.js#2 $
//------------------------------------------------------------------------------

function main()
{
   var rc = RC.OK;

   // Initialize the GPU.
   if (RC.OK != (rc = Gpu.Initialize()))
      return rc;

   // Set the clocks if necessary.
   var i;
   for (i = 1; i < Argv.length; ++i)
   {
      switch (Argv[i])
      {
         case "-mclk":
         {
            Dac.MEMORY_CLOCK = parseFloat(Argv[++i]) * 1e6;
            break;
         }
   
         case "-nvclk":
         {
            Dac.GRAPHICS_CLOCK = parseFloat(Argv[++i]) * 1e6;
            break;
         }
      
         case "-help":
         {
            Out.Printf(Out.PriHigh,
               Argv[0] + " command line options:\n"
               +  "   -mclk MHz     Set memory clock to MHz.\n"
               +  "   -nvclk MHz    Set NV (graphics) clock to MHz.\n"
               +  "   -help         Print command line options.\n"
               +  "\n"
            );

            return RC.OK;
         }
      }
   }
   var caps = [];
   NvRm.ConfigGet(18, caps);
   
   // Print report.
   Out.Printf(Out.PriHigh, "\nGPU Configuration\n");
   Out.Printf(Out.PriHigh, "DID            : %#06x (%s)\n", Gpu.PciDeviceId, Gpu.DeviceName);
   Out.Printf(Out.PriHigh, "GL?            : %s\n",         (caps[0] & 0x4000) ? "yes" : "no");
   Out.Printf(Out.PriHigh, "Revision       : %#04x\n",      Bus.Revision);
   Out.Printf(Out.PriHigh, "Subsystem VID  : %#06x\n",      Bus.SubsystemVendorId);
   Out.Printf(Out.PriHigh, "Subsystem DID  : %#06x\n",      Bus.SubsystemDeviceId);
   Out.Printf(Out.PriHigh, "Memory Size    : %u MB\n",      FrameBuffer.GRAPHICS_RAMAMOUNT / (1024*1024));
   Out.Printf(Out.PriHigh, "Memory Type    : %s\n",         FrameBuffer.TYPE_DDR ? "DDR" : "SDR");
   Out.Printf(Out.PriHigh, "ROM version    : %s\n",         BiosVersion);
   Out.Printf(Out.PriHigh, "TV Encoder     : %s\n",         TvEncoder);
   Out.Printf(Out.PriHigh, "IRQ            : %u\n",         Bus.Irq);

   Out.Printf(Out.PriHigh, "\nClocks\n");
   Out.Printf(Out.PriHigh, "Graphics       : %5.1f MHz\n", Dac.GRAPHICS_CLOCK / 1e6);
   Out.Printf(Out.PriHigh, "Memory         : %5.1f MHz\n", Dac.MEMORY_CLOCK / 1e6);

   if (Bus.IsAgpEnabled)
   {
      Out.Printf(Out.PriHigh, "\nAGP Configuration\n");
      Out.Printf(Out.PriHigh, "Revision       : %s\n", Bus.IsAgp3 ? "3.0" : "2.0");
      Out.Printf(Out.PriHigh, "Request Depth  : %u\n", Bus.AgpRequestDepth);
      Out.Printf(Out.PriHigh, "Sideband       : %s\n", Bus.IsAgpSbaEnabled ? "Enabled" : "Disabled");
      Out.Printf(Out.PriHigh, "Fastwrite      : %s\n", Bus.IsAgpFwEnabled ? "Enabled" : "Disabled");
      Out.Printf(Out.PriHigh, "Data Rate      : %u\n", Bus.AgpDataRate);
   }
   else
   {
      Out.Printf(Out.PriHigh, "\nAGP is disabled.\n");
   }

   return RC.OK;
}
