/*
 * NVIDIA_COPYRIGHT_BEGIN
 *
 * Copyright 1999-2001 by NVIDIA Corporation.  All rights reserved.  All
 * information contained herein is proprietary and confidential to NVIDIA
 * Corporation.  Any use, reproduction, or disclosure without the written
 * permission of NVIDIA Corporation is prohibited.
 *
 * NVIDIA_COPYRIGHT_END
 */

//------------------------------------------------------------------------------
// $Id: //sw/nvr40/diag/mods/js/mods.js#11 $
//------------------------------------------------------------------------------

// MODS script file.

#include "mods.h"
#include "drf.h"

//------------------------------------------------------------------------------
// Global variables.
//
var g_NvBaseAddress         = 0;
var g_FbBaseAddress         = 0;

//------------------------------------------------------------------------------
// Initialize variables.
//
Macros =
[
   [ "h",     ""    ],
   [ "q",     ""    ],
   [ "rr",    "x"   ],
   [ "rw",    "xx"  ],
   [ "fr",    "x"   ],
   [ "fw",    "xxx" ],
   [ "i",     "x"   ],
   [ "o",     "xx"  ],
   [ "cr",    "x"   ],
   [ "cw",    "xx"  ],
   [ "clks",  ""    ],
   [ "mclk",  "f"   ],
   [ "gclk",  "f"   ],
   [ "nvclk", "f"   ],
   [ "pclk",  "xf"  ],
   [ "mono",  ""    ],
   [ "vga",   ""    ],
];

//------------------------------------------------------------------------------
// main
//
function mods_js_main
(
)
{
   var rc = OK;

   // Setup the registry.
   if (OperatingSystem == "Dos")
   {
      if (undefined == Registry.ResourceManager)
      {
         Registry.ResourceManager = new Object;
      }
      Registry.ResourceManager.SoftEDIDs = 1;
      Registry.ResourceManager.RmRobustChannels = 1;
      Registry.ResourceManager.WatchdogRecalDriveSlew = 1;
      Registry.ResourceManager.MaxPerfWithPerfMon = 0;
      Registry.ResourceManager.RMTvoSuppressHDTV = 0;
   }

   // Initialize the graphics devices.
   CHECK_RC(Gpu.Initialize());

   if ("Dos" == OperatingSystem)
   {
      // First modeset, RM sets default mclk and nvclk, does first perfmode
      // transition from mode 0 (vbios) to mode 1 (2d quiet).
      CHECK_RC( Display.SetMode(640,480,32,60) );

      if (!Vga.MONOCHROME)
      {
         Vga.SetVesaMode(3);
      }
   }

   // Do chip specific overrides.
   CHECK_RC(DoGpuOverrides());

   // On boards with "silent running" aka "mojo" vbios, lock us into the max
   // permformance setting to prevent the rm changing clks/voltage/fanspeed
   // behind our backs.
   Master.PERF_MODE = 102;

   // Force RM to leave nv core voltage alone from here on out.
   Master.CORE_VOLTAGE_CHANGE_ALLOWED = false;

   return OK;
}

function main
(
)
{
   if( !Gpu.IsInitialized )
     return mods_js_main();
}

function doModsInitialize
(
)
{
   if( !Gpu.IsInitialized )
     return mods_js_main();  // needs renaming!
}

function DoGpuOverrides()
{
   // NV2A only runs in 640x480 mode.
   if (Gpu.NV2A == Gpu.DeviceId)
   {
      TestConfiguration.DisplayWidth  = 640;
      TestConfiguration.DisplayHeight = 480;
   }

   if (Gpu.NumSubdevices > 1)
   {
      TestConfiguration.PushBufferLocation = Memory.Fb;
   }

   return OK;
}

//------------------------------------------------------------------------------
// Quit
//------------------------------------------------------------------------------

function q()
{
   Exit();
   return OK;
}

//------------------------------------------------------------------------------
// Macro help utility.
//------------------------------------------------------------------------------

function h()
{
   for (var i in Macros)
   {
      Out.Printf(Out.ScreenOnly, "%s\n", Macros[i][0]);
   }

   return OK;
}

//------------------------------------------------------------------------------
// Syntactic sugar
//
// Some of us have a hard time remembering what language we are using...
//------------------------------------------------------------------------------

help = Help;
print = Print;
function gets() { return In.readln(); }

//------------------------------------------------------------------------------
// Print
//------------------------------------------------------------------------------

function Print
(
   // arguments
)
{
   for (var i = 0; i < arguments.length; ++i)
      Out.Print(Out.PriNormal, arguments[i]);

   Out.Print(Out.PriNormal, "\n");

   return OK;
}
function print()        // for the case-insensitive among us
{
   for (var i = 0; i < arguments.length; ++i)
      Out.Print(Out.PriNormal, arguments[i]);

   Out.Print(Out.PriNormal, "\n");

   return OK;
}



//------------------------------------------------------------------------------
// HPrint -- prints values in hexadecimal
//------------------------------------------------------------------------------

function HPrint
(
   // arguments
)
{
   for (var i = 0; i < arguments.length; ++i)
   {
      Out.Print(Out.PriNormal, "0x");
      Out.Print(Out.PriNormal, Out.ToHex(arguments[i]));
      Out.Print(Out.PriNormal, "      (");
      Out.Print(Out.PriNormal, arguments[i]);
      Out.Print(Out.PriNormal, ")\n");
   }


   return OK;
}

//------------------------------------------------------------------------------
// DumpData
//------------------------------------------------------------------------------

function DumpData
(
   ReadFunction,
   SizeOfData,
   StartAddress,
   NumReads
)
{
   var Format = "";
   switch (SizeOfData)
   {
      case 1:
         Format = "%02x ";
         break;

      case 2:
         Format = "%04x ";
         break;

      case 4:
         Format = "%08x ";
         break;

      default:
         Out.Printf(Out.PriNormal, "Invalid SizeOfData.\n");
         return RC.BAD_PARAMETER;
   }

   if (StartAddress < 0)
   {
      Out.Printf(Out.PriNormal, "Invalid StartAddress.\n");
      return RC.BAD_PARAMETER;
   }

   if (undefined == NumReads)
   {
      NumReads = 1;
   }
   else if (NumReads < 1)
   {
      Out.Printf(Out.PriNormal, "Invalid NumReads.\n");
      return RC.BAD_PARAMETER;
   }

   var EndAddress = StartAddress + NumReads*SizeOfData;
   data = new Array(16);

   var Address;
   var vertCurPos = 1;

   for
   (
          Address = StartAddress;
          Address < EndAddress;
          Address += SizeOfData
   )
   {
      // Print the address every 16 (0x10) bytes.
      if (((Address - StartAddress) % 16) == 0)
      {
         Out.Printf(Out.PriNormal, "%s%08x:  ",
            (Address == StartAddress) ? "" : "\n", Address);
         vertCurPos = 12;
      }

      Log.Next = false;
      data[(Address-StartAddress)%16] = ReadFunction(Address);
      Out.Printf(Out.PriNormal, Format, data[(Address-StartAddress)%16]);

      switch(SizeOfData)
      {
         case 1:
            vertCurPos += 3;
            break;
         case 2:
            vertCurPos += 5;
            break;
         case 4:
            vertCurPos += 9;
            break;
         default:
            break;
      }

      // Print the ASCII equivalents at the end of each row
      if (((Address - StartAddress) % 16) == 16-SizeOfData)
      {
         // align on the right side
         while(vertCurPos < 63)
         {
            Out.Printf(Out.PriNormal, " ");
            vertCurPos++;
         }

         for(var AsciiAdd = Address - 16 + SizeOfData; AsciiAdd <= Address; AsciiAdd += SizeOfData)
         {
         switch(SizeOfData)
            {
               case 1:
                  if (data[(AsciiAdd-StartAddress)%16]>31)
                     Out.Printf(Out.PriNormal, "%c", data[(AsciiAdd-StartAddress)%16]);
                  else
                     Out.Printf(Out.PriNormal, ".");
                  break;

               case 2:
                  if ((data[(AsciiAdd-StartAddress)%16] & 0xFF) > 31)
                     Out.Printf(Out.PriNormal, "%c", data[(AsciiAdd-StartAddress)%16] & 0xFF);
                  else
                     Out.Printf(Out.PriNormal, ".");

                  if (((data[(AsciiAdd-StartAddress)%16] >>> 8) & 0xFF) > 31)
                     Out.Printf(Out.PriNormal, "%c", (data[(AsciiAdd-StartAddress)%16] >>> 8) & 0xFF);
                  else
                     Out.Printf(Out.PriNormal, ".");
                  break;

               case 4:
                  if ((data[(AsciiAdd-StartAddress)%16] & 0xFF) > 31)
                     Out.Printf(Out.PriNormal, "%c", data[(AsciiAdd-StartAddress)%16] & 0xFF);
                  else
                     Out.Printf(Out.PriNormal, ".");

                  if (((data[(AsciiAdd-StartAddress)%16] >>> 8) & 0xFF) > 31)
                     Out.Printf(Out.PriNormal, "%c", (data[(AsciiAdd-StartAddress)%16] >>> 8) & 0xFF);
                  else
                     Out.Printf(Out.PriNormal, ".");

                  if (((data[(AsciiAdd-StartAddress)%16] >>> 16) & 0xFF) > 31)
                     Out.Printf(Out.PriNormal, "%c", (data[(AsciiAdd-StartAddress)%16] >>> 16) & 0xFF);
                  else
                     Out.Printf(Out.PriNormal, ".");

                  if (((data[(AsciiAdd-StartAddress)%16] >>> 24) & 0xFF) > 31)
                     Out.Printf(Out.PriNormal, "%c", (data[(AsciiAdd-StartAddress)%16] >>> 24) & 0xFF);
                  else
                     Out.Printf(Out.PriNormal, ".");
                  break;

               default:
                  Out.Printf(Out.PriNormal, "Invalid SizeOfData.\n");
                  return RC.BAD_PARAMETER;
            }
         }
      }
   }

   if (((Address - StartAddress) % 16) != 0)
   {
      // align on the right side
      while(vertCurPos < 63)
      {
         Out.Printf(Out.PriNormal, " ");
         vertCurPos++;
      }

      for(var AsciiAdd = Address - ((Address - StartAddress) % 16); AsciiAdd < EndAddress; AsciiAdd += SizeOfData)
      {
      switch(SizeOfData)
         {
            case 1:
               if (data[(AsciiAdd-StartAddress)%16]>31)
                  Out.Printf(Out.PriNormal, "%c", data[(AsciiAdd-StartAddress)%16]);
               else
                  Out.Printf(Out.PriNormal, ".");
               break;

            case 2:
               if ((data[(AsciiAdd-StartAddress)%16] & 0xFF) > 31)
                  Out.Printf(Out.PriNormal, "%c", data[(AsciiAdd-StartAddress)%16] & 0xFF);
               else
                  Out.Printf(Out.PriNormal, ".");

               if (((data[(AsciiAdd-StartAddress)%16] >>> 8) & 0xFF) > 31)
                  Out.Printf(Out.PriNormal, "%c", (data[(AsciiAdd-StartAddress)%16] >>> 8) & 0xFF);
               else
                  Out.Printf(Out.PriNormal, ".");
               break;

            case 4:
               if ((data[(AsciiAdd-StartAddress)%16] & 0xFF) > 31)
                  Out.Printf(Out.PriNormal, "%c", data[(AsciiAdd-StartAddress)%16] & 0xFF);
               else
                  Out.Printf(Out.PriNormal, ".");

               if (((data[(AsciiAdd-StartAddress)%16] >>> 8) & 0xFF) > 31)
                  Out.Printf(Out.PriNormal, "%c", (data[(AsciiAdd-StartAddress)%16] >>> 8) & 0xFF);
               else
                  Out.Printf(Out.PriNormal, ".");

               if (((data[(AsciiAdd-StartAddress)%16] >>> 16) & 0xFF) > 31)
                  Out.Printf(Out.PriNormal, "%c", (data[(AsciiAdd-StartAddress)%16] >>> 16) & 0xFF);
               else
                  Out.Printf(Out.PriNormal, ".");

               if (((data[(AsciiAdd-StartAddress)%16] >>> 24) & 0xFF) > 31)
                  Out.Printf(Out.PriNormal, "%c", (data[(AsciiAdd-StartAddress)%16] >>> 24) & 0xFF);
               else
                  Out.Printf(Out.PriNormal, ".");
               break;

            default:
               Out.Printf(Out.PriNormal, "Invalid SizeOfData.\n");
               return RC.BAD_PARAMETER;
         }
      }
   }

   Out.Printf(Out.PriNormal, "\n");

   return OK;
}

//------------------------------------------------------------------------------
// FillData
//------------------------------------------------------------------------------

function FillData
(
   WriteFunction,
   Data,
   SizeOfData,
   StartAddress,
   NumWrites
)
{
   if
   (
         (SizeOfData != 1)
      && (SizeOfData != 2)
      && (SizeOfData != 4)
   )
   {
      Out.Printf(Out.PriNormal, "Invalid SizeOfData.\n");
      return RC.BAD_PARAMETER;
   }

   if (StartAddress < 0)
   {
      Out.Printf(Out.PriNormal, "Invalid StartAddress.\n");
      return RC.BAD_PARAMETER;
   }

   if (undefined == NumWrites)
   {
      NumWrites = 1;
   }
   else if (NumWrites < 1)
   {
      Out.Printf(Out.PriNormal, "Invalid NumWrites.\n");
      return RC.BAD_PARAMETER;
   }

   var EndAddress = StartAddress + NumWrites*SizeOfData;

   for
   (
      var Address = StartAddress;
          Address < EndAddress;
          Address += SizeOfData
   )
   {
      Log.Next = false;
      WriteFunction(Address, Data);
   }

   return OK;
}

//------------------------------------------------------------------------------
// DumpPciData
//------------------------------------------------------------------------------

function DumpPciData
(
   ReadFunction,
   SizeOfData,
   Bus,
   Device,
   Function,
   StartAddress,
   NumReads
)
{
   var Format = "";
   switch (SizeOfData)
   {
      case 1:
         Format = "%02x ";
         break;

      case 2:
         Format = "%04x ";
         break;

      case 4:
         Format = "%08x ";
         break;

      default:
         Out.Printf(Out.PriNormal, "Invalid SizeOfData.\n");
         return RC.BAD_PARAMETER;
   }

   if (StartAddress < 0)
   {
      Out.Printf(Out.PriNormal, "Invalid StartAddress.\n");
      return RC.BAD_PARAMETER;
   }

   if (undefined == NumReads)
   {
      NumReads = 1;
   }
   else if (NumReads < 1)
   {
      Out.Printf(Out.PriNormal, "Invalid NumReads.\n");
      return RC.BAD_PARAMETER;
   }

   var EndAddress = StartAddress + NumReads*SizeOfData;

   for
   (
      var Address = StartAddress;
          Address < EndAddress;
          Address += SizeOfData
   )
   {
      // Print the address every 16 (0x10) bytes.
      if (((Address - StartAddress) % 16) == 0)
      {
         Out.Printf(Out.PriNormal, "%s%08x:  ",
            (Address == StartAddress) ? "" : "\n", Address);
      }

      Log.Next = false;
      Out.Printf(Out.PriNormal, Format,
         ReadFunction(Bus, Device, Function, Address));
   }

   Out.Printf(Out.PriNormal, "\n");

   return OK;
}

//------------------------------------------------------------------------------
// Get register and frame buffer base addresses.
//------------------------------------------------------------------------------

function GetNvBaseAddress()
{
   // Find a VGA device.
   var ReturnVals = new Array();
   Pci.FindClassCode(0x030000, 0, ReturnVals);

   var NvBaseAddress = IntToBits(Pci.Read32(
      ReturnVals[0], ReturnVals[1], ReturnVals[2], 0x10) & 0xFFF00000);

   Out.Printf(Out.ScreenOnly, "NV base address = %#010x\n", NvBaseAddress);

   return NvBaseAddress;
}
Log.Never("GetNvBaseAddress");

function GetFbBaseAddress()
{
   // Find a VGA device.
   var ReturnVals = new Array();
   Pci.FindClassCode(0x030000, 0, ReturnVals);

   var FbBaseAddress = IntToBits(Pci.Read32(
      ReturnVals[0], ReturnVals[1], ReturnVals[2], 0x14) & 0xFFF00000);

   Out.Printf(Out.ScreenOnly, "FB base address = %#010x\n", FbBaseAddress);

   return FbBaseAddress;
}
Log.Never("GetFbBaseAddress");

//------------------------------------------------------------------------------
// Register read/write functions.
//------------------------------------------------------------------------------

function RegRd08
(
   StartAddress,
   NumReads
)
{
   if (Gpu.IsInitialized)
   {
      DumpData(Gpu.RegRd08, 1, StartAddress, NumReads);
   }
   else
   {
      if (0 == g_NvBaseAddress)
         g_NvBaseAddress = GetNvBaseAddress();

      DumpData(Memory.Read08, 1, g_NvBaseAddress + StartAddress, NumReads);
   }

   return OK;
}

function RegRd16
(
   StartAddress,
   NumReads
)
{
   if (Gpu.IsInitialized)
   {
      DumpData(Gpu.RegRd16, 2, StartAddress, NumReads);
   }
   else
   {
      if (0 == g_NvBaseAddress)
         g_NvBaseAddress = GetNvBaseAddress();

      DumpData(Memory.Read16, 2, g_NvBaseAddress + StartAddress, NumReads);
   }

   return OK;
}

function RegRd32
(
   StartAddress,
   NumReads
)
{
   if (Gpu.IsInitialized)
   {
      DumpData(Gpu.RegRd32, 4, StartAddress, NumReads);
   }
   else
   {
      if (0 == g_NvBaseAddress)
         g_NvBaseAddress = GetNvBaseAddress();

      DumpData(Memory.Read32, 4, g_NvBaseAddress + StartAddress, NumReads);
   }

   return OK;
}

function RegWr08
(
   StartAddress,
   Data,
   NumWrites
)
{
   if (Gpu.IsInitialized)
   {
      FillData(Gpu.RegWr08, Data, 1, StartAddress, NumWrites);
   }
   else
   {
      if (0 == g_NvBaseAddress)
         g_NvBaseAddress = GetNvBaseAddress();

      FillData(Memory.Write08, Data, 1, g_NvBaseAddress + StartAddress,
         NumWrites);
   }

   return OK;
}

function RegWr16
(
   StartAddress,
   Data,
   NumWrites
)
{
   if (Gpu.IsInitialized)
   {
      FillData(Gpu.RegWr16, Data, 2, StartAddress, NumWrites);
   }
   else
   {
      if (0 == g_NvBaseAddress)
         g_NvBaseAddress = GetNvBaseAddress();

      FillData(Memory.Write16, Data, 2, g_NvBaseAddress + StartAddress,
         NumWrites);
   }

   return OK;
}

function RegWr32
(
   StartAddress,
   Data,
   NumWrites
)
{
   if (Gpu.IsInitialized)
   {
      FillData(Gpu.RegWr32, Data, 4, StartAddress, NumWrites);
   }
   else
   {
      if (0 == g_NvBaseAddress)
         g_NvBaseAddress = GetNvBaseAddress();

      FillData(Memory.Write32, Data, 4, g_NvBaseAddress + StartAddress,
         NumWrites);
   }

   return OK;
}

//------------------------------------------------------------------------------
// Frame buffer read/write functions.
//------------------------------------------------------------------------------

function FbRd08
(
   StartAddress,
   NumReads
)
{
   if (Gpu.IsInitialized)
   {
      DumpData(Gpu.FbRd08, 1, StartAddress, NumReads);
   }
   else
   {
      if (0 == g_FbBaseAddress)
         g_FbBaseAddress = GetFbBaseAddress();

      DumpData(Memory.Read08, 1, g_FbBaseAddress + StartAddress, NumReads);
   }

   return OK;
}

function FbRd16
(
   StartAddress,
   NumReads
)
{
   if (Gpu.IsInitialized)
   {
      DumpData(Gpu.FbRd16, 2, StartAddress, NumReads);
   }
   else
   {
      if (0 == g_FbBaseAddress)
         g_FbBaseAddress = GetFbBaseAddress();

      DumpData(Memory.Read16, 2, g_FbBaseAddress + StartAddress, NumReads);
   }

   return OK;
}

function FbRd32
(
   StartAddress,
   NumReads
)
{
   if (Gpu.IsInitialized)
   {
      DumpData(Gpu.FbRd32, 4, StartAddress, NumReads);
   }
   else
   {
      if (0 == g_FbBaseAddress)
         g_FbBaseAddress = GetFbBaseAddress();

      DumpData(Memory.Read32, 4, g_FbBaseAddress + StartAddress, NumReads);
   }

   return OK;
}

function FbWr08
(
   StartAddress,
   Data,
   NumWrites
)
{
   if (Gpu.IsInitialized)
   {
      FillData(Gpu.FbWr08, Data, 1, StartAddress, NumWrites);
   }
   else
   {
      if (0 == g_FbBaseAddress)
         g_FbBaseAddress = GetFbBaseAddress();

      FillData(Memory.Write08, Data, 1, g_FbBaseAddress + StartAddress,
         NumWrites);
   }

   return OK;
}

function FbWr16
(
   StartAddress,
   Data,
   NumWrites
)
{
   if (Gpu.IsInitialized)
   {
      FillData(Gpu.FbWr16, Data, 2, StartAddress, NumWrites);
   }
   else
   {
      if (0 == g_FbBaseAddress)
         g_FbBaseAddress = GetFbBaseAddress();

      FillData(Memory.Write16, Data, 2, g_FbBaseAddress + StartAddress,
         NumWrites);
   }

   return OK;
}

function FbWr32
(
   StartAddress,
   Data,
   NumWrites
)
{
   if (Gpu.IsInitialized)
   {
      FillData(Gpu.FbWr32, Data, 4, StartAddress, NumWrites);
   }
   else
   {
      if (0 == g_FbBaseAddress)
         g_FbBaseAddress = GetFbBaseAddress();

      FillData(Memory.Write32, Data, 4, g_FbBaseAddress + StartAddress,
         NumWrites);
   }

   return OK;
}

//------------------------------------------------------------------------------
// Memory read/write functions.
//------------------------------------------------------------------------------

function Rd08
(
   StartAddress,
   NumReads
)
{
   return DumpData(Memory.Read08, 1, StartAddress, NumReads);
}

function Rd16
(
   StartAddress,
   NumReads
)
{
   return DumpData(Memory.Read16, 2, StartAddress, NumReads);
}

function Rd32
(
   StartAddress,
   NumReads
)
{
   return DumpData(Memory.Read32, 4, StartAddress, NumReads);
}

function Wr08
(
   StartAddress,
   Data,
   NumWrites
)
{
   return FillData(Memory.Write08, Data, 1, StartAddress, NumWrites);
}

function Wr16
(
   StartAddress,
   Data,
   NumWrites
)
{
   return FillData(Memory.Write16, Data, 2, StartAddress, NumWrites);
}

function Wr32
(
   StartAddress,
   Data,
   NumWrites
)
{
   return FillData(Memory.Write32, Data, 4, StartAddress, NumWrites);
}

//------------------------------------------------------------------------------
// Port read/write functions.
//------------------------------------------------------------------------------

function Inp08
(
   StartAddress,
   NumReads
)
{
   return DumpData(Pio.Read08, 1, StartAddress, NumReads);
}

function Inp16
(
   StartAddress,
   NumReads
)
{
   return DumpData(Pio.Read16, 2, StartAddress, NumReads);
}

function Inp32
(
   StartAddress,
   NumReads
)
{
   return DumpData(Pio.Read32, 4, StartAddress, NumReads);
}

function Outp08
(
   StartAddress,
   Data,
   NumWrites
)
{
   return FillData(Pio.Write08, Data, 1, StartAddress, NumWrites);
}

function Outp16
(
   StartAddress,
   Data,
   NumWrites
)
{
   return FillData(Pio.Write16, Data, 2, StartAddress, NumWrites);
}

function Outp32
(
   StartAddress,
   Data,
   NumWrites
)
{
   return FillData(Pio.Write32, Data, 4, StartAddress, NumWrites);
}

//------------------------------------------------------------------------------
// Register read/write functions.
//------------------------------------------------------------------------------

function PciRd08
(
   Bus,
   Device,
   Function,
   StartAddress,
   NumReads
)
{
   return DumpPciData(Pci.Read08, 1, Bus, Device, Function,
                           StartAddress, NumReads);;

}

function PciRd16
(
   Bus,
   Device,
   Function,
   StartAddress,
   NumReads
)
{
   return DumpPciData(Pci.Read16, 2, Bus, Device, Function,
                           StartAddress, NumReads);;

}

function PciRd32
(
   Bus,
   Device,
   Function,
   StartAddress,
   NumReads
)
{
   return DumpPciData(Pci.Read32, 4, Bus, Device, Function,
                           StartAddress, NumReads);;

}

function PciWr08
(
   Bus,
   Device,
   Function,
   Address,
   Data
)
{
   Pci.Write08(Bus, Device, Function, Address, Data);

   Out.Printf(Out.PriNormal, "%08x <= %02x\n", Address, Data);

   return OK;
}

function PciWr16
(
   Bus,
   Device,
   Function,
   Address,
   Data
)
{
   Pci.Write16(Bus, Device, Function, Address, Data);

   Out.Printf(Out.PriNormal, "%08x <= %04x\n", Address, Data);

   return OK;
}

function PciWr32
(
   Bus,
   Device,
   Function,
   Address,
   Data
)
{
   Pci.Write32(Bus, Device, Function, Address, Data);

   Out.Printf(Out.PriNormal, "%08x <= %08x\n", Address, Data);

   return OK;
}

//------------------------------------------------------------------------------
// Set debug and normal level.
//------------------------------------------------------------------------------

function LevDebug
(
)
{
   Out.ScreenLevel = Out.LevDebug;
   Out.FileLevel   = Out.LevDebug;
   Out.SerialLevel = Out.LevDebug;

   return OK;
}

function LevVerbose
(
)
{
   Out.ScreenLevel = Out.LevLow;
   Out.FileLevel   = Out.LevLow;
   Out.SerialLevel = Out.LevLow;

   return OK;
}

function LevNormal
(
)
{
   Out.ScreenLevel = Out.LevNormal;
   Out.FileLevel   = Out.LevNormal;
   Out.SerialLevel = Out.LevNormal;

   return OK;
}

function LevTerse
(
)
{
   Out.ScreenLevel = Out.LevHigh;
   Out.FileLevel   = Out.LevHigh;
   Out.SerialLevel = Out.LevHigh;

   return OK;
}

//------------------------------------------------------------------------------
// Execute NV script.
//------------------------------------------------------------------------------

function ExecuteNvScript
(
   Script
)
{
   var ScriptFile = new File(Script);

   ScriptFile.open();

   if (!ScriptFile.isOpen)
   {
      Out.Printf(Out.PriHigh, "Can not open %s.\n", Script);
      return RC.FILE_DOES_NOT_EXIST;
   }

   var Line = new String();

   ReadFile:
   while (null != (Line = ScriptFile.readln()))
   {
      var Command = Line.split(/\s+/);

      Out.Printf(Out.PriNormal, "NV: %s\n", Line);

      // All the commands arguments are in hexadecimal.
      switch (Command[0])
      {
         case "db":
         {
            // db offset l length
            if (2 == Command.length)
            {
               RegRd08(parseInt(Command[1], 16));
            }
            else
            {
               if (Command[2] != "l")
                  return RC.BAD_PARAMETER;

               RegRd08(parseInt(Command[1], 16),
                  parseInt(Command[3], 16) / 1);
            }

            break;
         }

         case "dw":
         {
            // dw offset l length
            if (2 == Command.length)
            {
               RegRd16(parseInt(Command[1], 16));
            }
            else
            {
               if (Command[2] != "l")
                  return RC.BAD_PARAMETER;

               RegRd16(parseInt(Command[1], 16),
                  parseInt(Command[3], 16) / 2);
            }

            break;
         }

         case "dd":
         {
            // dd offset l length
            if (2 == Command.length)
            {
               RegRd32(parseInt(Command[1], 16));
            }
            else
            {
               if (Command[2] != "l")
                  return RC.BAD_PARAMETER;

               RegRd32(parseInt(Command[1], 16),
                  parseInt(Command[3], 16) / 4);
            }

            break;
         }

         case "eb":
         {
            // eb offset value
            if (Command.length != 3)
               return RC.BAD_PARAMETER;

            RegWr08(parseInt(Command[1], 16), parseInt(Command[2], 16));

            break;
         }

         case "ew":
         {
            // ew offset value
            if (Command.length != 3)
               return RC.BAD_PARAMETER;

            RegWr16(parseInt(Command[1], 16), parseInt(Command[2], 16));

            break;
         }

         case "ed":
         {
            // ed offset value
            if (Command.length != 3)
               return RC.BAD_PARAMETER;

            RegWr32(parseInt(Command[1], 16), parseInt(Command[2], 16));

            break;
         }

         case "fb":
         {
            // fb offset l length value
            if
            (
                  (Command.length != 5)
               || (Command[2] != "l")
            )
            {
               return RC.BAD_PARAMETER;
            }

            RegWr08(parseInt(Command[1], 16), parseInt(Command[4], 16),
               parseInt(Command[3], 16) / 1);

            break;
         }

         case "fw":
         {
            // fw offset l length value
            if
            (
                  (Command.length != 5)
               || (Command[2] != "l")
            )
            {
               return RC.BAD_PARAMETER;
            }

            RegWr16(parseInt(Command[1], 16), parseInt(Command[4], 16),
               parseInt(Command[3], 16) / 2);

            break;
         }

         case "fd":
         {
            // fd offset l length value
            if
            (
                  (Command.length != 5)
               || (Command[2] != "l")
            )
            {
               return RC.BAD_PARAMETER;
            }

            RegWr32(parseInt(Command[1], 16), parseInt(Command[4], 16),
               parseInt(Command[3], 16) / 4);

            break;
         }

         case "q":
         {
            break ReadFile;
         }

         default:
         {
            Out.Printf(Out.PriHigh, "Invalid NV input: %s\n", Line);
            return RC.BAD_PARAMETER;
         }

      } // end of switch

   } // read file

}

//------------------------------------------------------------------------------
// Get tests.
//------------------------------------------------------------------------------

Log.Never("GetTests");

function GetTests
(
   FileName
)
{
   // Read the input file.

   var InFile = new File(FileName);
   InFile.open("text", "read");
   if (!InFile.isOpen)
   {
      Out.Printf(Out.ScreenOnly, "Can not open %s.\n", FileName)
      return new Array();
   }

   var Lines = InFile.readAll();

   InFile.close();

   // Find the tests.
   // Lines the begin with a # (optionally preceded by whitespace) are ignored.

   var Tests = new Array();
   for (i in Lines)
   {
      // Skip blank lines.
      if (Lines[i].search(/^\s*$/) != -1)
         continue;

      // Skip comment lines.
      if (Lines[i].search(/^\s*#/) != -1)
         continue;

      // Test found.
      Tests.push(Lines[i]);
   }

   return Tests;
}

//------------------------------------------------------------------------------
// Step through MEMORY and GRAPHICS clocks.
//------------------------------------------------------------------------------

function StepClocks
(
   MemoryClockStart,
   MemoryClockEnd,
   MemoryClockStep,
   GraphicsClockStart,
   GraphicsClockEnd,
   GraphicsClockStep,
   Loops,
   FileName
)
{
   // Check that all arguments were supplied.

   if (arguments.length != 8)
   {
      Out.Printf(Out.PriNormal,
         "Usage: StepClocks(memory clock start, end, step, "
         + "graphics clock start, end, step, loops, test file)\n");

      return RC.BAD_PARAMETER;
   }

   if (0 == MemoryClockStep)
   {
      MemoryClockEnd  = MemoryClockStart;
      MemoryClockStep = 1;
   }

   if (0 == GraphicsClockStep)
   {
      GraphicsClockEnd  = GraphicsClockStart;
      GraphicsClockStep = 1;
   }

   // Store the current memory and graphics clocks so we can restore it on exit.
   var CurrentMemoryClock   = Dac.MEMORY_CLOCK;
   var CurrentGraphicsClock = Dac.GRAPHICS_CLOCK;

   var LogFileName = Out.FileName;

   // Loop over MEMORY and GRAPHICS clocks.

   var Tests   = GetTests(FileName);
   var Results = new Array();

   for
   (
      var MemoryClock = MemoryClockStart;
      MemoryClock    <= MemoryClockEnd;
      MemoryClock    += MemoryClockStep
   )
   {
      Dac.MEMORY_CLOCK = MemoryClock * 1e6;

      var GraphicClockResults = new Array();

      // Print result header for current memory clock and graphics clock range.
      Out.Printf(Out.PriNormal, "MEMORY_CLOCK   = %d\n", MemoryClock);
      Out.Printf(Out.PriNormal, "GRAPHICS_CLOCK = ");
      for
      (
         i  = GraphicsClockStart;
         i <= GraphicsClockEnd;
         i += GraphicsClockStep
      )
      {
         Out.Printf(Out.PriNormal, "%5.1f ", i);
      }
      Out.Printf(Out.PriNormal, "\n                 ");

      // Write the log file to disk in case the system hangs.
      Out.Close();
      Out.Open(LogFileName, Out.Append);

      for
      (
         var GraphicsClock = GraphicsClockStart;
         GraphicsClock    <= GraphicsClockEnd;
         GraphicsClock    += GraphicsClockStep
      )
      {
         Dac.GRAPHICS_CLOCK = GraphicsClock * 1e6;

         var Fails = 0;

         for (i in Tests)
         {
            for (var j = Loops; j > 0; --j)
            {
               var Result = System(Tests[i]);
               Out.Printf(Out.ScreenOnly, "%s => %d\n", Tests[i], Result);
               if (Result != 0)
               {
                  ++Fails;
               }
            }
         }

         GraphicClockResults.push(Fails);

         Out.Printf(Out.PriNormal, "%5d ", Fails);

         // Write the log file to disk in case the system hangs.
         Out.Close();
         Out.Open(LogFileName, Out.Append);

      } // for each GRAPHICS clock

      Out.Printf(Out.PriNormal, "\n\n");

      // Set the graphics clock back to starting value.
      Dac.GRAPHICS_CLOCK = GraphicsClockStart * 1e6;

      // Save results.
      Results.push(GraphicClockResults);

   } // for each MEMORY clock

   // Print all the results.

   Out.Printf(Out.PriNormal, "\nMEMORY_CLOCK     GRAPHICS_CLOCK");
   Out.Printf(Out.PriNormal, "\n      ");
   for
   (
      i  = GraphicsClockStart;
      i <= GraphicsClockEnd;
      i += GraphicsClockStep
   )
   {
      Out.Printf(Out.PriNormal, "%5.1f ", i);
   }
   Out.Printf(Out.PriNormal, "\n");

   for
   (
      i  = MemoryClockStart, I = 0;
      i <= MemoryClockEnd;
      i += MemoryClockStep, ++I
   )
   {
      Out.Printf(Out.PriNormal, "%-6.1f", i);

      for
      (
         j  = GraphicsClockStart, J = 0;
         j <= GraphicsClockEnd;
         j += GraphicsClockStep, ++J
      )
      {
         Out.Printf(Out.PriNormal, "%5d ", Results[I][J]);
      }
      Out.Printf(Out.PriNormal, "\n");
   }

   // Write the log file to disk in case the system hangs.
   Out.Close();
   Out.Open(LogFileName, Out.Append);

   // Restore the memory and graphics clocks.
   Dac.MEMORY_CLOCK   = CurrentMemoryClock;
   Dac.GRAPHICS_CLOCK = CurrentGraphicsClock;

   return OK;

} // StepClocks()

//------------------------------------------------------------------------------
// Set register field.
//------------------------------------------------------------------------------

function SetField
(
   Register,
   Field,
   Num
)
{
   var Data = Gpu.RegRd32(Register);

   var Extent = Number(Field.split(":")[0]);
   var Base   = Number(Field.split(":")[1]);

   var Mask = 0xFFFFFFFF >>> (31 - Extent + Base);

   // Zero out the field.
   Data &= ~(Mask << Base);

   // Set the field.
   Gpu.RegWr32(Register, Data | ((Num & Mask) << Base));

   return OK;
}

//------------------------------------------------------------------------------
// Step through register fields.
//------------------------------------------------------------------------------

function StepFields
(
   Register,
   Start,
   End,
   Step,
   Loops,
   FileName
// Fields ...
)
{
   // Check the arguments.

   if (arguments.length < 7)
   {
      Out.Printf(Out.PriNormal,
         "Usage: StepFields(register, start, end, step, file name, loops, "
         + "fields ...)\n");

      return RC.BAD_PARAMETER;
   }

   if (0 == Step)
   {
      End  = Start;
      Step = 1;
   }

   // Loop over the fields.

   var Tests   = GetTests(FileName);
   var Results = new Array();

   for (var Num = Start; Num <= End; Num += Step)
   {
      for (var Field = 6; Field < arguments.length; ++Field)
      {
         SetField(Register, arguments[ Field ], Num);
      }

      var Fails = 0;

      for (var i in Tests)
      {
         for (var j = Loops; j > 0; --j)
         {
            if (System(Tests[i]) != 0)
               ++Fails;
         }
      }

      Results[ Num ] = Fails;

   }

   // Print results.

   Out.Printf(Out.PriNormal, "Register = %#010x\n", Register);
   for (var Num = Start; Num <= End; Num += Step)
   {
      Out.Printf(Out.PriNormal, "NUM = %#010x Fails = %d\n",
         Num, Results[ Num ]);
   }

   return OK;

} // StepFields()

//------------------------------------------------------------------------------
// NV like functions.
//------------------------------------------------------------------------------

function rr
(
   Address
)
{
   if (arguments.length != 1)
   {
      Print("Invalid arguments supplied to rr.");
      return OK;
   }

   RegRd32(Address, 64);

   return OK;
}

function rw
(
   Address,
   Data
)
{
   if (arguments.length != 2)
   {
      Print("Invalid arguments supplied to rw.");
      return OK;
   }

   RegWr32(Address, Data);
   RegRd32(Address);

   return OK;
}

function fr
(
   Address
)
{
   if (arguments.length != 1)
   {
      Print("Invalid arguments supplied to fr.");
      return OK;
   }

   FbRd32(Address);

   return OK;
}

function fw
(
   Address,
   Data,
   Words
)
{
   if (arguments.length == 2)
   {
      FbWr32(Address, Data);
      FbRd32(Address);
   }
   else if (arguments.length == 3)
   {
      Memory.Fill32(Bus.FbBaseAddress + Address, Data, Words);
   }
   else
   {
      Print("Invalid arguments supplied to fr.");
   }

   return OK;
}

function i
(
   Address
)
{
   if (arguments.length != 1)
   {
      Print("Invalid argument supplied to i.");
      return OK;
   }

   Inp32(Address);

   return OK;
}

function o
(
   Address,
   Data
)
{
   if (arguments.length != 2)
   {
      Print("Invalid argument supplied to o.");
      return OK;
   }

   Outp32(Address, Data);
   Inp32(Address);

   return OK;
}

function cr
(
   Address
)
{
   if (arguments.length != 1)
   {
      Print("Invalid argument supplied to cr.");
      return OK;
   }

   Outp32(0xcf8, Address);
   Inp32(0xcfc);

   return OK;
}

function cw
(
   Address,
   Data
)
{
   if (arguments.length != 2)
   {
      Print("Invalid argument supplied to cw.");
      return OK;
   }

   Outp32(0xcf8, Address);
   Outp32(0xcfc, Data);
   Inp32(0xcfc);

   return OK;
}


function clks()
{
   Out.Printf(Out.PriNormal, "nvclk  = %7.3f MHz\n", Dac.GRAPHICS_CLOCK / 1e6);
   Out.Printf(Out.PriNormal, "mclk   = %7.3f MHz\n", Dac.MEMORY_CLOCK   / 1e6);

   for (var i = 0; i < 24; ++i)
   {
      if (Display.Selected & (1 << i))
      {
         var Head = new Array;
         Display.GetHead(1 << i, Head);
         Out.Printf(Out.PriNormal, "pclk %d = %7.3f MHz\n", Head[0], Dac.GetPixelClock(1 << i) / 1e6);
      }
   }

   return OK;
}

function mclk
(
   FrequencyMHz
)
{
   if (1 == arguments.length)
      Dac.MEMORY_CLOCK = FrequencyMHz * 1e6;

   Out.Printf(Out.PriNormal, "mclk = %7.3f MHz\n", Dac.MEMORY_CLOCK / 1e6);

   return OK;
}

function nvclk
(
   FrequencyMHz
)
{
   if (1 == arguments.length)
      Dac.GRAPHICS_CLOCK = FrequencyMHz * 1e6;

   Out.Printf(Out.PriNormal, "nvclk = %7.3f MHz\n", Dac.GRAPHICS_CLOCK / 1e6);

   return OK;
}
var gclk = nvclk;

function pclk
(
   disp,
   FrequencyMHz
)
{
   if (2 == arguments.length)
      Dac.SetPixelClock(disp, FrequencyMHz * 1e6);

   for (var i = 0; i < 24; ++i)
   {
      if (Display.Selected & (1 << i))
      {
         var Head = new Array;
         Display.GetHead(1 << i, Head);
         Out.Printf(Out.PriNormal, "pclk %d = %7.3f MHz\n", Head[0], Dac.GetPixelClock(1 << i) / 1e6);
      }
   }

   return OK;
}


function straps(arg)
{
   // figure out which NV chip this is
   var NV_CFG_PCI_ID = 13;
   var r = new Array();
   if ( NvRm.ConfigGet(NV_CFG_PCI_ID,r)) return( OK );
   var NvDevice = r[0] >> 20;

   var StrapsReg;
   if (arguments.length != 1)
   {
      StrapsReg = Gpu.RegRd32(0x101000);
   }
   else if (arguments.length == 1)
   {
       StrapsReg = arg;
   }

   var PciDir    = ((StrapsReg >>>  0) & 0x1)        ? "NORMAL"   : "REVERSED";
   var Bios      = ((StrapsReg >>>  1) & 0x1)        ? "PRESENT"  : "NOT PRESENT";
   var MemType   = ((StrapsReg >>>  2) & 0xF);
   var Crystal   = ((StrapsReg >>>  6) & 0x1)        ? "14.31818" : "13.50000";

   if(NvDevice >= 0x17)
   {
      var TvMode = ((StrapsReg >>>  7) & 0x3)==0     ? "SECAM"    :
                   ((StrapsReg >>>  7) & 0x3)==1     ? "NTSC"     :
                   ((StrapsReg >>>  7) & 0x3)==2     ? "PAL"      : "CRT/DISABLED";
   }
   else
   {
      var TvMode = ((StrapsReg >>>  7) & 0x3)==0     ? "PAL"      :
                   ((StrapsReg >>>  7) & 0x3)==1     ? "NTSC"     :
                   ((StrapsReg >>>  7) & 0x3)==2     ? "RESERVED" : "DISABLED";
   }

   var Agp8x     = ((StrapsReg >>>  9) & 0x1);
   var Sideband  = ((StrapsReg >>> 10) & 0x1);

   var FastWrite = ((StrapsReg >>> 11) & 0x1)        ? "DISABLED" : "ENABLED";
   var DeviceId  = ((StrapsReg >>> 12) & 0x3);
   var HostIntf  = ((StrapsReg >>> 14) & 0x1)        ? "AGP"      : "PCI";
   var FlatPanel = ((StrapsReg >>> 15) & 0x1)        ? "12-BIT"   : "24-BIT";
   var Override  = ((StrapsReg >>> 31) & 0x1)        ? "ENABLED"  : "DISABLED";


   var Nv17FbBar     = ((StrapsReg >>> 23) & 0x3)==0 ? "64 MB"    :
                       ((StrapsReg >>> 23) & 0x3)==1 ? "128 MB"   :
                       ((StrapsReg >>> 23) & 0x3)==2 ? "256 MB"   : "512 MB";
   var Nv17Bridge    = ((StrapsReg >>> 25) & 0x1)    ? "ENABLED"  : "DISABLED";
   var Nv17Bridge128M= ((StrapsReg >>> 26) & 0x1)    ? "ENABLED"  : "DISABLED";
   var Nv17BridgeAgp = ((StrapsReg >>> 27) & 0x1)    ? "ENABLED"  : "DISABLED";
   var Nv17BridgeIo  = ((StrapsReg >>> 28) & 0x1)    ? "ENABLED"  : "DISABLED";
   var Nv17RomType   = ((StrapsReg >>> 29) & 0x3)==0 ? "PARALLEL" :
                       ((StrapsReg >>> 29) & 0x3)==1 ? "SERIAL_AT25F"   :
                       ((StrapsReg >>> 29) & 0x3)==2 ? "SERIAL_SST45VF" : "SERIAL_FUTURE";
   var Nv17User      = ((StrapsReg >>> 16) & 0xF);

   Out.Printf(Out.PriNormal,"Straps Register 0x101000 = %#08x\n\n",StrapsReg);

   Out.Printf(Out.PriNormal,"[0]            PCI_AD bus is %s\n",PciDir);
   Out.Printf(Out.PriNormal,"[1]            BIOS is %s on card\n",Bios);
   Out.Printf(Out.PriNormal,"[5:2]          Memory Type is %x\n",MemType);
   Out.Printf(Out.PriNormal,"[8:7]          TV Mode is %s\n",TvMode);

   if ((NvDevice > 0x28) || (NvDevice == 0x18))
   {
      Out.Printf(Out.PriNormal,"[9]            AGP rate is %s on on AGP 2.0 systems\n",Agp8x ? "2X" : "4X");
      Out.Printf(Out.PriNormal,"                           %s on on AGP 3.0 systems\n",Agp8x ? "4X" : "8X");

      if(Sideband)
      {
         Out.Printf(Out.PriNormal,"[10]           Sideband addressing is DISABLED on AGP 2.0 systems\n");
         Out.Printf(Out.PriNormal,"                                      ENABLED  on AGP 3.0 systems\n");
      }
      else
      {
         Out.Printf(Out.PriNormal,"[10]           Sideband addressing is ENABLED on AGP 2.0 systems\n");
         Out.Printf(Out.PriNormal,"                                      ENABLED on AGP 3.0 systems\n");
      }
   }
   else
   {
      Out.Printf(Out.PriNormal,"[9]            AGP 4X is %s\n",Agp8x ? "DISABLED" : "ENABLED");
      Out.Printf(Out.PriNormal,"[10]           SBA is %s\n",Sideband ? "DISABLED" : "ENABLED");
   }
   Out.Printf(Out.PriNormal,"[11]           Fast Writes are %s\n",FastWrite);
   Out.Printf(Out.PriNormal,"[14]           Host Interface is %s\n",HostIntf);
   Out.Printf(Out.PriNormal,"[15]           Flat Panel Interface is %s\n",FlatPanel);

   if(NvDevice >= 0x17)
   {
      // the straps below are only defined on NV17 and up

      DeviceId |= ((StrapsReg >>> 18) & 0xC);

      if((StrapsReg >>> 22) & 0x1) Crystal =  "27.0000";

      Out.Printf(Out.PriNormal,"[19:16]        User straps are %01x\n",Nv17User);
      Out.Printf(Out.PriNormal,"[21:20, 13:12] Device ID is %x (X+%x)\n", Gpu.PciDeviceId, DeviceId);
      Out.Printf(Out.PriNormal,"[22, 6]        Crystal Reference is %s\n",Crystal);
      Out.Printf(Out.PriNormal,"[24:23]        Framebuffer BAR is %s\n",Nv17FbBar);
      Out.Printf(Out.PriNormal,"[25]           Bridge is %s\n",Nv17Bridge);
      Out.Printf(Out.PriNormal,"[26]           Bridge 128M is %s\n",Nv17Bridge128M);
      Out.Printf(Out.PriNormal,"[27]           Bridge AGP is %s\n",Nv17BridgeAgp);
      Out.Printf(Out.PriNormal,"[28]           Bridge I/O is %s\n",Nv17BridgeIo);
   }
   else
   {
      Out.Printf(Out.PriNormal,"[13:12]        Device ID is X+%x\n",DeviceId);
      Out.Printf(Out.PriNormal,"[6]            Crystal Reference is %s\n",Crystal);
   }

   Out.Printf(Out.PriNormal,"[31]           Strap Override is %s\n",Override);

   return OK;
}

function screenon()
{
   Vga.SCREEN_OFF = false;

   return OK;
}

function screenoff()
{
   Vga.SCREEN_OFF = true;

   return OK;
}

function mono()
{
   Vga.MONOCHROME = true;

   return OK;
}

function vga()
{
   Vga.MONOCHROME = false;

   return OK;
}

//------------------------------------------------------------------------------
// DmaPush
//------------------------------------------------------------------------------

var IsMemfmtJsImported = false;

function DmaPush
(
   Words,
   Loops,
   Pause
)
{
   if (!IsMemfmtJsImported)
   {
      Import("memfmt.js");
      IsMemfmtJsImported = true;
   }

   DisableUserInterface(640, 480, 32, 60);
   screenoff();
   var rc = MemoryToMemoryFormat(Words, Loops);
   screenon();

   // Pause for user input.
   if (Pause)
      GetChar();

   EnableUserInterface();

   return rc;
}

//------------------------------------------------------------------------------
// Render a rectangle.
//------------------------------------------------------------------------------

var IsRectJsImported = false;

function Render
(
   Loops,
   IsDmaChannel
)
{
   if (!IsRectJsImported)
   {
      Import("rect.js");
      IsRectJsImported = true;
   }

   return SolidRectangleTest(Loops, IsDmaChannel);
}

//------------------------------------------------------------------------------
// DumpRegisters
//------------------------------------------------------------------------------

function DumpRegisters()
{
   //      offset       words
   RegRd32(0x00000000,  1*1024);    // PMC
   RegRd32(0x00001000,  1*1024);    // PBUS
   RegRd32(0x00002000,  2*1024);    // PFIFO
   RegRd32(0x00005000,  1*1024);    // HDIAG
   RegRd32(0x00006000,  1*1024);    // PRAM
   RegRd32(0x00007000,  1*1024);    // PRMIO
   RegRd32(0x00008000,  1*1024);    // PVIDEO
   RegRd32(0x00009000,  1*1024);    // PTIMER
   RegRd32(0x0000A000,  1*1024);    // PPM
   RegRd32(0x000A0000, 32*1024);    // PRMVGA
   RegRd32(0x000C0000,  1*1024);    // PRMVIO
   RegRd32(0x000C0000,  1*1024);    // PRMVIO
   RegRd32(0x00100000,  1*1024);    // PFB
   RegRd32(0x00200000,  1*1024);    // PME
   RegRd32(0x00300000, 16*1024);    // PROM
   RegRd32(0x00400000,  2*1024);    // PGRAPH
// RegRd32(0x00500000,  ?*1024);    // Reserved
   RegRd32(0x00600000,  1*1024);    // PCRTC
   RegRd32(0x00601000,  1*1024);    // PRMCIO
   RegRd32(0x00680000,  1*1024);    // PDAC
   RegRd32(0x00681000,  1*1024);    // PRMDIO

   return OK;
}

//------------------------------------------------------------------------------
// LoopTest
//
function LoopTest
(
   Test
)
{
   while (1)
   {
      var Result = System(Test);
      Out.Printf(Out.PriNormal, "%s: %s: %s\n",
         Test, ((Result == 0) ? "PASS" : "FAIL"), Date())
   }

   return OK;
}
Log.Always("LoopTest");

//------------------------------------------------------------------------------
// Fill screen with pattern.
//
function FillScreen( pattern, seed )
{
   if( "undefined" == typeof(pattern) )
   {
      Out.Printf(Out.PriHigh, "Useage: FillScreen(pattern, <seed> ) \n");
      Out.Printf(Out.PriHigh, "  fill screen with pattern, where pattern is one of:\n");
      Out.Printf(Out.PriHigh, "  \"rgb\" \"random\" \"stripey\" \"address\" or an integer constant color\n");
      Out.Printf(Out.PriHigh, "Try first calling Display.SetMode(w, h, d, refresh-rate) if you get bad results.\n");

      return 0;
   }

   var rv = new Array;
   Display.GetImageOffset(Display.Selected, rv);
   addr = Bus.FbBaseAddress + rv[0];

   Display.GetMode(Display.Selected, rv);
   w = rv[0];
   h = rv[1];
   d = rv[2];

   Display.GetPitch(Display.Selected, rv);
   p = rv[0];

   if( pattern == "rgb" )
   {
      Memory.FillRgb(addr, w, h, d, p);
   }
   else if( pattern == "random" )
   {
      Memory.FillRandom(addr, seed, 0, 0xffffffff, w, h, d, p);
   }
   else if( pattern == "stripey" )
   {
      Memory.FillStripey(addr, w, h, d, p);
   }
   else if( pattern == "address" )
   {
      Memory.FillAddress(addr, w, h, d, p);
   }
   else
   {
      Memory.FillConstant(addr, pattern, w, h, d, p);
   }

   return OK;
}

//------------------------------------------------------------------------------
// Fill screen with pattern.
//
function LoadYuv( filename, doSwapBytes )
{
   if( "undefined" == typeof(filename) )
   {
      Out.Printf(Out.PriHigh, "Useage: LoadYuv(filename) \n");
      Out.Printf(Out.PriHigh, "  fill screen with .yuv file image\n");
      Out.Printf(Out.PriHigh, "Try first calling Display.SetMode(w, h, d, refresh-rate) if you get bad results.\n");

      return 0;
   }
   if( "undefined" == typeof(doSwapBytes) )
      doSwapBytes = false;

   var rv = new Array;
   Display.GetImageOffset(Display.Selected, rv);
   addr = Bus.FbBaseAddress + rv[0];

   Display.GetMode(Display.Selected, rv);
   w = rv[0];
   h = rv[1];
   d = rv[2];

   Display.GetPitch(Display.Selected, rv);
   p = rv[0];

   return Memory.ReadYuv(filename, doSwapBytes, addr, w, h, p);
}

//------------------------------------------------------------------------------
// Fill screen with pattern.
//
function LoadTga( filename )
{
   if( "undefined" == typeof(filename) )
   {
      Out.Printf(Out.PriHigh, "Useage: LoadTga(filename) \n");
      Out.Printf(Out.PriHigh, "  fill screen with .tga file image\n");
      Out.Printf(Out.PriHigh, "Try first calling Display.SetMode(w, h, d, refresh-rate) if you get bad results.\n");

      return 0;
   }

   var rv = new Array;
   Display.GetImageOffset(Display.Selected, rv);
   addr = Bus.FbBaseAddress + rv[0];

   Display.GetMode(Display.Selected, rv);
   w = rv[0];
   h = rv[1];
   d = rv[2];

   Display.GetPitch(Display.Selected, rv);
   p = rv[0];

   return Memory.ReadTga(filename, addr, w, h, d, p);
}
//------------------------------------------------------------------------------
// Dump screen to file.
//
function DumpTga( filename, atorgb )
{
   if( "undefined" == typeof(filename) )
   {
      Out.Printf(Out.PriHigh, "Useage: DumpTga(filename, AtoRGB) \n");
      Out.Printf(Out.PriHigh, "  copy screen to .tga file image\n");

      return 0;
   }

   var rv = new Array;
   Display.GetImageOffset(Display.Selected, rv);
   addr = Bus.FbBaseAddress + rv[0];

   Display.GetMode(Display.Selected, rv);
   w = rv[0];
   h = rv[1];
   d = rv[2];

   Display.GetPitch(Display.Selected, rv);
   p = rv[0];

   return Memory.WriteTga(filename, atorgb, addr, w, h, d, p);
}

//------------------------------------------------------------------------------
// Compare 2 tga files, or one tga with the current screen display.
//
function CompareTga( w, h, d, tga1, tga2 )
{
   if( "undefined" == typeof(tga1) )
   {
      Out.Printf(Out.PriHigh, "Useage: CompareTga(w,h,d, tga1, tga2) \n" +
                              "  Load tga1 at 0, tga2 at pitch*height and toggle between them.\n" +
                              "  or, if tga2 is not specified, load tga1 high and toggle with current screen.\n" +
                              "  Press any key to flip images, q to quit.\n" );
      return 0;
   }

   // Get FB memory layout.
   rc = DisableUserInterface(w, h, d, 75);
   if (0 != rc)
   {
      EnableUserInterface();
      return rc;
   }

   var rv = new Array;
   Display.GetPitch(Display.Selected, rv);
   p = rv[0];

   Display.GetImageOffset(Display.Selected, rv);
   var offset1 = rv[0];
   addr1 = Bus.FbBaseAddress + offset1;

   Display.GetMode(Display.Selected, rv);
   w = rv[0];
   h = rv[1];
   d = rv[2];

   var offset2 = offset1 + h*p;
   addr2 = Bus.FbBaseAddress + offset2;

   if( "undefined" == typeof(tga2) )
   {
      tga2 = tga1;
      tga1 = "cur FB image";
   }
   else
   {
      // Load tga1 to current FB offset.

      rc = Memory.ReadTga(tga1, addr1, w, h, d, p);
      if (0 != rc)
      {
         EnableUserInterface();
         Out.Printf(Out.PriHigh, "Error loading file \"%s\"\n.", tga1);
         return rc;
      }
   }

   // Load tga2 to current FB offset + height*pitch.

   rc = Memory.ReadTga(tga2, addr2, w, h, d, p);
   if (0 != rc)
   {
      EnableUserInterface();
      Out.Printf(Out.PriHigh, "Error loading file \"%s\"\n.", tga2);
      return rc;
   }

   var nerrs = Memory.CompareImages(offset1, offset2, w, h, d, p, 10 /*max errs to print*/);
   Out.Printf(Out.PriHigh, "CompareTga found %d errors between \"%s\" and \"%s\"\n",
      nerrs, tga1, tga2);


   var flop = 0;
   key = " ";
   while( ("undefined" != typeof(key)) && ("q" != key) )
   {
      key = GetChar();

      if( flop )
         Display.SetImageOffset(addr2);
      else
         Display.SetImageOffset(addr1);

      flop = ~flop;

      Out.Printf(Out.ScreenOnly, "\rTga%d", 1 + (0 != flop));
   }
   Display.SetImageOffset(addr1);
   EnableUserInterface();
   return 0;
}

