/*
 * NVIDIA_COPYRIGHT_BEGIN
 *
 * Copyright 1999-2002 by NVIDIA Corporation.  All rights reserved.  All
 * information contained herein is proprietary and confidential to NVIDIA
 * Corporation.  Any use, reproduction, or disclosure without the written
 * permission of NVIDIA Corporation is prohibited.
 *
 * NVIDIA_COPYRIGHT_END
 */

//------------------------------------------------------------------------------
var g_ProgtvId = "$Id: //sw/nvr40/diag/mods/js/progtv.js#9 $";

#ifndef INCLUDED_MODS_H
#include "mods.h"
#endif

// The I2C values in here don't make much sense unless you also look at the
// various Philips datasheets.
// The Philips decoder datasheets are on the Nvidia network at:
//   \\syseng-sc-1\Archive\datasheets\Video_Devices\Video_Decoders\Philips

//------------------------------------------------------------------------------
// Code for programming RF receivers
//------------------------------------------------------------------------------
function SetRfFrequency( Frequency , Channel, ProgramPal, ProgramNtsc )
{
   if(("MICROTUNE_MT2050" == TvTuner) || IsNv35Tuner())
      MicrotuneSetRfFrequency( Frequency , Channel, ProgramPal, ProgramNtsc );
   else
      PhilipsSetRfFrequency( Frequency , Channel, ProgramPal, ProgramNtsc );
}

//------------------------------------------------------------------------------
// Code for programming the Microtune RF receiver
// This code stolen from \\sw\main\drivers\wdm\nvtunep\...
//------------------------------------------------------------------------------
#define PART_REV     0x00  //  0x00: Part/Rev Code
#define LO1C_1       0x01  //  0x01: LO1C Byte 1
#define LO1C_2       0x02  //  0x02: LO1C Byte 2
#define LO2C_1       0x03  //  0x03: LO2C Byte 1
#define LO2C_2       0x04  //  0x04: LO2C Byte 2
#define LO2C_3       0x05  //  0x05: LO2C Byte 3
#define PWR          0x06  //  0x06: PWR Byte
#define LO_STATUS    0x07  //  0x07: LO Status Byte
#define RSVD_08      0x08  //  0x08: Reserved
#define MISC_STATUS  0x09  //  0x09: Misc Status Byte
#define HOHP         0x0A  //  0x0A: HOHP Byte
#define RSVD_0B      0x0B  //  0x0B: Reserved
#define RSVD_0C      0x0C  //  0x0C: Reserved
#define SRO          0x0D  //  0x0D: SRO Byte
#define RSVD_0E      0x0E  //  0x0E: Reserved
#define MIX_1        0x0F  //  0x0F: Mix 1 Byte
#define PK_DET       0x10  //  0x10: Peak Detect Byte

// Constants used by the tuning algorithm
#define REF_FREQ        4000000     // Reference oscillator Frequency (in Hz)
#define IF1_CENTER      1220000000  // Center of the IF1 filter (in Hz)
#define IF1_BW          16000000    // The IF1 filter bandwidth (in Hz)
#define TUNE_STEP_SIZE  50000       // Tune in steps of 50 kHz
#define SCALE_TO_STEP(x) (parseInt(((x) + TUNE_STEP_SIZE/2) / TUNE_STEP_SIZE))
#define PART_CODE()     (RtnVals[PART_REV] >> 4)
#define REV_CODE()      (RtnVals[PART_REV] & 0x0f)

function MicrotuneSetRfFrequency( Frequency , Channel, ProgramPal, ProgramNtsc )
{
   if(ProgramPal)
   {
      PalTunerVals = new Array();
      ChangeMicrotuneFreq( Frequency + 3000000, 36150000, 8750000, PalTunerVals);

      Class04d.I2cSeqTunerPalChannel =
         [
            "Tuner",   [ PalTunerVals ],
            "TunerIf", [ [ 0x00, 0x16, 0x70, 0x49 ] ]
         ];

      Class04d.DispPalChannel   = Channel;
      Class04d.DispPalFrequency = Frequency / 1e6;

      Out.Printf(Out.PriLow,"    PalTunerArray    = ");
      for(i=0; i< PalTunerVals.length; i++)
         Out.Printf(Out.PriLow,"%02x ", PalTunerVals[i]);
      Out.Printf(Out.PriLow,"\n");

   }

   if(ProgramNtsc)
   {
      NtscTunerVals = new Array();
      ChangeMicrotuneFreq( Frequency + 1750000, 43750000, 6750000, NtscTunerVals);

      Class04d.I2cSeqTunerNtscChannel =
         [
            "Tuner",   [ NtscTunerVals ],
            "TunerIf", [ [ 0x00, 0xD2, 0x30, 0x04 ] ]
         ];
      Class04d.DispNtscChannel   = Channel;
      Class04d.DispNtscFrequency = Frequency / 1e6;

      Out.Printf(Out.PriLow,"    NtscTunerArray   = ");
      for(i=0; i< NtscTunerVals.length; i++)
         Out.Printf(Out.PriLow,"%02x ", NtscTunerVals[i]);
      Out.Printf(Out.PriLow,"\n");
   }
}

// helper function for ChangeMicrotuneFreq
function IsSpurInBand(fLO1, fLO2, fIFOut, fIFBW)
{
    var MAX_LOSpurHarmonic = 11;
    var nLO1 = 1;
    var nLO2;
    var fSpur;

    //  Start out with fLO1 - 2*fLO2 - fIFOut
    //  Add LO1 harmonics until MAX_LOSpurHarmonic is reached.
    do
    {
        nLO2 = -nLO1;
        fSpur = nLO1 * (fLO1 - fLO2);

        // Subtract LO2 harmonics until fSpur is below fIFOut band pass.
        do
        {
            --nLO2;
            fSpur -= fLO2;

            // If the spur is within the band width - quit looking.
            // Compare 2x < BW vs. x < BW/2 for accuracy
            if (2 * (Math.abs(Math.abs(fSpur) - fIFOut)) < fIFBW)
                return 1;

          // To avoid losing a low order bit, compare 2x < BW vs. x < BW/2
        } while ( (2 * (fLO2 - fIFOut - fSpur) < fIFBW)
               && (nLO2 > -MAX_LOSpurHarmonic) );

    } while (++nLO1 < MAX_LOSpurHarmonic);
}
Log.Never("IsSpurInBand");

// params: RF input center frequency, IF output center frequency,
// IF output bandwidth, plus guard
function ChangeMicrotuneFreq( f_in, f_out, f_IFBW, RtnVals)
{
    //  These constants are in units of TUNE_STEP_SIZE
    var PKENRANGE = SCALE_TO_STEP(275000000); // Min frequency where the PKEN bit should be set. (275 MHz)
    var f_Ref = SCALE_TO_STEP(REF_FREQ);          // SRO frequency
    var f_IF1_BW = SCALE_TO_STEP(IF1_BW);         // 1st IF filter BW
    var f_IF1_Center = SCALE_TO_STEP(IF1_CENTER); // 1st IF filter Center
    var ONE_MEG = SCALE_TO_STEP(1000000);         // LO1 has 1 MHz step

    var status = 1;           // status of operation
    var SpurFound;            // Flag for locating spurs
    var LO1;                  // 1st LO register value
    var Num1;                 // Numerator for LO1 reg. value
    var LO2;                  // 2nd LO register value
    var Num2;                 // Numerator for LO2 reg. value
    var fLO1;                 // 1st LO frequency (in Hz)
    var fLO2;                 // 2nd LO frequency (in Hz)
    var zfLO1;                // Temp 1st LO frequency (in Hz)
    var zfLO2;                // Temp 2nd LO frequency (in Hz)
    var nLOAdjust;            // # of frequency adjustments
    var fAdjust;              // Frequency adjustment size

    //  Quantize all input arguments to the nearest TUNE_STEP_SIZE
    f_in   = SCALE_TO_STEP(f_in);
    f_out  = SCALE_TO_STEP(f_out);
    f_IFBW = SCALE_TO_STEP(f_IFBW);

    // Calculate frequency settings.  f_IF1_Center + f_in is the desired LO1
    // frequency, but LO1 only operates in steps of 1 MHz, so we have to round
    // fLO1 to the nearest 1 MHz;
    fLO1 = f_IF1_Center + f_in;
    fLO1 = ONE_MEG * parseInt((fLO1 + ONE_MEG/2) / ONE_MEG);
    fLO2 = fLO1 - f_out - f_in;

    // If there is an LO spur in this band, work outward until we find a
    // spur-free frequency or run up against the 1st IF SAW band edge.
    // Use temporary copies of fLO1 and fLO2 so that they will be unchanged
    // if a spur-free setting is not found.  NOTE: If the input IF1 freq
    // is NOT 1220 MHz, this search will hit one band edge before searching
    // the entire bandwidth. This should not be a problem since there should
    // be little reason to use a different IF1 value.

    var AvoidSpurs = 1; //Switch to allow spur-avoidance to be enabled/disabled

    if ( AvoidSpurs &&
         (SpurFound = IsSpurInBand(fLO1, fLO2, f_out, f_IFBW)) )
    {
        nLOAdjust = 1;
        zfLO1 = fLO1;
        zfLO2 = fLO2;

        do
        {
            fAdjust = nLOAdjust * ONE_MEG;
            if (nLOAdjust % 2)  /* Apply any adjustments */
            {
                zfLO1 += fAdjust;
                zfLO2 += fAdjust;
            }
            else
            {
                zfLO1 -= fAdjust;
                zfLO2 -= fAdjust;
            }

            // If our next try is outside the 1st IF bandpass, quit trying
            // To avoid loosing a low order bit, compare 2x > BW vs. x > BW/2
            if ((2 * Math.abs(zfLO1 - f_in - f_IF1_Center) + f_IFBW) > f_IF1_BW)
                break;


            ++nLOAdjust;

            SpurFound = IsSpurInBand(zfLO1, zfLO2, f_out, f_IFBW);
        }
        while ( SpurFound );

        // Use the LO-spur free values found.  If the search went all the way to
        // the 1st IF band edge and always found spurs, just use the original
        // choice.  It's as "good" as any other.
        if ( SpurFound == 0 )
        {
            fLO1 = zfLO1;
            fLO2 = zfLO2;
        }

    }

    // Recalculate the LO information since it may have changed while
    // searching for spur-free settings.
    LO1  = fLO1 / f_Ref;
    Num1 = parseInt((4 * (fLO1 % f_Ref)) / f_Ref);      // Truncate here
    LO2  = fLO2 / f_Ref;
    Num2 = parseInt(((4095 * (fLO2 % f_Ref)) + f_Ref/2) / f_Ref);  // Round here

    // Place all of the calculated values varo the local tuner register fields.
    RtnVals[0] = LO1C_1;
    RtnVals[LO1C_1] = (RtnVals[LO1C_1] & ~0x03) | Num1;              // LO1N
    RtnVals[LO1C_1] = (RtnVals[LO1C_1] & ~0x3C) | ((LO1 % 12) << 2); // LO1B
    RtnVals[LO1C_2] = (RtnVals[LO1C_2] & ~0x3F) | (LO1 / 12 - 1);    // LO1A

    if (Num2 == 0)   // FNON
        RtnVals[LO2C_3] &= ~0x40;
    else
        RtnVals[LO2C_3] |= 0x40;

    RtnVals[LO2C_2] = (Num2) & 0xFF;                              // F2N0-7
    RtnVals[LO2C_1] = (RtnVals[LO2C_1] & ~0x0F) | (Num2 >> 8) & 0x0F; // F2N8-11
    RtnVals[LO2C_1] = (RtnVals[LO2C_1] & ~0xE0) | ((LO2 % 8) << 5);   // LO2B
    RtnVals[LO2C_3] = (RtnVals[LO2C_3] & ~0x3F) | (LO2 / 8 - 1);      // LO2A

    if (f_in < PKENRANGE)    // VLLP
        RtnVals[LO1C_1] |= 0x80;
    else
        RtnVals[LO1C_1] &= ~0x80;
}

//------------------------------------------------------------------------------
// Code for programming the old FQ- and FM- series of RF receivers
//------------------------------------------------------------------------------
// Upper low and upper mid range band definitions
#define kUpperLowBand_MN         157250000 //0x0CB0
#define kUpperMidBand_MN         451250000 //0x1F10
#define kUpperLowBand_BGLDKI     168250000 //
#define kUpperMidBand_BGLDKI     447250000 //
#define kUpperLowBand_MK3        160000000 // philips mk3 tuners have similar low,mid
#define kUpperMidBand_MK3        442000000 // and high band ranges for ntsc and pal

// Low, Mid and High band control definitions
#define kLowBand                 0x8EA0
#define kMidBand                 0x8E90
#define kHighBand                0x8E30

#define kLowBand_fq1216me_L      0x8Eab
#define kMidBand_fq1216me_L      0x8E9b
#define kHighBand_fq1216me_L     0x8E3b
#define kLowBand_fq1216me_L1     0x8Eaa
#define kMidBand_fq1216me_L1     0x8E9a
#define kHighBand_fq1216me_L1    0x8E3a
#define kLowBand_fq1216me_BGDK   0x8Ea9
#define kMidBand_fq1216me_BGDK   0x8E99
#define kHighBand_fq1216me_BGDK  0x8E39
#define kLowBand_fq1216me_I      0x8Ea1
#define kMidBand_fq1216me_I      0x8E91
#define kHighBand_fq1216me_I     0x8E31

#define kLowBand_fq12x6me_mk3    0x8621
#define kMidBand_fq12x6me_mk3    0x8622
#define kHighBand_fq1x16me_mk3   0x8624

#define kIF_MN         45750000
#define kIF_BGLDK      38900000

#define FI1216            1
#define FI1216MF          2
#define FI1236            3
#define FI1246            4
#define FI1256            5
#define FQ1216ME          6
#define FQ1216ME_MK3      7
#define FQ1236ME_MK3      8
#define TEMIC_403         11   //ntsc
#define TEMIC_400         12   //pal b/g
#define TEMIC_406         13   //pal i
#define TEMIC_401         14   //pal d/k
#define FM1216            81
#define FM1216MF          82
#define FM1236            83
#define FM1246            84
#define FM1256            85
#define FM1216ME_MK3      87
#define FM1236ME_MK3      88

#define KS_AnalogVideo_NTSC_M   1
#define KS_AnalogVideo_PAL_B    2
#define KS_AnalogVideo_PAL_D    3
#define KS_AnalogVideo_PAL_G    4
#define KS_AnalogVideo_PAL_I    5

#define PAL_INT_FREQ   kIF_BGLDK
#define PAL_TUNER_ID   FQ1236ME_MK3
#define PAL_VIDEO_STD  KS_AnalogVideo_PAL_G

#define NTSC_INT_FREQ  kIF_MN
#define NTSC_TUNER_ID  FQ1216ME_MK3
#define NTSC_VIDEO_STD KS_AnalogVideo_NTSC_M

//------------------------------------------------------------------------------
// Set the tuner frequency
// This code stolen from \\sw\main\drivers\wdm\nvtunep\tunerhdw.cpp#13
//------------------------------------------------------------------------------
function PhilipsSetRfFrequency( Frequency , Channel, ProgramPal, ProgramNtsc )
{
   PalTunerArray    = new Array;
   PalTunerIfArray  = new Array;
   NtscTunerArray   = new Array;
   NtscTunerIfArray = new Array;

   // Set the video carrier frequency by controlling the programmable divider
   // N = ( 16 * ( FreqRF + FreqIntermediate)) / 1000000
   PalFrequencyDivider  = 16 * ( Frequency + PAL_INT_FREQ  ) / 1000000;
   NtscFrequencyDivider = 16 * ( Frequency + NTSC_INT_FREQ ) / 1000000;

   PalControlCode  = GetTunerControlCode( Frequency, PAL_TUNER_ID  );
   NtscControlCode = GetTunerControlCode( Frequency, NTSC_TUNER_ID );

   if((0 == PalControlCode) || (0 == NtscControlCode))
   {
      Out.Printf(Out.PriNormal, "Couldn't determine control code\n");
      return RC.SOFTWARE_ERROR;
   }

   PalTunerArray[0]  = ( PalFrequencyDivider >>> 8 ) & 0xff;
   PalTunerArray[1]  = ( PalFrequencyDivider       ) & 0xff;
   PalTunerArray[2]  = ( PalControlCode >>> 8      ) & 0xff;
   PalTunerArray[3]  = ( PalControlCode            ) & 0xff;

   NtscTunerArray[0] = ( NtscFrequencyDivider >>> 8) & 0xff;
   NtscTunerArray[1] = ( NtscFrequencyDivider      ) & 0xff;
   NtscTunerArray[2] = ( NtscControlCode >>> 8     ) & 0xff;
   NtscTunerArray[3] = ( NtscControlCode           ) & 0xff;

   if((PAL_TUNER_ID != FQ1236ME_MK3) && (PAL_TUNER_ID != FM1236ME_MK3))
   {
      Out.Printf(Out.PriNormal, "This PAL tuner type is unsupported.\n");
      return RC.SOFTWARE_ERROR;
   }

   if((NTSC_TUNER_ID != FQ1216ME_MK3) && (NTSC_TUNER_ID != FM1216ME_MK3))
   {
      Out.Printf(Out.PriNormal, "This PAL tuner type is unsupported.\n");
      return RC.SOFTWARE_ERROR;
   }

   PalTunerArray[4]    = 0x20;
   NtscTunerArray[4]   = 0xA0;

   PalTunerIfArray[0]  = 0; //subaddr
   NtscTunerIfArray[0] = 0; //subaddr

   //program IF section
   switch(PAL_VIDEO_STD)
   {
      case KS_AnalogVideo_PAL_B:
      case KS_AnalogVideo_PAL_G:
         PalTunerIfArray[1] = 0x16;
         PalTunerIfArray[2] = 0x70;
         PalTunerIfArray[3] = 0x49;
         break;
      case KS_AnalogVideo_PAL_I:
         PalTunerIfArray[1] = 0x16;
         PalTunerIfArray[2] = 0x70;
         PalTunerIfArray[3] = 0x4A;
         break;
      case KS_AnalogVideo_PAL_D:
         PalTunerIfArray[1] = 0x16;
         PalTunerIfArray[2] = 0x70;
         PalTunerIfArray[3] = 0x4B;
         break;
      default:
         Out.Printf(Out.PriNormal, "Invalid PAL video standard\n");
         return RC.SOFTWARE_ERROR;
   }

   NtscTunerIfArray[1] = 0x16;
   NtscTunerIfArray[2] = 0x10;
   NtscTunerIfArray[3] = 0x44;

   var i;
   if(ProgramPal)
   {
      Out.Printf(Out.PriLow,"PAL Tuner Information:\n");
      Out.Printf(Out.PriLow,"   PAL Frequency     = %d\n", Frequency);
      Out.Printf(Out.PriLow,"   PAL IF            = %d\n", PAL_INT_FREQ);
      Out.Printf(Out.PriLow,"   PAL FreqDiv       = %d\n", PalFrequencyDivider);
      Out.Printf(Out.PriLow,"   PAL CtrlCode      = %d\n", PalControlCode);
      Out.Printf(Out.PriLow,"   PAL TunerArray    = ");

      for(i=0; i< PalTunerArray.length; i++)
         Out.Printf(Out.PriLow,"%02x ", PalTunerArray[i]);

      Out.Printf(Out.PriLow,"\n   PAL TunerIfArray  = ", i, PalTunerIfArray[i]);
      for(i=0; i< PalTunerIfArray.length; i++)
         Out.Printf(Out.PriLow,"%02x ", PalTunerIfArray[i]);

      Out.Printf(Out.PriLow,"\n");

      Class04d.I2cSeqTunerPalChannel =
         [
            "Tuner",   [ PalTunerArray   ],
            "TunerIf", [ PalTunerIfArray ]
         ];
      // This doesn't actually do anything with the tuner...it is just for the
      // message that gets displayed for the user.
      Class04d.DispPalChannel   = Channel;
      // Re-calculate the exact frequency using the I2C values that were
      // send to the tuner.  This may be slightly different than the requested
      // frequency because of roundoff error.
      Class04d.DispPalFrequency =
         ((PalFrequencyDivider * 1000000.0 / 16.0) - PAL_INT_FREQ) / 1000000.0;
   }

   if(ProgramNtsc)
   {
      Out.Printf(Out.PriLow,"NTSC Tuner Information:\n");
      Out.Printf(Out.PriLow,"   NTSC Frequency    = %d\n", Frequency);
      Out.Printf(Out.PriLow,"   NTSC IF           = %d\n", NTSC_INT_FREQ)
      Out.Printf(Out.PriLow,"   NTSC FreqDiv      = %d\n", NtscFrequencyDivider);
      Out.Printf(Out.PriLow,"   NTSC CtrlCode     = %d\n", NtscControlCode);

      Out.Printf(Out.PriLow,"   NTSC TunerArray   = ");
      for(i=0; i< NtscTunerArray.length; i++)
         Out.Printf(Out.PriLow,"%02x ", NtscTunerArray[i]);

      Out.Printf(Out.PriLow,"\n   NTSC TunerIfArray = ", i, NtscTunerIfArray[i]);
      for(i=0; i< NtscTunerIfArray.length; i++)
         Out.Printf(Out.PriLow,"%02x ", NtscTunerIfArray[i]);

      Out.Printf(Out.PriLow,"\n");

      Class04d.I2cSeqTunerNtscChannel =
         [
            "Tuner",   [ NtscTunerArray   ],
            "TunerIf", [ NtscTunerIfArray ]
         ];
      // This doesn't actually do anything with the tuner...it is just for the
      // message that gets displayed for the user.
      Class04d.DispNtscChannel   = Channel;
      // Re-calculate the exact frequency using the I2C values that were
      // send to the tuner.  This may be slightly different than the requested
      // frequency because of roundoff error.
      Class04d.DispNtscFrequency =
         ((NtscFrequencyDivider * 1000000.0 / 16.0) - NTSC_INT_FREQ) / 1000000.0;
   }

   return OK;
}

//------------------------------------------------------------------------------
// Determines the Tuner control code to be send to tuner with a new frequency value
// This code stolen from \\sw\main\drivers\wdm\nvtunep\tunerhdw.cpp#13
//------------------------------------------------------------------------------
function GetTunerControlCode( Frequency, TunerId )
{
   var ControlCode = 0;

   switch( TunerId)
   {
      case FI1236 :    // NTSC N/A
      case FM1236 :    // NTSC Japan
         Out.Printf(Out.PriLow, "NVtunep: Type=1236\n");
         if( Frequency <= kUpperLowBand_MN)
            ControlCode = kLowBand;
         else if( Frequency <= kUpperMidBand_MN)
            ControlCode = kMidBand;
         else
            ControlCode = kHighBand;
         break;

      case FI1216:    // SECAM & PAL B/G
      case FM1216:
         Out.Printf(Out.PriLow, "NVtunep: Type=1216\n");

      case FI1246:
      case FM1246:
         Out.Printf(Out.PriLow, "NVtunep: Type=1246\n");

      case FI1256:
      case FM1256:
         Out.Printf(Out.PriLow, "NVtunep: Type=1256\n");
         if( Frequency <= kUpperLowBand_BGLDKI)
            ControlCode = kLowBand;
         else if( Frequency <= kUpperMidBand_BGLDKI)
            ControlCode = kMidBand;
         else
            ControlCode = kHighBand;
         break;

      case FQ1216ME:
      //case FM1216MF:
         Out.Printf(Out.PriLow, "NVtunep: Type=fq1216me\n");
         switch(VideoStandard)
         {
            case KS_AnalogVideo_PAL_B:
            case KS_AnalogVideo_PAL_G:
            case KS_AnalogVideo_PAL_D:
            case KS_AnalogVideo_SECAM_B:
            case KS_AnalogVideo_SECAM_G:
            case KS_AnalogVideo_SECAM_D:
            case KS_AnalogVideo_SECAM_K:
               Out.Printf(Out.PriLow, "NVtunep: Std=BGDK\n");
               if( Frequency <= kUpperLowBand_BGLDKI)
                  ControlCode = kLowBand_fq1216me_BGDK;
               else if( Frequency <= kUpperMidBand_BGLDKI)
                  ControlCode = kMidBand_fq1216me_BGDK;
               else
                  ControlCode = kHighBand_fq1216me_BGDK;
               break;

            case KS_AnalogVideo_PAL_I:
               Out.Printf(Out.PriLow, "NVtunep: Std=I\n");
               if( Frequency <= kUpperLowBand_BGLDKI)
                  ControlCode = kLowBand_fq1216me_I;
               else if( Frequency <= kUpperMidBand_BGLDKI)
                  ControlCode = kMidBand_fq1216me_I;
               else
                  ControlCode = kHighBand_fq1216me_I;
               break;

            case KS_AnalogVideo_SECAM_L:
               Out.Printf(Out.PriLow, "NVtunep: Std=L\n");
               if( Frequency <= kUpperLowBand_BGLDKI)
                  ControlCode = kLowBand_fq1216me_L;
               else if( Frequency <= kUpperMidBand_BGLDKI)
                  ControlCode = kMidBand_fq1216me_L;
               else
                  ControlCode = kHighBand_fq1216me_L;
               break;

            case KS_AnalogVideo_SECAM_L1:
               Out.Printf(Out.PriLow, "NVtunep: Std=L1\n");
               if( Frequency <= kUpperLowBand_BGLDKI)
                  ControlCode = kLowBand_fq1216me_L1;
               else if( Frequency <= kUpperMidBand_BGLDKI)
                  ControlCode = kMidBand_fq1216me_L1;
               else
                  ControlCode = kHighBand_fq1216me_L1;
               break;
         }
         break;

      case FQ1216ME_MK3:
         if( Frequency <= kUpperLowBand_MK3)
            ControlCode = kLowBand_fq12x6me_mk3;
         else if( Frequency <= kUpperMidBand_MK3)
            ControlCode = kMidBand_fq12x6me_mk3;
         else
            ControlCode = kHighBand_fq1x16me_mk3;
         break;

      case FQ1236ME_MK3:
         if( Frequency <= kUpperLowBand_MK3)
            ControlCode = kLowBand_fq12x6me_mk3;
         else if( Frequency <= kUpperMidBand_MK3)
            ControlCode = kMidBand_fq12x6me_mk3;
         else
            ControlCode = kHighBand_fq1x16me_mk3;
         break;

      default :
         Out.Printf(Out.PriLow,  "NVTUNEP: Unable to determine ctrl code - ERROR!!!!\n");
   }

   return ControlCode;
}
Log.Never("GetTunerControlCode");

function TvType()
{
   var TvStandard = null;

   // Find the first detected TV.
   var Detected = new Array;
   Display.GetConnectors(Detected);
   for (var i = 8; i < 16; ++i)
   {
      if (Detected[0] & (1 << i))
      {
         var r = new Array;
         Display.GetTvStandard(1 << i, r)
         TvStandard = r[0];
         break;
      }
   }

   return TvStandard;
}
Log.Never("TvType");

// For the NV35 Personal Cinema card, the design team wanted to use the
// "normal" NV35 card and just put a daughter card.  They wanted to just attach
// the card and make everything "just work" without flashing a new vbios.
// So, all NV35 P.C. cards have DCB entries for a non-tuner card.  We need this
// function to special-case some decisions elsewhere in this file.
function IsNv35Tuner()
{
   if(Gpu.DeviceId == Gpu.NV35)
   {
      // Look and see if there's an I2C EPROM at address 0xA0.  If there isn't,
      // we should get RC.SOFTWARE_ERROR.
      if( OK == I2C.Read( 3, 0xA0, 0, new Array ))
      {
         return true;
      }
   }

   return false;
}
Log.Never("IsNv35Tuner");

// Check to make sure that if an I2C EEPROM is present that it has been
// programmed.
function IsI2cEpromOk()
{
   RetVal = new Array();
   var rc;

   rc = I2C.Read(3, 0xA0, 0x40, RetVal);

   // If there isn't an I2C EEPROM, that's OK, because this board configuration
   // has all the necessary data in the vbios DCB tables.
   if(rc)
      return OK;

   // If there is an I2C EEPROM, and it has valid data at offset 0x40 (this offset
   // should always have a value), that's OK too.
   if(RetVal[0] != 0xFF)
      return OK;

   // If we got here, there is an I2C EEPROM but it has no data.  Fail.
   Out.Printf(Out.PriNormal, "I2C EPROM not initialized\n");

   return RC.WAS_NOT_INITIALIZED;
}
Log.Never("IsI2cEpromOk");

// The purpose of this function is to program some known reference values into
// non-volatile I2C memory chips.
function ProgramTvMemory()
{
   var Port        = 3;
   var ChipAddress = 0xA0;

   var RegisterList = new Array();


   if("object" != typeof(Class04d.DeviceAddresses.VideoDecoder))
   {
      Out.Printf(Out.PriNormal, "Couldn't find the list of decoder addresses.\n");
      return RC.SERIAL_COMMUNICATION_ERROR;
   }

   var PhAddress = 0;
   var i;
   var Junk = new Array;
   for( i in Class04d.DeviceAddresses.VideoDecoder )
   {
      if( OK == I2C.Read(Port, Class04d.DeviceAddresses.VideoDecoder[i], 0, Junk) )
      {
         PhAddress =Class04d.DeviceAddresses.VideoDecoder[i],
         Out.Printf(Out.PriLow, "Found decoder at address %02x\n", PhAddress);
         TvSetup( Port, PhAddress )
         FoundIt = true;
         break;
      }
   }

   if( 0 == PhAddress)
   {
      Out.Printf(Out.PriNormal, "Couldn't find a decoder.\n");
      return RC.SERIAL_COMMUNICATION_ERROR;
   }

   if(IsNv35Tuner())
   {
      if( (TvType() == Display.NtscM) || (TvType() == Display.NtscJ))
      {
         Out.Printf(Out.PriLow, "Writing P186 NTSC register set\n");
         RegisterList =
           [[ 0x00, 0x02 ], [ 0x01, 0x00 ], [ 0x40, 0x07 ], [ 0x41, 0x00 ],
            [ 0x42, 0x02 ], [ 0x44, 0x04 ], [ 0x45, 0x00 ], [ 0x46, 0x00 ],
            [ 0x48, 0x05 ], [ 0x49, 0x01 ], [ 0x4a, 0x01 ], [ 0x4c, 0xFF ],
            [ 0x4d, 0xFF ], [ 0x4e, 0xFF ], [ 0x50, 0xFF ], [ 0x51, 0xFF ],
            [ 0x52, 0xFF ], [ 0x54, 0xFF ], [ 0x55, 0xFF ], [ 0x56, 0xFF ],
            [ 0x5c, 0x69 ], [ 0x5d, 0x06 ], [ 0x5e, 0x05 ], [ 0x5f, 0xde ],
            [ 0x6a, 0x03 ], [ 0x7d, 0x09 ], [ 0x7e, 0x18 ], [ 0x8a, 0x00 ],
            [ 0x8b, 0x00 ], [ 0x9a, 0x01 ]];
      }
      else if( (TvType() == Display.PalBDGHI) || (TvType() == Display.PalN) ||
               (TvType() == Display.PalM)     || (TvType() == Display.PalNC) )
      {
         Out.Printf(Out.PriLow, "Writing P186 PAL register set\n");
         RegisterList =
            [[ 0x00, 0x03 ], [ 0x01, 0x01 ], [ 0x40, 0x07 ], [ 0x41, 0x00 ],
             [ 0x42, 0x02 ], [ 0x44, 0x04 ], [ 0x45, 0x00 ], [ 0x46, 0x00 ],
             [ 0x48, 0x05 ], [ 0x49, 0x01 ], [ 0x4a, 0x01 ], [ 0x4c, 0xFF ],
             [ 0x4d, 0xFF ], [ 0x4e, 0xFF ], [ 0x50, 0xFF ], [ 0x51, 0xFF ],
             [ 0x52, 0xFF ], [ 0x54, 0xFF ], [ 0x55, 0xFF ], [ 0x56, 0xFF ],
             [ 0x5c, 0x69 ], [ 0x5d, 0x06 ], [ 0x5e, 0x05 ], [ 0x5f, 0xde ],
             [ 0x6a, 0x03 ], [ 0x7d, 0x0a ], [ 0x7e, 0x18 ], [ 0x8a, 0x00 ],
             [ 0x8b, 0x00 ], [ 0x9a, 0x02 ]];
      }
      else
      {
         Out.Printf(Out.PriLow, "Couldn't determine the TV mode\n");
         return RC.INVALID_TV_STANDARD;
      }
   }
   else if((0x7114 == g_Device) || (0x7108 == g_Device))
   {
      if( (TvType() == Display.NtscM) || (TvType() == Display.NtscJ))
      {
         Out.Printf(Out.PriLow, "Writing P79 NTSC_BTSC register set\n");
         RegisterList =
           [[ 0x3d, 0x02 ], [ 0x3e, 0xa0 ], [ 0x3f, 0xa3 ], [ 0x40, 0x00 ],
            [ 0x41, 0x00 ], [ 0x42, 0x00 ], [ 0x43, 0x00 ], [ 0x44, 0x01 ],
            [ 0x45, 0x01 ], [ 0x46, 0x01 ], [ 0x47, 0x00 ], [ 0x48, 0x06 ],
            [ 0x49, 0x00 ], [ 0x4A, 0x02 ], [ 0x4B, 0x00 ], [ 0x4F, 0x00 ],
            [ 0x53, 0x00 ], [ 0x57, 0x00 ], [ 0x58, 0x00 ], [ 0x59, 0x00 ],
            [ 0x5A, 0x00 ], [ 0x5B, 0x00 ], [ 0x5C, 0x00 ], [ 0x5D, 0x00 ],
            [ 0x5E, 0x01 ], [ 0x5F, 0xDE ], [ 0x60, 0x00 ], [ 0x61, 0x00 ],
            [ 0x62, 0x00 ], [ 0x63, 0x00 ], [ 0x64, 0x00 ], [ 0x65, 0x00 ],
            [ 0x66, 0x00 ], [ 0x67, 0x00 ], [ 0x68, 0x00 ], [ 0x69, 0x00 ],
            [ 0x6A, 0x02 ], [ 0x7D, 0x84 ], [ 0x7E, 0x08 ], [ 0x7F, 0xCB ]];
      }
      else if( (TvType() == Display.PalBDGHI) || (TvType() == Display.PalN) ||
               (TvType() == Display.PalM)     || (TvType() == Display.PalNC) )
      {
         Out.Printf(Out.PriLow, "Writing P79 PAL-NICAM register set\n");
         RegisterList =
           [[ 0x40, 0x00 ], [ 0x41, 0x00 ], [ 0x42, 0x00 ], [ 0x43, 0x00 ],
            [ 0x44, 0x01 ], [ 0x45, 0x01 ], [ 0x46, 0x01 ], [ 0x47, 0x00 ],
            [ 0x48, 0x06 ], [ 0x49, 0x00 ], [ 0x4A, 0x02 ], [ 0x4B, 0x00 ],
            [ 0x4F, 0x00 ], [ 0x53, 0x00 ], [ 0x57, 0x00 ], [ 0x58, 0x00 ],
            [ 0x59, 0x00 ], [ 0x5A, 0x00 ], [ 0x5B, 0x00 ], [ 0x5C, 0x00 ],
            [ 0x5D, 0x00 ], [ 0x5E, 0x01 ], [ 0x5F, 0xDE ], [ 0x60, 0x00 ],
            [ 0x61, 0x00 ], [ 0x62, 0x00 ], [ 0x63, 0x00 ], [ 0x64, 0x00 ],
            [ 0x65, 0x00 ], [ 0x66, 0x00 ], [ 0x67, 0x00 ], [ 0x68, 0x00 ],
            [ 0x69, 0x00 ], [ 0x6A, 0x02 ], [ 0x7D, 0x83 ], [ 0x7E, 0x07 ],
            [ 0x7F, 0xCB ]];
      }
      else
      {
         Out.Printf(Out.PriLow, "Couldn't determine the TV mode\n");
         return RC.INVALID_TV_STANDARD;
      }
   }
   else // else not 7114 or 7108, so it must be a P104 board
   {
      if( (TvType() == Display.NtscM) || (TvType() == Display.NtscJ))
      {
         Out.Printf(Out.PriLow, "Writing P104 NTSC_BTSC register set\n");
         RegisterList =
            [[ 0x40, 0x00 ], [ 0x41, 0x00 ], [ 0x42, 0x00 ], [ 0x44, 0x01 ],
             [ 0x45, 0x01 ], [ 0x46, 0x01 ], [ 0x48, 0x06 ], [ 0x49, 0x00 ],
             [ 0x4a, 0x02 ], [ 0x5e, 0x01 ], [ 0x5f, 0xde ], [ 0x6a, 0x03 ],
             [ 0x7d, 0x86 ], [ 0x7e, 0x08 ]];
      }
      else if( (TvType() == Display.PalBDGHI) || (TvType() == Display.PalN) ||
               (TvType() == Display.PalM)     || (TvType() == Display.PalNC) )
      {
         Out.Printf(Out.PriLow, "Writing P104 PAL_NICAM register set\n");
         RegisterList =
            [[ 0x40, 0x00 ], [ 0x41, 0x00 ], [ 0x42, 0x00 ], [ 0x44, 0x01 ],
             [ 0x45, 0x01 ], [ 0x46, 0x01 ], [ 0x48, 0x06 ], [ 0x49, 0x00 ],
             [ 0x4a, 0x02 ], [ 0x5e, 0x01 ], [ 0x5f, 0xde ], [ 0x6a, 0x03 ],
             [ 0x7d, 0x87 ], [ 0x7e, 0x07 ]];

      }
      else
      {
         Out.Printf(Out.PriLow, "Couldn't determine the TV mode.\n");
         return RC.INVALID_TV_STANDARD;
      }
   } // if(7114 || 7108)

   // Now we have a list of registers to program in RegisterList.
   var ListSize = RegisterList.length;
   Out.Printf(Out.PriLow, "Writing %d registers\n", ListSize);
   var i=0;
   var rc = OK;

   // Write out the list
   for( i=0; i<ListSize ; i++ )
   {
      // Do the I2C Write
      rc = I2C.Write( Port,
                      ChipAddress,
                      RegisterList[i][0],
                      RegisterList[i][1]) ;

      // Give the operation a chance to complete
      Sleep(10);

      // Bail out if the resman reported a failure
      if (OK != rc )
      {
         Out.Printf(Out.PriNormal,
                    "Error %d occurred while writing register %x\n",
                    rc,
                    RegisterList[i][0]);

         return rc;
      }
   } // for i

   // Read it back to confirm the writes went out OK
   for( i=0; i<ListSize ; i++ )
   {
      var ReadValue = new Array();

      // Do the I2C Read
      rc = I2C.Read( Port,
                     ChipAddress,
                     RegisterList[i][0],
                     ReadValue);

      // Bail out if the resman reported a failure
      if (OK != rc )
      {
         Out.Printf(Out.PriNormal,
                    "Error %d occurred while reading register %x\n",
                    rc,
                    RegisterList[i][0]);

         return rc;
      }

      // Confirm the value is correct
      if ( ReadValue[0] != RegisterList[i][1] )
      {
         Out.Printf(Out.PriNormal,
                    "Error occurred while comparing register %02x " +
                    "Read %02x Expected %02x\n",
                    RegisterList[i][0],
                    ReadValue[0],
                    RegisterList[i][1]
                    );

         return RC.SERIAL_COMMUNICATION_ERROR;
      }
   } // for i

   return OK;
} // function ProgramTvMemory()


//------------------------------------------------------------------------------
/*
 * The Class04d.Run() test will use these tables to set up the Philips
 * decoder/encoder/tuner parts over the I2C connection.
 *
 * For the vivo test (no tuner):
 *  - examine strap register to determine NTSC vs. PAL.
 *  - send I2cSequencePal or I2cSequenceNtsc.
 *  - send I2cSequenceExternComposite.
 *    capture an image (this image is discarded).
 *  - send I2cSequenceExternSVideo.
 *    capture an image.
 *  - send I2cSequenceExternComposite.
 *    capture an image.
 *  - analyze S-Video and composite, find which one has chroma, display that one
 *    for the user to examine, prompt for pass/fail.
 *
 * For the tuner test:
 *  - examine strap register to determine NTSC vs. PAL.
 *  - send I2cSequencePal or I2cSequenceNtsc.
 *  - read from TunerIf and I2CExpander to verify tuner HW is present.
 *  - send I2cSequenceTunerPal or I2cSequenceTunerNtsc.
 *  - capture and display until user selects pass/fail for video quality.
 *  - send I2cSequenceAudioCdrom, prompt for pass/fail.
 *  - send I2cSequenceAudioAvpod, prompt for pass/fail.
 *  - send I2cSequenceAudioTuner, prompt for pass/fail.
 */

Class04d.DeviceAddresses =
   {
      "VideoDecoder"       : [0x40, 0x42, 0x48, 0x4A],
      // If the StereoSoundDecoder is at address 0x80, it is a Micronas 3425G.
      // 0xB0, 0xB4 and 0xB6 use the older part (I don't know the part #)
      "StereoSoundDecoder" : [0xB0, 0xB4, 0xB6, 0x80],
      "I2CExpander"        : [0x46],
      "Tuner"              : [0xC0],
      "TunerIf"            : [0x86],
   };

//------------------------------------------------------------------------------
// Placeholders for programming values
//------------------------------------------------------------------------------
Class04d.I2cSeqPal             = [ "VideoDecoder", [[]]];
Class04d.I2cSeqNtsc            = [ "VideoDecoder", [[]]];
Class04d.I2cSeqExternComposite = [ "VideoDecoder", [[]]];
Class04d.I2cSeqExternSVideo    = [ "VideoDecoder", [[]]];
Class04d.I2cSeqTunerPal        = [ "VideoDecoder", [[]]];
Class04d.I2cSeqTunerNtsc       = [ "VideoDecoder", [[]]];
Class04d.I2cSeqAudioCdrom      = [ "VideoDecoder", [[]]];
Class04d.I2cSeqAudioAvpod      = [ "VideoDecoder", [[]]];
Class04d.I2cSeqAudioTuner      = [ "VideoDecoder", [[]]];

//------------------------------------------------------------------------------
// Programming values for NV35 Personal Cinema Card
//------------------------------------------------------------------------------
Class04d.Nv35_I2cSeqTunerPal =
   [
      "VideoDecoder", [ [0x02, 0xc5], [0x09, 0x40] ],
      "StereoSoundDecoder",
        [ [ 0x00, 0x80, 0x00             ],
          [ 0x00, 0x00, 0x00             ],
          [ 0x10, 0x00, 0x30, 0x20, 0x03 ],
          [ 0x12, 0x00, 0x08, 0x03, 0x20 ],
          [ 0x12, 0x00, 0x0E, 0x24, 0x03 ],
          [ 0x12, 0x00, 0x10, 0x5A, 0x00 ],
          [ 0x10, 0x00, 0x20, 0x00, 0x03 ],
          [ 0x12, 0x00, 0x00, 0x79, 0x00 ] ],
   ];


Class04d.Nv35_I2cSeqTunerNtsc =
   [
      "VideoDecoder", [ [0x02, 0xC5], [0x09, 0x40] ],
      "StereoSoundDecoder",
        [ [ 0x00, 0x80, 0x00             ],
          [ 0x00, 0x00, 0x00             ],
          [ 0x10, 0x00, 0x30, 0x20, 0x03 ],
          [ 0x12, 0x00, 0x08, 0x04, 0x20 ],
          [ 0x12, 0x00, 0x0e, 0x24, 0x03 ],
          [ 0x10, 0x00, 0x20, 0x00, 0x21 ],
          [ 0x12, 0x00, 0x00, 0x79, 0x00 ] ],
        ];

Class04d.Nv35_I2cSeqAudioCdrom = [ ];

Class04d.Nv35_I2cSeqAudioAvpod =
   [
      "StereoSoundDecoder",
        [ [ 0x00, 0x80, 0x00             ],
          [ 0x00, 0x00, 0x00             ],
          [ 0x10, 0x00, 0x30, 0x20, 0x03 ],
          [ 0x12, 0x00, 0x08, 0x02, 0x20 ],
          [ 0x12, 0x00, 0x0D, 0x19, 0x00 ],
          [ 0x12, 0x00, 0x0e, 0x24, 0x03 ],
          [ 0x10, 0x00, 0x20, 0x00, 0x20 ],
          [ 0x12, 0x00, 0x00, 0x79, 0x00 ] ],

   ];

Class04d.Nv35_I2cSeqPowerDown =
   [
      "VideoDecoder",       [ [ 0x1f, 0x88 ] ],
      "StereoSoundDecoder", [ [ 0x12, 0x00, 0x00, 0x00, 0x00 ] ], // Mute Audio
   ];

//------------------------------------------------------------------------------
// Programming values for Philips 7108/7114
//------------------------------------------------------------------------------
Class04d.P7108_I2cSeqPal =
   [
      "VideoDecoder",
      [
         [0x00, 0x01], [0x01, 0x08], [0x02, 0xc0], [0x03, 0x33], [0x04, 0x00],
         [0x05, 0x00], [0x06, 0xeb], [0x07, 0xe0], [0x08, 0xb8], [0x09, 0x40],
         [0x0a, 0x80], [0x0b, 0x40], [0x0c, 0x40], [0x0d, 0x00], [0x0e, 0x09],
         [0x0f, 0x29], [0x10, 0x0e], [0x11, 0x00], [0x12, 0x00], [0x13, 0x00],
         [0x14, 0x04], [0x15, 0x11], [0x16, 0xfe], [0x17, 0xc0], [0x18, 0x40],
         [0x19, 0x80], [0x1a, 0x00], [0x1b, 0x00], [0x1c, 0x00], [0x1d, 0x00],
         [0x1e, 0x00], [0x1f, 0x81], [0x30, 0xbc], [0x31, 0xdf], [0x32, 0x02],
         [0x34, 0xcd], [0x35, 0xcc], [0x36, 0x3a], [0x38, 0x03], [0x39, 0x20],
         [0x3a, 0x00], [0x40, 0x00], [0x41, 0xff], [0x42, 0xff], [0x43, 0xff],
         [0x44, 0xff], [0x45, 0xff], [0x46, 0x77], [0x47, 0x77], [0x48, 0x77],
         [0x49, 0x77], [0x4a, 0x77], [0x4b, 0x77], [0x4c, 0x77], [0x4d, 0x77],
         [0x4e, 0x77], [0x4f, 0x77], [0x50, 0x77], [0x51, 0x77], [0x52, 0x77],
         [0x53, 0x77], [0x54, 0x77], [0x55, 0x77], [0x56, 0x77], [0x57, 0xff],
         [0x58, 0x00], [0x59, 0x47], [0x5a, 0x03], [0x5b, 0x03], [0x5c, 0x00],
         [0x5d, 0x00], [0x5e, 0x00], [0x5f, 0x00], [0x60, 0x00], [0x61, 0x21],
         [0x62, 0xcf], [0x63, 0x00], [0x64, 0x00], [0x65, 0x00], [0x66, 0x00],
         [0x67, 0x00], [0x68, 0x00], [0x80, 0x30], [0x81, 0x00], [0x82, 0x00],
         [0x83, 0x00], [0x84, 0xa0], [0x85, 0x10], [0x86, 0x45], [0x87, 0x01],
         [0x88, 0xf0], [0x8f, 0x0b], [0x90, 0x00], [0x91, 0x48], [0x92, 0x00],
         [0x93, 0x84], [0x94, 0x02], [0x95, 0x00], [0x96, 0xd0], [0x97, 0x02],
         [0x98, 0x07], [0x99, 0x00], [0x9a, 0x10], [0x9b, 0x00], [0x9c, 0xa0],
         [0x9d, 0x05], [0x9e, 0x10], [0x9f, 0x00], [0xa0, 0x01], [0xa1, 0x00],
         [0xa2, 0x00], [0xa4, 0x80], [0xa5, 0x40], [0xa6, 0x40], [0xa8, 0x00],
         [0xa9, 0x02], [0xaa, 0x00], [0xac, 0x00], [0xad, 0x01], [0xae, 0x00],
         [0xb0, 0x00], [0xb1, 0x04], [0xb2, 0x00], [0xb3, 0x04], [0xb4, 0x00],
         [0xb8, 0x00], [0xb9, 0x00], [0xba, 0x00], [0xbb, 0x00], [0xbc, 0x00],
         [0xbd, 0x00], [0xbe, 0x00], [0xbf, 0x00], [0xc0, 0x00], [0xc1, 0x08],
         [0xc2, 0x00], [0xc3, 0x80], [0xc4, 0x00], [0xc5, 0x00], [0xc6, 0xd0],
         [0xc7, 0x02], [0xc8, 0x16], [0xc9, 0x00], [0xca, 0x20], [0xcb, 0x01],
         [0xcc, 0xd0], [0xcd, 0x02], [0xce, 0x20], [0xcf, 0x01], [0xd0, 0x01],
         [0xd1, 0x00], [0xd2, 0x00], [0xd4, 0x80], [0xd5, 0x40], [0xd6, 0x40],
         [0xd8, 0x00], [0xd9, 0x04], [0xda, 0x00], [0xdc, 0x00], [0xdd, 0x02],
         [0xde, 0x00], [0xe0, 0x00], [0xe1, 0x04], [0xe2, 0x00], [0xe3, 0x04],
         [0xe4, 0x00], [0xe8, 0x00], [0xe9, 0x00], [0xea, 0x00], [0xeb, 0x00],
         [0xec, 0x00], [0xed, 0x00], [0xee, 0x00], [0xef, 0x00], [0x88, 0xd0],
         [0x88, 0xf0],
      ]
   ];
Class04d.P7108_I2cSeqNtsc =
   [
      "VideoDecoder",
      [
         [0x00, 0x01], [0x01, 0x08], [0x02, 0xc0], [0x03, 0x33], [0x04, 0x00],
         [0x05, 0x00], [0x06, 0xeb], [0x07, 0xe0], [0x08, 0xb8], [0x09, 0x40],
         [0x0a, 0x80], [0x0b, 0x40], [0x0c, 0x40], [0x0d, 0x00], [0x0e, 0x89],
         [0x0f, 0x1b], [0x10, 0x0e], [0x11, 0x00], [0x12, 0x00], [0x13, 0x00],
         [0x14, 0x04], [0x15, 0x11], [0x16, 0xfe], [0x17, 0xc0], [0x18, 0x40],
         [0x19, 0x80], [0x1a, 0x00], [0x1b, 0x00], [0x1c, 0x00], [0x1d, 0x00],
         [0x1e, 0x00], [0x1f, 0xa1], [0x30, 0xbc], [0x31, 0xdf], [0x32, 0x02],
         [0x34, 0xcd], [0x35, 0xcc], [0x36, 0x3a], [0x38, 0x03], [0x39, 0x20],
         [0x3a, 0x00], [0x40, 0x00], [0x41, 0xff], [0x42, 0xff], [0x43, 0xff],
         [0x44, 0xff], [0x45, 0xff], [0x46, 0x77], [0x47, 0x77], [0x48, 0x77],
         [0x49, 0x77], [0x4a, 0x77], [0x4b, 0x77], [0x4c, 0x77], [0x4d, 0x77],
         [0x4e, 0x77], [0x4f, 0x77], [0x50, 0x77], [0x51, 0x77], [0x52, 0x77],
         [0x53, 0x77], [0x54, 0x77], [0x55, 0x77], [0x56, 0x77], [0x57, 0xff],
         [0x58, 0x00], [0x59, 0x47], [0x5a, 0x06], [0x5b, 0x03], [0x5c, 0x00],
         [0x5d, 0x00], [0x5e, 0x00], [0x5f, 0x00], [0x60, 0x00], [0x61, 0x27],
         [0x62, 0x8f], [0x63, 0x00], [0x64, 0x00], [0x65, 0x00], [0x66, 0x00],
         [0x67, 0x00], [0x68, 0x00], [0x80, 0x30], [0x81, 0x00], [0x82, 0x00],
         [0x83, 0x00], [0x84, 0xa0], [0x85, 0x10], [0x86, 0x45], [0x87, 0x01],
         [0x88, 0xf0], [0x8f, 0x08], [0x90, 0x00], [0x91, 0x48], [0x92, 0x00],
         [0x93, 0x84], [0x94, 0x02], [0x95, 0x00], [0x96, 0xd0], [0x97, 0x02],
         [0x98, 0x04], [0x99, 0x00], [0x9a, 0x0d], [0x9b, 0x00], [0x9c, 0xf7],
         [0x9d, 0x05], [0x9e, 0x0d], [0x9f, 0x00], [0xa0, 0x01], [0xa1, 0x00],
         [0xa2, 0x00], [0xa4, 0x80], [0xa5, 0x40], [0xa6, 0x40], [0xa8, 0xe2],
         [0xa9, 0x01], [0xaa, 0x00], [0xac, 0xf1], [0xad, 0x00], [0xae, 0x00],
         [0xb0, 0x00], [0xb1, 0x04], [0xb2, 0x00], [0xb3, 0x04], [0xb4, 0x00],
         [0xb8, 0x00], [0xb9, 0x00], [0xba, 0x00], [0xbb, 0x00], [0xbc, 0x00],
         [0xbd, 0x00], [0xbe, 0x00], [0xbf, 0x00], [0xc0, 0x00], [0xc1, 0x08],
         [0xc2, 0x00], [0xc3, 0x80], [0xc4, 0x00], [0xc5, 0x00], [0xc6, 0xd0],
         [0xc7, 0x02], [0xc8, 0x11], [0xc9, 0x00], [0xca, 0xf0], [0xcb, 0x00],
         [0xcc, 0xd2], [0xcd, 0x02], [0xce, 0xf0], [0xcf, 0x00], [0xd0, 0x01],
         [0xd1, 0x00], [0xd2, 0x00], [0xd4, 0x80], [0xd5, 0x40], [0xd6, 0x40],
         [0xd8, 0x00], [0xd9, 0x04], [0xda, 0x00], [0xdc, 0x00], [0xdd, 0x02],
         [0xde, 0x00], [0xe0, 0x00], [0xe1, 0x04], [0xe2, 0x00], [0xe3, 0x04],
         [0xe4, 0x00], [0xe8, 0x00], [0xe9, 0x00], [0xea, 0x00], [0xeb, 0x00],
         [0xec, 0x00], [0xed, 0x00], [0xee, 0x00], [0xef, 0x00], [0x88, 0xd0],
         [0x88, 0xf0],
      ]
   ];
Class04d.P7108_I2cSeqTunerPal =
   [
      "VideoDecoder", [ [0x02, 0xc1], [0x09, 0x40] ],
      // Added by Albert Au for PAL Tuner audio
      "StereoSoundDecoder",
            [ [0x00, 0x1a],
              [0x01, 0xd4],
              [0x01, 0xc4],
              [0x02, 0x13],
              [0x0a, 0x88],
              [0x0c, 0x04],
              [0x0d, 0x04],
              [0x0e, 0x00],
              [0x0f, 0x03],
              [0x10, 0x88],
              [0x11, 0x50],
              [0x12, 0xbb],
              [0x14, 0x00],
              [0x15, 0x18],
              [0x03, 0x72],
              [0x04, 0x95],
              [0x05, 0x55],
              [0x06, 0x77],
              [0x07, 0xa1],
              [0x08, 0x01],
              [0x09, 0x00],
              [0x0b, 0x04], // was 02 for dual mode, 0x04 now set Stereo
              [0x13, 0x00] ],
      "I2CExpander", [ [0xff, 0x02] ],
   ];

Class04d.P7108_I2cSeqTunerNtsc =
   [
      "VideoDecoder", [ [0x02, 0xc1], [0x09, 0x40] ],
      "StereoSoundDecoder",
            [ [0x04, 0x0f], [0x05, 0x0f],
              [0x06, 0x40], [0x07, 0x00],
              [0x08, 0xA0], [0x09, 0xA3],
              [0x0a, 0x03] ],
      "I2CExpander", [ [0xff, 0x02] ],
   ];

Class04d.P7108_I2cSeqAudioCdrom =
   [
      "I2CExpander", [ [0xff, 0xf0] ]
   ];
Class04d.P7108_I2cSeqAudioAvpod =
   [
      "I2CExpander", [ [0xff, 0xf1] ]
   ];
Class04d.P7108_I2cSeqAudioTuner =
   [
      "I2CExpander", [ [0xff, 0xf2] ]
   ];
Class04d.P7108_I2cSeqPowerDown =
   [
      "VideoDecoder", [ [0x1f, 0x88] ]
   ];

//------------------------------------------------------------------------------
// Programming values for Philips 713X
//------------------------------------------------------------------------------
Class04d.P713X_I2cSeqPal =
   [
      "VideoDecoder",
      [
         // These values are the same as NTSC except for:
         //  0x48, 0x4A, 0x4E, 0x58, 0x5A, 0x5E, 0x5F
         [ 0x00, 0x00, 0x06, 0x20, 0x00, 0x00 ],
         [ 0x00, 0x04, 0x03, 0x00, 0x01, 0x30 ],
         [ 0x00, 0x0c, 0x00, 0x00, 0x00, 0x00 ],
         [ 0x00, 0x10, 0x10, 0x20, 0x30, 0x40 ],
         [ 0x00, 0x14, 0x50, 0x60, 0x70, 0x80 ],
         [ 0x00, 0x18, 0x90, 0xa0, 0xb0, 0xc0 ],
         [ 0x00, 0x1c, 0xd0, 0xe0, 0xf0, 0xff ],
         [ 0x00, 0x20, 0x10, 0x20, 0x30, 0x40 ],
         [ 0x00, 0x24, 0x50, 0x60, 0x70, 0x80 ],
         [ 0x00, 0x28, 0x90, 0xa0, 0xb0, 0xc0 ],
         [ 0x00, 0x2c, 0xd0, 0xe0, 0xf0, 0xff ],
         [ 0x00, 0x30, 0x10, 0x20, 0x30, 0x40 ],
         [ 0x00, 0x34, 0x50, 0x60, 0x70, 0x80 ],
         [ 0x00, 0x38, 0x90, 0xa0, 0xb0, 0xc0 ],
         [ 0x00, 0x3c, 0xd0, 0xe0, 0xf0, 0xff ],
         [ 0x00, 0x40, 0x0f, 0x01, 0xc3, 0x00 ],
         [ 0x00, 0x44, 0x00, 0x00, 0xcf, 0x02 ],
         [ 0x00, 0x48, 0x04, 0x00, 0x12, 0x00 ],
         [ 0x00, 0x4c, 0xa0, 0x05, 0x0f, 0x00 ],
         [ 0x00, 0x54, 0x00, 0x00, 0xcf, 0x02 ],
         [ 0x00, 0x58, 0x13, 0x00, 0x34, 0x01 ],
         [ 0x00, 0x5c, 0xd0, 0x02, 0x20, 0x01 ],
         [ 0x00, 0x60, 0x01, 0x01, 0x01, 0x00 ],
         [ 0x00, 0x64, 0x80, 0x40, 0x40, 0x00 ],
         [ 0x00, 0x68, 0x00, 0x02, 0x00, 0x00 ],
         [ 0x00, 0x6c, 0x00, 0x04, 0x00, 0x00 ],
         [ 0x00, 0x70, 0x00, 0x04, 0x00, 0x00 ],
         [ 0x00, 0x74, 0x00, 0x00, 0x00, 0x00 ],
         [ 0x00, 0x80, 0x0f, 0x01, 0x03, 0x00 ],
         [ 0x00, 0x84, 0x00, 0x00, 0xcf, 0x02 ],
         [ 0x00, 0x88, 0x00, 0x00, 0x0b, 0x00 ],
         [ 0x00, 0x8c, 0xd0, 0x02, 0x0c, 0x00 ],
         [ 0x00, 0x94, 0x00, 0x00, 0xcf, 0x02 ],
         [ 0x00, 0x98, 0x0e, 0x00, 0xff, 0x00 ],
         [ 0x00, 0x9c, 0xd0, 0x02, 0xf0, 0x00 ],
         [ 0x00, 0xa0, 0x01, 0x00, 0x00, 0x00 ],
         [ 0x00, 0xa4, 0x80, 0x40, 0x40, 0x00 ],
         [ 0x00, 0xa8, 0x00, 0x04, 0x00, 0x00 ],
         [ 0x00, 0xac, 0x00, 0x04, 0x00, 0x00 ],
         [ 0x00, 0xb0, 0x00, 0x04, 0x00, 0x00 ],
         [ 0x00, 0xb4, 0x00, 0x00, 0x00, 0x00 ],
         [ 0x01, 0x00, 0x00, 0x48, 0x83, 0x30 ],
         [ 0x01, 0x04, 0x90, 0x90, 0xeb, 0xe0 ],
         [ 0x01, 0x08, 0x98, 0x40, 0x80, 0x44 ],
         [ 0x01, 0x0c, 0x40, 0x00, 0x01, 0x21 ],
         [ 0x01, 0x10, 0x00, 0x00, 0x00, 0x00 ],
         [ 0x01, 0x14, 0x01, 0x11, 0xfe, 0x18 ],
         [ 0x01, 0x18, 0x40, 0x80, 0x77, 0x00 ],
         [ 0x01, 0x1c, 0x00, 0x00, 0xa5, 0xb7 ],
         [ 0x01, 0x5c, 0xb7, 0xb7, 0xb7, 0xb7 ],
         [ 0x01, 0x90, 0x90, 0xb0, 0x00, 0xac ],
         [ 0x01, 0x94, 0x00, 0x00, 0x00, 0x00 ],
         [ 0x01, 0x98, 0x00, 0x00, 0x00, 0x00 ],
         [ 0x01, 0xa0, 0x6c, 0xb9, 0x48, 0x01 ],
         [ 0x01, 0xb0, 0x00, 0x00, 0x00, 0x00 ],
         [ 0x01, 0xb4, 0x00, 0x00, 0x01, 0x00 ],
         [ 0x01, 0xc0, 0x03, 0x00, 0x00, 0x00 ],
         [ 0x01, 0xd0, 0x01, 0x00, 0x00, 0x00 ],
         [ 0x02, 0xa8, 0x00, 0xfd, 0x00, 0x00 ],
         [ 0x02, 0xc0, 0xf0, 0x05, 0x00, 0x00 ],
         [ 0x02, 0xc8, 0x00, 0x00, 0x00, 0x00 ],
         [ 0x02, 0xcc, 0x80, 0x3f, 0x3f, 0x00 ],
      ]
   ];
Class04d.P713X_I2cSeqNtsc =
   [
      "VideoDecoder",
      [
         [ 0x00, 0x00, 0x06, 0x20, 0x00, 0x00 ],
         [ 0x00, 0x04, 0x03, 0x00, 0x01, 0x30 ],
         [ 0x00, 0x0c, 0x00, 0x00, 0x00, 0x00 ],
         [ 0x00, 0x10, 0x10, 0x20, 0x30, 0x40 ],
         [ 0x00, 0x14, 0x50, 0x60, 0x70, 0x80 ],
         [ 0x00, 0x18, 0x90, 0xa0, 0xb0, 0xc0 ],
         [ 0x00, 0x1c, 0xd0, 0xe0, 0xf0, 0xff ],
         [ 0x00, 0x20, 0x10, 0x20, 0x30, 0x40 ],
         [ 0x00, 0x24, 0x50, 0x60, 0x70, 0x80 ],
         [ 0x00, 0x28, 0x90, 0xa0, 0xb0, 0xc0 ],
         [ 0x00, 0x2c, 0xd0, 0xe0, 0xf0, 0xff ],
         [ 0x00, 0x30, 0x10, 0x20, 0x30, 0x40 ],
         [ 0x00, 0x34, 0x50, 0x60, 0x70, 0x80 ],
         [ 0x00, 0x38, 0x90, 0xa0, 0xb0, 0xc0 ],
         [ 0x00, 0x3c, 0xd0, 0xe0, 0xf0, 0xff ],
         [ 0x00, 0x40, 0x0f, 0x01, 0xc3, 0x00 ],
         [ 0x00, 0x44, 0x00, 0x00, 0xcf, 0x02 ],
         [ 0x00, 0x48, 0x07, 0x00, 0x14, 0x00 ],
         [ 0x00, 0x4c, 0xa0, 0x05, 0x0e, 0x00 ],
         [ 0x00, 0x54, 0x00, 0x00, 0xcf, 0x02 ],
         [ 0x00, 0x58, 0x15, 0x00, 0x07, 0x01 ],
         [ 0x00, 0x5c, 0xd0, 0x02, 0xf2, 0x00 ],
         [ 0x00, 0x60, 0x01, 0x01, 0x01, 0x00 ],
         [ 0x00, 0x64, 0x80, 0x40, 0x40, 0x00 ],
         [ 0x00, 0x68, 0x00, 0x02, 0x00, 0x00 ],
         [ 0x00, 0x6c, 0x00, 0x04, 0x00, 0x00 ],
         [ 0x00, 0x70, 0x00, 0x04, 0x00, 0x00 ],
         [ 0x00, 0x74, 0x00, 0x00, 0x00, 0x00 ],
         [ 0x00, 0x80, 0x0f, 0x01, 0x03, 0x00 ],
         [ 0x00, 0x84, 0x00, 0x00, 0xcf, 0x02 ],
         [ 0x00, 0x88, 0x00, 0x00, 0x0b, 0x00 ],
         [ 0x00, 0x8c, 0xd0, 0x02, 0x0c, 0x00 ],
         [ 0x00, 0x94, 0x00, 0x00, 0xcf, 0x02 ],
         [ 0x00, 0x98, 0x0e, 0x00, 0xff, 0x00 ],
         [ 0x00, 0x9c, 0xd0, 0x02, 0xf0, 0x00 ],
         [ 0x00, 0xa0, 0x01, 0x00, 0x00, 0x00 ],
         [ 0x00, 0xa4, 0x80, 0x40, 0x40, 0x00 ],
         [ 0x00, 0xa8, 0x00, 0x04, 0x00, 0x00 ],
         [ 0x00, 0xac, 0x00, 0x04, 0x00, 0x00 ],
         [ 0x00, 0xb0, 0x00, 0x04, 0x00, 0x00 ],
         [ 0x00, 0xb4, 0x00, 0x00, 0x00, 0x00 ],
         [ 0x01, 0x00, 0x00, 0x48, 0x83, 0x30 ],
         [ 0x01, 0x04, 0x90, 0x90, 0xeb, 0xe0 ],
         [ 0x01, 0x08, 0x98, 0x40, 0x80, 0x44 ],
         [ 0x01, 0x0c, 0x40, 0x00, 0x01, 0x21 ],
         [ 0x01, 0x10, 0x00, 0x00, 0x00, 0x00 ],
         [ 0x01, 0x14, 0x01, 0x11, 0xfe, 0x18 ],
         [ 0x01, 0x18, 0x40, 0x80, 0x77, 0x00 ],
         [ 0x01, 0x1c, 0x00, 0x00, 0xa5, 0xb7 ],
         [ 0x01, 0x5c, 0xb7, 0xb7, 0xb7, 0xb7 ],
         [ 0x01, 0x90, 0x90, 0xb0, 0x00, 0xac ],
         [ 0x01, 0x94, 0x00, 0x00, 0x00, 0x00 ],
         [ 0x01, 0x98, 0x00, 0x00, 0x00, 0x00 ],
         [ 0x01, 0xa0, 0x6c, 0xb9, 0x48, 0x01 ],
         [ 0x01, 0xb0, 0x00, 0x00, 0x00, 0x00 ],
         [ 0x01, 0xb4, 0x00, 0x00, 0x01, 0x00 ],
         [ 0x01, 0xc0, 0x03, 0x00, 0x00, 0x00 ],
         [ 0x01, 0xd0, 0x01, 0x00, 0x00, 0x00 ],
         [ 0x02, 0xa8, 0x00, 0xfd, 0x00, 0x00 ],
         [ 0x02, 0xc0, 0xf0, 0x05, 0x00, 0x00 ],
         [ 0x02, 0xc8, 0x00, 0x00, 0x00, 0x00 ],
         [ 0x02, 0xcc, 0x80, 0x3f, 0x3f, 0x00 ],
      ]
   ];
Class04d.P713X_I2cSeqTunerPal =
   [
      "VideoDecoder", [ [ 0x01, 0x02, 0x81, ],
                        [ 0x05, 0x94, 0x02, ] ] // stereo sound from DSP DAC
   ];

Class04d.P713X_I2cSeqTunerNtsc =
   [
      "VideoDecoder", [ [ 0x01, 0x02, 0x81, ],
                        [ 0x05, 0x94, 0x02, ] ] // stereo sound from DSP DAC
   ];
Class04d.P713X_I2cSeqPowerDown =
   [
      "VideoDecoder", [ [ 0x01, 0x1f, 0x88, ],
                        [ 0x01, 0x6E, 0x07, ],  // turn off 7134 audio
                        [ 0x05, 0x94, 0x07, ] ] // turn off 7133 audio
   ];

//------------------------------------------------------------------------------
// Programming values for Philips 7133
//------------------------------------------------------------------------------
Class04d.P7133_I2cSeqAudioCdrom =
   [
      "VideoDecoder", [ [ 0x05, 0x94, 0x00, ] ] // analog input 1
   ];
Class04d.P7133_I2cSeqAudioAvpod =
   [
      "VideoDecoder", [ [ 0x05, 0x94, 0x01, ] ] // analog input 2
   ];
Class04d.P7133_I2cSeqAudioTuner =
   [
      "VideoDecoder", [
                        [ 0x04, 0x40, 0x11, 0x63, 0x00, 0x00 ],
                        [ 0x04, 0x44, 0x11, 0x00, 0x01, 0x00 ],
                        [ 0x04, 0x48, 0x93, 0x00, 0x01, 0x00 ],
                        [ 0x04, 0x4c, 0x80, 0x00, 0x00, 0x00 ],
                        [ 0x04, 0x50, 0x80, 0x0c, 0x19, 0x00 ],
                        [ 0x04, 0x54, 0x4c, 0x00, 0x00, 0x00 ],
                        [ 0x04, 0x58, 0x80, 0x00, 0x00, 0x00 ],
                        [ 0x04, 0x5c, 0x0c, 0x00, 0x00, 0x00 ],
                        [ 0x04, 0x64, 0x00, 0x00, 0x00, 0x00 ],
                        [ 0x04, 0x68, 0x00, 0x00, 0x00, 0x00 ],
                        [ 0x04, 0x70, 0x10, 0x10, 0x10, 0x00 ],
                        [ 0x04, 0x74, 0x00, 0x00, 0x00, 0x00 ],
                        [ 0x04, 0x78, 0x10, 0x3c, 0x00, 0x00 ],
                        [ 0x04, 0x7c, 0x00, 0x20, 0x00, 0x00 ],
                        [ 0x04, 0x80, 0x00, 0x00, 0x00, 0x00 ],
                        [ 0x04, 0x84, 0x00, 0x00, 0x00, 0x00 ],
                        [ 0x04, 0x88, 0x00, 0x00, 0x00, 0x00 ],
                        [ 0x04, 0x8c, 0x00, 0x00, 0x00, 0x00 ],
                        [ 0x05, 0x84, 0x00, 0x00, 0x00, 0x00 ],
                        [ 0x05, 0x90, 0x00, 0x00, 0x00, 0x00 ],
                        [ 0x05, 0x94, 0x02, 0x00, 0x00, 0x00 ],
                        [ 0x05, 0x98, 0x00, 0x00, 0x20, 0x02 ],
                        [ 0x05, 0x9c, 0x00, 0x20, 0x00, 0x00 ],
                        [ 0x04, 0x54, 0xcc, 0x00, 0x00, 0x00 ]]
   ];

//------------------------------------------------------------------------------
// Programming values for Philips 7134
//------------------------------------------------------------------------------
Class04d.P7134_I2cSeqAudioCdrom =
   [
      "VideoDecoder", [ [ 0x01, 0x6E, 0x00, ] ] // analog input 1
   ];
Class04d.P7134_I2cSeqAudioAvpod =
   [
      "VideoDecoder", [ [ 0x01, 0x6E, 0x01, ] ] // analog input 2
   ];
Class04d.P7134_I2cSeqAudioTuner =
   [
      "VideoDecoder", [ [ 0x01, 0x6E, 0x02, ], // stereo sound from DSP DAC
                        [ 0x01, 0x48, 0x14, 0x50, 0x04, 0x00 ],
                        [ 0x01, 0x4c, 0x1f, 0x00, 0x00, 0x00 ],
                        [ 0x01, 0x50, 0x55, 0x95, 0x72, 0x00 ],
                        [ 0x01, 0x54, 0x00, 0xa1, 0x77, 0x00 ],
                        [ 0x01, 0x5c, 0x00, 0x00, 0x00, 0x00 ],
                        [ 0x01, 0x60, 0x20, 0x00, 0x80, 0x00 ],
                        [ 0x01, 0x64, 0x00, 0x03, 0x00, 0x01 ],
                        [ 0x01, 0x68, 0x00, 0x00, 0x00, 0x00 ],
                        [ 0x01, 0x6c, 0x00, 0x00, 0x02, 0x00 ],
                        [ 0x01, 0x70, 0x00, 0x00, 0x20, 0x02 ],
                        [ 0x01, 0x74, 0x00, 0x20, 0x00, 0x00 ] ]
   ];

//------------------------------------------------------------------------------
// TvSetup (called from C code)
//------------------------------------------------------------------------------
var g_Device  = 0;
var g_Port    = 0;
var g_Address = 0;

function TvSetup( Port, Address )
{
   g_Port    = Port;
   g_Address = Address;

   // Set the default PAL and NTSC frequency and channel.  Note that the channel
   // parameter doesn't actually set the channel; it is just what gets displayed
   // on the screen that prompts the user.
   if(g_TunerRfFrequency)
   {
      SetRfFrequency( g_TunerRfFrequency, 0, 1, 1 ); // Set both NTSC and PAL
   }
   else
   {
      SetRfFrequency(  61250000,  3, 0, 1 ); // NTSC  CH3= 61.25 MHz
      SetRfFrequency( 591250000, 36, 1, 0 ); // PAL  CH36=591.25 MHz
   }

   var InputModes = new Array;
   GetVideoInputModes(InputModes);

   var i;
   var a = new Array;
   for(i=2;i<6;i++)
   {
      I2C.Write(Port, Address, 0, i);
      I2C.Read(Port, Address, 0, a);
      g_Device = g_Device | (a[0] << 4*(5-i));
   }

   if(IsNv35Tuner())
   {
      Out.Printf(Out.PriLow,"Philips device is a 7108/7114 on daughter card\n");

      // We know that we definitely have a 7108/7114.
      var CompMode = (InputModes[0] == null) ? 0 : InputModes[0];
      var SvidMode = (InputModes[1] == null) ? 6 : InputModes[1];

      Class04d.I2cSeqAudioCdrom      = Class04d.Nv35_I2cSeqAudioCdrom;
      Class04d.I2cSeqAudioAvpod      = Class04d.Nv35_I2cSeqAudioAvpod;
      Class04d.I2cSeqTunerPal        = Class04d.Nv35_I2cSeqTunerPal;
      Class04d.I2cSeqTunerNtsc       = Class04d.Nv35_I2cSeqTunerNtsc;
      Class04d.I2cSeqPowerDown       = Class04d.Nv35_I2cSeqPowerDown;


      Class04d.I2cSeqPal             = Class04d.P7108_I2cSeqPal;
      Class04d.I2cSeqNtsc            = Class04d.P7108_I2cSeqNtsc;

      if( (TvType() == Display.NtscM) || (TvType() == Display.NtscJ))
      {
         Class04d.I2cSeqAudioTuner   = Class04d.Nv35_I2cSeqTunerNtsc;
      }
      else
      {
         Class04d.I2cSeqAudioTuner   = Class04d.Nv35_I2cSeqTunerPal;
      }

      Class04d.I2cSeqExternComposite =
         [ "VideoDecoder", [ [0x02, 0xc0 | CompMode], [0x09, 0x40] ]];

      Class04d.I2cSeqExternSVideo =
         [ "VideoDecoder", [ [0x02, 0xc0 | SvidMode], [0x09, 0xc0] ]];

      Class04d.I2cSeqExternCompositeAlternate =
         [ "VideoDecoder", [ [0x02, 0xc4 ], [0x09, 0x40] ]];

      Class04d.I2cSeqExternSVideoAlternate =
         [ "VideoDecoder", [ [0x02, 0xc7 ], [0x09, 0xc0] ]];
   }
   else if((0x7114 == g_Device) || (0x7108 == g_Device))
   {
      Out.Printf(Out.PriLow,"Philips device is a 7108/7114\n");

      var CompMode = (InputModes[0] == null) ? 0 : InputModes[0];
      var SvidMode = (InputModes[1] == null) ? 6 : InputModes[1];

      Class04d.I2cSeqPal             = Class04d.P7108_I2cSeqPal;
      Class04d.I2cSeqNtsc            = Class04d.P7108_I2cSeqNtsc;
      Class04d.I2cSeqTunerPal        = Class04d.P7108_I2cSeqTunerPal;
      Class04d.I2cSeqTunerNtsc       = Class04d.P7108_I2cSeqTunerNtsc;
      Class04d.I2cSeqAudioCdrom      = Class04d.P7108_I2cSeqAudioCdrom;
      Class04d.I2cSeqAudioAvpod      = Class04d.P7108_I2cSeqAudioAvpod;
      Class04d.I2cSeqAudioTuner      = Class04d.P7108_I2cSeqAudioTuner;
      Class04d.I2cSeqPowerDown       = Class04d.P7108_I2cSeqPowerDown;

      Class04d.I2cSeqExternComposite =
         [ "VideoDecoder", [ [0x02, 0xc0 | CompMode], [0x09, 0x40] ]];

      Class04d.I2cSeqExternSVideo =
         [ "VideoDecoder", [ [0x02, 0xc0 | SvidMode], [0x09, 0xc0] ]];

      Class04d.I2cSeqExternCompositeAlternate = [];
      Class04d.I2cSeqExternSVideoAlternate    = [];
   }
   else  // must be a 713X
   {
      Out.Printf(Out.PriLow,"Philips device is a 713X\n");

      var CompMode = (InputModes[0] == null) ? 3 : InputModes[0];
      var SvidMode = (InputModes[1] == null) ? 6 : InputModes[1];

      Class04d.I2cSeqPal             = Class04d.P713X_I2cSeqPal;
      Class04d.I2cSeqNtsc            = Class04d.P713X_I2cSeqNtsc;
      Class04d.I2cSeqTunerPal        = Class04d.P713X_I2cSeqTunerPal;
      Class04d.I2cSeqTunerNtsc       = Class04d.P713X_I2cSeqTunerNtsc;
      Class04d.I2cSeqPowerDown       = Class04d.P713X_I2cSeqPowerDown;

      Class04d.I2cSeqExternComposite =
         [ "VideoDecoder", [ [ 0x01, 0x02, 0x80 | CompMode, ], [ 0x01, 0x09, 0x40,], ]];
      Class04d.I2cSeqExternSVideo    =
         [ "VideoDecoder", [ [ 0x01, 0x02, 0x80 | SvidMode, ], [ 0x01, 0x09, 0xC0,], ]];

      // now we need to differentiate between 7133 and 7134
      var ReadData = 0;
      for(i=0;i<3;i++)
      {
         I2C.WriteBytes(Port, Address, [0x00, 0x24 + i, 0]);
         I2C.ReadDouble(Port, Address, 0x424 + i, a);
         ReadData = ReadData | (a[0] << 4*i);
      }

      if((0x00000000 == ReadData) || (0xFFFFFFFF == ReadData))
      {
         g_Device = 0x7134;
         Class04d.I2cSeqAudioCdrom      = Class04d.P7134_I2cSeqAudioCdrom;
         Class04d.I2cSeqAudioAvpod      = Class04d.P7134_I2cSeqAudioAvpod;
         Class04d.I2cSeqAudioTuner      = Class04d.P7134_I2cSeqAudioTuner;
      }
      else
      {
         g_Device = 0x7133;
         Class04d.I2cSeqAudioCdrom      = Class04d.P7133_I2cSeqAudioCdrom;
         Class04d.I2cSeqAudioAvpod      = Class04d.P7133_I2cSeqAudioAvpod;
         Class04d.I2cSeqAudioTuner      = Class04d.P7133_I2cSeqAudioTuner;
      }

      Class04d.I2cSeqExternCompositeAlternate = [];
      Class04d.I2cSeqExternSVideoAlternate    = [];
   }
   // NV31 and higher Personal Cinema Cards don't use the CD-ROM input.
   if( Gpu.DeviceId >= Gpu.NV31 )
      Class04d.DoCdromAudio = 0;
}

//------------------------------------------------------------------------------
// Determine if the signal is locked (called from C code)
//------------------------------------------------------------------------------
function IsSignalLocked()
{
   var rval = new Array;
   if((0x7114 == g_Device) || (0x7108 == g_Device))
   {
      I2C.Read( g_Port, g_Address, 0x1f , rval );
   }
   else  // must be a 713X
   {
      I2C.ReadDouble( g_Port, g_Address, 0x11f , rval );
   }

   return (1 == (rval[0] & 0x41));
}
Log.Never("IsSignalLocked");
