/*
 * NVIDIA_COPYRIGHT_BEGIN
 *
 * Copyright 1999-2003 by NVIDIA Corporation.  All rights reserved.  All
 * information contained herein is proprietary and confidential to NVIDIA
 * Corporation.  Any use, reproduction, or disclosure without the written
 * permission of NVIDIA Corporation is prohibited.
 *
 * NVIDIA_COPYRIGHT_END
 */

#include "mods.h"
#include "boards.js"
#include "glrandom.js"
#include "progtv.js"

//------------------------------------------------------------------------------
// Global variables.
//
var g_MfgtestId = "$Id: //sw/nvr40/diag/mods/js/mfgtest.js#101 $";

#define TM_OQA    (1 << 0)
#define TM_GPU    (1 << 1)
#define TM_ENGR   (1 << 2)
#define TM_MCP    (1 << 3)
#define TM_ACR    (1 << 4)
#define TM_BAY    (1 << 5)

#define RM_PERF_MODE_ALWAYS_2D              100
#define RM_PERF_MODE_ALWAYS_3D_LOW_POWER    101
#define RM_PERF_MODE_ALWAYS_3D_PERFORMANCE  102

#define POWER_MIZER_MAX_PERF 1
#define POWER_MIZER_BALANCED 2
#define POWER_MIZER_MAX_BATT 3

var g_MemoryClock_Hz       = null;
var g_GraphicsClock_Hz     = null;
var g_TestMode             = TM_OQA;
var g_Soak_sec             = 0;
var g_FbiCheck             = 15;
var g_RunOnError           = false;
var g_CsumReport           = false;
var g_Chipset              = null;
var g_AgpRate              = null;
var g_SideBandEnabled      = null;
var g_FastWriteEnabled     = null;
var g_DoExternalPowerCheck = true;
var g_CtxSw                = null;
var g_NVClkSlowdown        = null;
var g_DynEngCtrl           = null;
var g_PowerMizerLevel      = null;
var g_DynamicVoltage       = false;
var g_DynamicClocks        = false;
var g_RobustChannels       = 1;
var g_Loops                = 1;
var g_RomVersions          = new Array;
var g_Revision             = null;
var g_SubsytemId           = null;
var g_RecordTime           = false;
var g_IsVerbose            = false;
var g_Display              = null;
var g_DdScalerMode         = Display.DdScalerScaled;
var g_TVOutType            = Display.TvTypeAuto;
var g_TvEncoder            = null;
var g_DisplayMode          = null;
var g_RecordId             = false;
var g_LoadGoldenValues     = true;
var g_OldGold              = false;
var g_UsbPorts             = 2;
var g_UsbAcrPort           = 4;
var g_FwDeviceId           = 0x006e;      // Fw DeviceId in Mcp2
var g_FwDevices            = 2;
var g_FwLoopback           = false;
var g_FwPorts              = 2;
var g_FwMultiPhyLink       = false;
var g_FwSkipAsyRd          = false;
var g_AciMic               = false;
var g_AciFrontMic          = false;
var g_AciCdIn              = false;
var g_AciAuxIn             = false;
var g_AciPhone             = false;
var g_AciSpdifIn           = false;
var g_AciVideoIn           = false;
var g_AciAll               = false;
var g_AciMbo6729           = false;
var g_AciShuttle           = false;
var g_AciA7N8X             = false;
var g_StressDoAta          = true;
var g_StressDoEth          = true;
var g_StressDoAci          = true;
var g_SmbDoPort1           = true;
var g_ShowMemUsed          = false;
var g_DoTmdsCrc            = false;
var g_TunerRfFrequency     = 0;
var g_AdjustDriveSlew      = 1;
var g_RecalDriveSlew       = 1;
var g_SimulatedDdEdid      = null;
var g_ExpRemoteKey         = 26;          // for the sound bay remote controller default to "Num 5"
var g_McpVersion           = 0xFF         // Mcp.INVALID
var g_ShowGold             = false;
var g_IsTcoReboot          = false;
var g_TcoThreadId          = 0;
var g_NvEnetMacLoopback    = false;
var g_NvEnetPhyLoopback    = false;
var g_NvEnetLinkSpeed      = null;
var g_TcomEnetMacLoopback  = false;
var g_TcomEnetPhyLoopback  = false;
var g_TcomEnetLinkSpeed    = null;
var g_AtaClassCode         = [0x01018a];  // default
var g_SerialPorts          = 1;
var g_IsSAta               = false;
var g_IsSpecialUsb2        = false;
var g_RmPerfMode           = RM_PERF_MODE_ALWAYS_3D_PERFORMANCE;
var g_CustomerGen          = false;
var g_IsFramelockAuto      = false;
var g_IsFramelockLock      = true;
var g_MpegInFb             = false;
var g_NoTest               = false;
var g_RequireFOS           = true;
var g_DoBgStress           = false;
var g_GpuCoreVoltage       = null;
var g_OverrideScriptFile   = null;
var g_TempTarget           = null;
var g_NoCompression        = false;
var g_HdtvMode             = null;

#define HIGHEST_TEST_NUMBER 57
#define OS_DOS     (1 << 0)
#define OS_WIN     (1 << 1)
#define OS_MAC     (1 << 2)
#define OS_XOS     (1 << 3)
#define OS_VXWORKS (1 << 4)
#define OS_LINUX   (1 << 5)
#define OS_NONE    (0)
#define OS_ALL     (OS_DOS | OS_WIN | OS_MAC | OS_XOS | OS_VXWORKS | OS_LINUX)

if (("Mac" == OperatingSystem) || ("MacOSX" == OperatingSystem) || ("VxWorks" == OperatingSystem) || ("Linux" == OperatingSystem))
{
   var SystemStressTest = new Object;
   var TagRamTest = new Object;
}

if (OperatingSystem != "Dos")
{
   var VgaTest = new Object;
}

if (Official)
{
   var GLStress         = new Object;
   var Run3DTrace       = new Object;
   var SystemStressTest = new Object;
}

// GPU tests.
var g_GpuTests =
[
   // Configuration
   {Name:"CheckRomVersion",    Number:1,  Object:null,         Gold:false, OperatingSystems:OS_ALL},
   {Name:"CheckRevision",      Number:30, Object:null,         Gold:false, OperatingSystems:OS_ALL},
   {Name:"CheckSubsystemIds",  Number:25, Object:null,         Gold:false, OperatingSystems:OS_ALL},
   {Name:"CheckTvEncoderType", Number:40, Object:null,         Gold:false, OperatingSystems:OS_ALL},
   {Name:"GetDisplayConfig",   Number:33, Object:null,         Gold:false, OperatingSystems:OS_NONE},
   {Name:"ProgramTvRom",       Number:41, Object:null,         Gold:false, OperatingSystems:OS_NONE},
   {Name:"CrystalStrap.Run",   Number:44, Object:null,         Gold:false, OperatingSystems:OS_DOS},
   {Name:"I2CTest.Run",        Number:50, Object:null,         Gold:false, OperatingSystems:OS_NONE},
   {Name:"CheckThermalSanity", Number:31, Object:null,         Gold:false, OperatingSystems:OS_ALL},
   {Name:"PllTest.Run",        Number:56, Object:null,         Gold:false, OperatingSystems:OS_ALL},

   // Memory
   {Name:"MemSize.Run",        Number:17, Object:MemSize,      Gold:false, OperatingSystems:OS_ALL},
   {Name:"Mats.Run",           Number:3,  Object:Mats,         Gold:false, OperatingSystems:OS_ALL},
   {Name:"ByteTest",           Number:18, Object:Mats,         Gold:false, OperatingSystems:OS_NONE},
   {Name:"FastMats.Run",       Number:19, Object:FastMats,     Gold:false, OperatingSystems:OS_ALL},
   {Name:"MarchTest",          Number:52, Object:Mats,         Gold:false, OperatingSystems:OS_NONE},
   {Name:"TagRamTest.Run",     Number:57, Object:TagRamTest,   Gold:false, OperatingSystems:OS_NONE},

   // VGA
   {Name:"VgaTest.Run",        Number:43, Object:VgaTest,      Gold:false, OperatingSystems:OS_NONE},

   // Display
   {Name:"CheckDisplay",       Number:22, Object:null,         Gold:false, OperatingSystems:OS_NONE},
   {Name:"CheckDisplays",      Number:35, Object:null,         Gold:false, OperatingSystems:OS_NONE},
   {Name:"CheckDisplayBar",    Number:42, Object:null,         Gold:false, OperatingSystems:OS_NONE},
   {Name:"CheckDisplayBars",   Number:34, Object:null,         Gold:false, OperatingSystems:OS_NONE},
   {Name:"CheckFpGray",        Number:38, Object:null,         Gold:false, OperatingSystems:OS_NONE},
   {Name:"CheckFpStripes",     Number:45, Object:null,         Gold:false, OperatingSystems:OS_NONE},
   {Name:"StereoTest.Run",     Number:36, Object:StereoTest,   Gold:false, OperatingSystems:OS_NONE},
   {Name:"CheckTvo",           Number:51, Object:null,         Gold:false, OperatingSystems:OS_NONE},
   {Name:"CheckFrameLock",     Number:53, Object:null,         Gold:false, OperatingSystems:OS_NONE},

   // Memory Interface
   {Name:"MemInterface.Run",   Number:4,  Object:MemInterface, Gold:true,  OperatingSystems:OS_MAC},
   {Name:"MemRefresh.Run",     Number:26, Object:MemRefresh,   Gold:false, OperatingSystems:OS_DOS | OS_WIN | OS_VXWORKS | OS_LINUX},
   {Name:"MemPatt.Run",        Number:23, Object:MemPatt,      Gold:true,  OperatingSystems:OS_MAC},

   // DMA and AGP/PCI Bus
   {Name:"Class039.Run",       Number:5,  Object:Class039,     Gold:false, OperatingSystems:OS_ALL},
   {Name:"AgpTest.Run",        Number:29, Object:AgpTest,      Gold:false, OperatingSystems:OS_ALL},

   // Cursor
   {Name:"Class046.Run",       Number:6,  Object:Class046,     Gold:true,  OperatingSystems:OS_NONE},
   {Name:"Class07c.Run",       Number:20, Object:Class07c,     Gold:true,  OperatingSystems:OS_DOS | OS_VXWORKS },

   // 2-D
   {Name:"Class04a.Run",       Number:7,  Object:Class04a,     Gold:true,  OperatingSystems:OS_ALL},
   {Name:"Class05c.Run",       Number:8,  Object:Class05c,     Gold:true,  OperatingSystems:OS_NONE},
   {Name:"Class05d.Run",       Number:55, Object:Class05d,     Gold:true,  OperatingSystems:OS_NONE},
   {Name:"Old05d.Run",         Number:9,  Object:Old05d,       Gold:true,  OperatingSystems:OS_NONE},
   {Name:"Class05f.Run",       Number:10, Object:Class05f,     Gold:true,  OperatingSystems:OS_ALL},
   {Name:"Class061.Run",       Number:11, Object:Class061,     Gold:true,  OperatingSystems:OS_NONE},
   {Name:"Class064.Run",       Number:12, Object:Class064,     Gold:true,  OperatingSystems:OS_NONE},

   // Video
   {Name:"Class038.Run",       Number:13, Object:Class038,     Gold:true,  OperatingSystems:OS_NONE},
   {Name:"Class047.Run",       Number:14, Object:Class047,     Gold:true,  OperatingSystems:OS_NONE},
   {Name:"Class04d.Run",       Number:37, Object:Class04d,     Gold:false, OperatingSystems:OS_NONE},
   {Name:"Class07a.Run",       Number:21, Object:Class07a,     Gold:true,  OperatingSystems:OS_DOS | OS_VXWORKS },
   {Name:"Class077.Run",       Number:15, Object:Class077,     Gold:true,  OperatingSystems:OS_ALL},
   {Name:"Class089.Run",       Number:48, Object:Class089,     Gold:true,  OperatingSystems:OS_ALL},
   {Name:"Class1774.Run",      Number:32, Object:Class1774,    Gold:true,  OperatingSystems:OS_NONE},
   {Name:"Class3174.Run",      Number:49, Object:Class3174,    Gold:true,  OperatingSystems:OS_NONE},

   // 3-D
   {Name:"glr_hwtest",         Number:16, Object:GLRandom,     Gold:true,  OperatingSystems:OS_ALL},
   {Name:"glr_pro10",          Number:28, Object:GLRandom,     Gold:true,  OperatingSystems:OS_NONE},
   {Name:"glr_display",        Number:46, Object:GLRandom,     Gold:true,  OperatingSystems:OS_DOS},
   {Name:"glr_ctxsw",          Number:54, Object:GLRandom,     Gold:false, OperatingSystems:OS_NONE}, // uses glr_hwtest's goldens

   // System Stress
   {Name:"WfMats.Run",         Number:27, Object:WfMats,       Gold:false, OperatingSystems:OS_NONE},
   {Name:"GLStress.Run",       Number:2,  Object:GLStress,     Gold:false, OperatingSystems:OS_NONE},
   {Name:"StressBaseline",     Number:24, Object:null,         Gold:false, OperatingSystems:OS_NONE},
   {Name:"JsPowerPusher",      Number:39, Object:PowerPusher,  Gold:false, OperatingSystems:OS_NONE},

   // Firewire (1394)
   {Name:"RunFwTest",          Number:47, Object:null,         Gold:false, OperatingSystems:OS_NONE},

   // Deleted tests.  We need a placeholder or else g_GpuTests will not be the highest test number
];

// MCP Tests.
var g_McpTests =
[
   {Name:"RunSystemMemoryTest",  Number:16, OperatingSystems:OS_DOS},
   {Name:"RunAtaTest",           Number:1,  OperatingSystems:OS_DOS},
   {Name:"RunAtapiTest",         Number:2,  OperatingSystems:OS_DOS},
   {Name:"RunFloppyTest",        Number:3,  OperatingSystems:OS_DOS},
   {Name:"RunNvEthernetTest",    Number:4,  OperatingSystems:OS_DOS | OS_XOS},
   {Name:"RunSBiosTest",         Number:22, OperatingSystems:OS_DOS},
   {Name:"RunAciTest",           Number:5,  OperatingSystems:OS_DOS},
   {Name:"RunSmbusTest",         Number:6,  OperatingSystems:OS_DOS},
   {Name:"RunApuMats",           Number:24, OperatingSystems:OS_DOS},
   {Name:"RunApuTest",           Number:7,  OperatingSystems:OS_DOS | OS_XOS},
   {Name:"RunUsbTest",           Number:8,  OperatingSystems:OS_DOS},
   {Name:"RunMciTest",           Number:9,  OperatingSystems:OS_DOS},
   {Name:"RunSerialTest",        Number:10, OperatingSystems:OS_DOS},
   {Name:"RunParallelTest",      Number:11, OperatingSystems:OS_DOS},
   {Name:"RunRtcTest",           Number:12, OperatingSystems:OS_DOS},
   {Name:"RunPs2Test",           Number:13, OperatingSystems:OS_DOS},
   {Name:"RunTimerTest",         Number:14, OperatingSystems:OS_DOS},
   {Name:"RunGameTest",          Number:15, OperatingSystems:OS_DOS},
   {Name:"RunTcomEthernetTest",  Number:18, OperatingSystems:OS_DOS},
   {Name:"RunSmbus2Test",        Number:19, OperatingSystems:OS_DOS},
   {Name:"RunUsb2Test",          Number:20, OperatingSystems:OS_DOS},
   {Name:"RunFwTest",            Number:21, OperatingSystems:OS_DOS},
   {Name:"RunAcrEEPROMTest",     Number:23, OperatingSystems:OS_DOS},
   {Name:"RunSystemStressTest",  Number:17, OperatingSystems:OS_DOS},
   {Name:"AcpiTimerTest",        Number:25, OperatingSystems:OS_DOS},
];

var g_AcrTests =
[
   {Name:"RunNvEthernetTest",    Number:1,  OperatingSystems:OS_DOS},
   {Name:"RunAcrAciTest",        Number:2,  OperatingSystems:OS_DOS},
   {Name:"RunAcrMciTest",        Number:3,  OperatingSystems:OS_DOS},
   {Name:"RunAciSpdifTest",      Number:4,  OperatingSystems:OS_DOS},
   {Name:"RunHpnaPhyTest",       Number:5,  OperatingSystems:OS_DOS},
   {Name:"RunAciTest",           Number:6,  OperatingSystems:OS_DOS},
   {Name:"RunTcomEthernetTest",  Number:7,  OperatingSystems:OS_DOS},
   {Name:"RunAcrUsbTest",        Number:8,  OperatingSystems:OS_DOS},
   {Name:"RunAcrUsb2Test",       Number:9,  OperatingSystems:OS_DOS},
   {Name:"RunFwTest",            Number:10, OperatingSystems:OS_DOS},
   {Name:"RunAcrEEPROMTest",     Number:11, OperatingSystems:OS_DOS},
]

var g_BayTests =
[
   {Name:"RunBayAciTest1",       Number:1,  OperatingSystems:OS_DOS},
   {Name:"RunBayAciTest2",       Number:2,  OperatingSystems:OS_DOS},
   {Name:"RunBayAciTest3",       Number:3,  OperatingSystems:OS_DOS},
   {Name:"RunBayRemoteTest",     Number:4,  OperatingSystems:OS_DOS},
   {Name:"RunBayUsbTest",        Number:5,  OperatingSystems:OS_DOS},
]

var g_TestMask                 = new TestMask;
var g_IsGpuInitialized         = false;
var g_IsMcpInitialized         = false;
var g_OriginalMemoryClock_Hz   = null;
var g_OriginalGraphicsClock_Hz = null;
var g_OriginalDisplay          = null;
var g_ExecuteEnd               = true;
var g_TestId                   = null;
var g_FailedTests              = new Array;
var g_TestMemoryClock_Hz       = 0;
var g_TestGraphicsClock_Hz     = 0;

//------------------------------------------------------------------------------
// TestMask - Manage the enabled tests.
//
function TestMask_Set(This, State, TestNumber)
{
   // If TestNumber is undefined set all tests.
   if (undefined == TestNumber)
   {
      for (var i = 1; i <= This.NumTests; ++i)
      {
         This.Tests[i] = State;
      }
   }
   else
   {
      This.Tests[TestNumber] = State;
   }

   return OK;
}

function TestMask_Enable(TestNumber)
{
   return TestMask_Set(this, true, TestNumber);
}

function TestMask_Disable(TestNumber)
{
   return TestMask_Set(this, false, TestNumber);
}

Log.Never("TestMask_Enabled");
function TestMask_Enabled(TestNumber)
{
   return this.Tests[TestNumber];
}

Log.Never("TestMask");
function TestMask()
{
   // Properties
   this.Tests    = new Array
   this.NumTests = HIGHEST_TEST_NUMBER;

   // Methods
   this.Enable  = TestMask_Enable;
   this.Disable = TestMask_Disable;
   this.Enabled = TestMask_Enabled;

   // By default enable all tests.
   this.Enable();

}

//------------------------------------------------------------------------------
// Find the specified test.
// If the search Key is a string search for the 'Name' property.
// If the search Key is a number search for the 'Number' property.
// If the test is not found return null.
//
Log.Never("FindTest");
function FindTest(Tests, Key)
{
   var i;
   if (typeof(Key) == "string")
   {
      for (i in Tests)
      {
         if (Tests[i].Name == Key)
         {
            return Tests[i];
         }
      }
   }
   else if (typeof(Key) == "number")
   {
      for (i in Tests)
      {
         if (Tests[i].Number == Key)
         {
            return Tests[i];
         }
      }
   }

   return null;
}

//------------------------------------------------------------------------------
// Perform GPU test initialization.
//
Log.Never("InitializeGpuTests");
function InitializeGpuTests()
{
   var rc = OK;

   // Give time for the chip to reach thermal equilibrium.
   Sleep(g_Soak_sec * 1000 /* milliseconds */);

   // Setup registry.
   if (OperatingSystem == "Dos")
   {
      if (undefined == Registry.ResourceManager)
      {
         Registry.ResourceManager = new Object;
      }
      Registry.ResourceManager.SoftEDIDs              = 1;
      Registry.ResourceManager.TVOutType              = g_TVOutType;
      Registry.ResourceManager.RmRobustChannels       = Ikos ? 0 : g_RobustChannels;
      Registry.ResourceManager.AdjustDriveSlew        = Ikos ? 0 : g_AdjustDriveSlew;
      Registry.ResourceManager.WatchdogRecalDriveSlew = Ikos ? 0 : g_RecalDriveSlew;
      Registry.ResourceManager.MaxPerfWithPerfMon     = 0;
      Registry.ResourceManager.RMTvoSuppressHDTV      = 0;
      if (g_Chipset          != null) Registry.ResourceManager[g_Chipset]   = 1
      if (g_AgpRate          != null) Registry.ResourceManager.ReqAGPRate   = g_AgpRate;
      if (g_SideBandEnabled  != null) Registry.ResourceManager.EnableAGPSBA = g_SideBandEnabled;
      if (g_FastWriteEnabled != null) Registry.ResourceManager.EnableAGPFW  = g_FastWriteEnabled;
      if (g_CtxSw            != null) Registry.ResourceManager.GrCtxSwMode  = g_CtxSw;
      if (g_NVClkSlowdown    != null) Registry.ResourceManager.EnableNVClkSlowdown = g_NVClkSlowdown;
      if (g_DynEngCtrl       != null) Registry.ResourceManager.dynEngCtrlEnabled   = g_DynEngCtrl;
      if (g_NoCompression)            Registry.ResourceManager.RMDisableCompression = 3;
   }

   // Initialize the hardware.
   CHECK_RC(Gpu.Initialize(g_DoExternalPowerCheck));
   g_IsGpuInitialized = true;

   // Check bus setting overrides.
   if (OperatingSystem == "Dos")
   {
      // Make sure that requested AGP rate, sba, and fw overrides
      // have been successful.
      if ((g_AgpRate != null) && (g_AgpRate != Bus.AgpDataRate))
      {
         Out.Printf(Out.PriHigh, "Cannot set requested AGP data rate.\n");
         return RC.BUS_SET_AGP_DATARATE_FAILED;
      }
      if ((g_SideBandEnabled != null) &&
          ((g_SideBandEnabled && !Bus.IsAgpSbaEnabled) ||
           (!g_SideBandEnabled && Bus.IsAgpSbaEnabled)))
      {
         Out.Printf(Out.PriHigh, "Cannot set requested AGP sideband addressing mode.\n");
         return RC.BUS_SET_AGP_SBA_FAILED;
      }
      if ((g_FastWriteEnabled != null) &&
          ((g_FastWriteEnabled && !Bus.IsAgpFwEnabled) ||
           (!g_FastWriteEnabled && Bus.IsAgpFwEnabled)))
      {
         Out.Printf(Out.PriHigh, "Cannot set requested AGP fast write mode.\n");
         return RC.BUS_SET_AGP_FW_FAILED;
      }
      if(Gpu.DeviceId==Gpu.NV18)
      {
         CHECK_RC( Fw.CreateCrush17() );
         g_FwDeviceId = 0x00b0;// Fw DeviceId in Nv18
      }
   }
   if (OperatingSystem == "Mac")
   {
      // Bug workaround:
      // MacOs9 mods doesn't handle flat-panel scaling correctly on head 0,
      // if two flat panels are connected.  Default to head 1 in this case.
      var Detected = new Array;
      CHECK_RC(Display.GetDetected(Detected));
      if (Detected[0] == 0x30000)
         Display.Select(0x20000);
   }

   if("MacOSX" == OperatingSystem)
   {
      // Can't poll in userspace because OSX is preemptive.
      Dac.UseResmanCrc = 1;

      // WAR: Q37 agp is broken
      if (AppleWAR & 0x2)
         DosOpenGL.Registry.CmdBufMemorySpaceEnables = 2;

   }

   if (Gpu.NumSubdevices)
   {
      // Broadcast multichip configurations should put pushbuffer in FB memory.
      TestConfiguration.PushBufferLocation = Memory.Fb;
   }

   // Set the simulated digital display EDID if necessary.
   if (g_SimulatedDdEdid != null)
   {
      if (g_Display == null)
         g_Display = 0x00010000;

      if ((g_Display != 0x00010000) && (g_Display != 0x00020000))
      {
         Out.Printf(Out.PriHigh, "You can only specify 0x00010000 or 0x00020000 display when using -simulate_dfp.\n");
         return RC.INVALID_DISPLAY_MASK;
      }

      CHECK_RC(Display.SetEdid (g_Display, g_SimulatedDdEdid));
   }

   // Set the display type if necessary.
   if (g_Display != null)
   {
      Out.Printf(Out.PriNormal, "Display = %#010x\n", g_Display);

      // Store original display head and type so we can restore them on exit.
      g_OriginalDisplay = Display.Selected;

      // Select the given display.
      CHECK_RC(Display.Select(g_Display));
   }

   // For DOS only, perform a modeset to initialize parts of the GPU and memory interface.
   if (OperatingSystem == "Dos")
   {
      CHECK_RC(Display.SetMode(640, 480, 32, 60));

      // Go back to text mode for user display.
      if (!Vga.MONOCHROME)
      {
         Vga.SetVesaMode(3);
      }
   }

   if(g_HdtvMode)
      CHECK_RC(Display.SetHdtvMode(g_HdtvMode));

   // Set the test properties.
   CHECK_RC(SetGpuTestProperties());

   // Set test limits for this product.
   CHECK_RC(SetGpuTestLimits());

   // Set the memory type to PCI or AGP.
   // WAR: Q37 Agp is broken. Don't use AGP
   if ( (OperatingSystem == "MacOSX") &&
        (AppleWAR & 0x2) )
   {
      Out.Printf(Out.PriLow, "Using Q37 workaround.\n");
   }
   else
   {
      TestConfiguration.MemoryType         = (Bus.AgpDataRate != 0) ? Memory.Agp : Memory.Pci;
      TestConfiguration.PushBufferLocation = (Bus.AgpDataRate != 0) ? Memory.Agp : Memory.Pci;
   }

   // Set the clocks.
   CHECK_RC(SetGpuClocks());

   if ((OperatingSystem == "Dos") &&
       (0x1039 == Pci.Read16(0,0,0,0)) && // vendor ID == SiS
       (0x0648 == Pci.Read16(0,0,0,2)) && // P4 chipset model 648
       (Gpu.DeviceId >= Gpu.NV20))        // has 3d textures
   {
      // WAR (Work ARound) chipset bug 72783
      // Disable PCIS_WR_BURST in PBUS_DEBUG_1.
      Gpu.RegWr32(0x1084, Gpu.RegRd32(0x1084) & ~(1<<9));
   }

   // Load the golden values if necessary.
   if (g_LoadGoldenValues && (g_TestMode != TM_MCP) && (g_TestMode != TM_ACR) && (g_TestMode != TM_BAY))
   {
      CHECK_RC(LoadGoldenValues());
   }

   CHECK_RC(MemSize.Parse(Boards));

   // Log all the tests.
   for (var i in g_GpuTests)
   {
      Log.Always(g_GpuTests[i].Name);
   }

   // Perform any product specific overrides.
   CHECK_RC(DoGpuOverrides());

   if("MacOSX" == OperatingSystem)
   {
      // Comment these out if you have a driver above 4.5.0a5, or 4.2.0b3
      // and you can run the DAC CRC tests

      // Make Dac Crc tests Unrunnable
      FindTest(g_GpuTests, "Class07a.Run"   ).Name = "WarningMessage";
      FindTest(g_GpuTests, "Class07c.Run"   ).Name = "WarningMessage";

      // Dos Only
      FindTest(g_GpuTests, "TagRamTest.Run"   ).Name = "WarningMessage";

      // WAR: Q37 Agp doen't work. Remove Class039
      if (AppleWAR & 0x2)
      {
         FindTest(g_GpuTests, "Class039.Run"   ).OperatingSystems &= ~OS_MAC;
         FindTest(g_GpuTests, "Class039.Run"   ).Name = "WarningMessage";
      }
   }

   if(Official)
   {
      // Make Engineering-Only tests unrunnable cleanly through JScript
      FindTest(g_GpuTests, "GLStress.Run"       ).Name = "WarningMessage";
      FindTest(g_GpuTests, "JsPowerPusher"      ).Name = "WarningMessage";
      FindTest(g_GpuTests, "glr_ctxsw"          ).Name = "WarningMessage";
      FindTest(g_McpTests, "RunSystemStressTest").Name = "WarningMessage";
   }

   if("Dos" == OperatingSystem)
   {
      /****************************************************************************
      This is a hack for internal/external agp switching
      ****************************************************************************/
      // Check the Chipset and Gpu Agp Version
      if( (Pci.Read08(0,0,0,0x44)==0x1b)
          &&(Pci.Read08(3,0,0,0x48)==0x17) )
      {
         Out.Printf(Out.PriNormal, "...Agp Version mismatch, Force AgpRate to 4x.\n");
         var Command1 = Pci.Read16(0,0,0,0x48);
         // Set chipset Agp: Agp3.0 Rate 4x
         CHECK_RC( Pci.Write16(0,0,0,0x48,(Command1&(0xfff0))|1/*0x0311*/) );
         Command1 = Pci.Read16(0,0,0,0x48);
         if( (Command1&0xf)!=1 )
         {
            Out.Printf(Out.PriHigh, "Cannot set requested AGP data rate.\n");
            return RC.BUS_SET_AGP_DATARATE_FAILED;
         }

         // Set GPU Agp: Agp2.0 Rate 4x
         var Command2 = Pci.Read16(3,0,0,0x4c);
         CHECK_RC( Pci.Write16(3,0,0,0x4c,(Command2&(0xfff0))|4/*0x0314*/) );
         Command2 = Pci.Read16(3,0,0,0x4c);
         if( (Command2&0xf)!=4 )
         {
            Out.Printf(Out.PriHigh, "Cannot set requested AGP data rate.\n");
            return RC.BUS_SET_AGP_DATARATE_FAILED;
         }
      }
      // End of special agp rate handeling
   }

   return OK;
}

//------------------------------------------------------------------------------
function MfgtestStressThread()
{
   // Background thread: run a version of the resman's 3d stress test.
   // The stress test only works on nv3x, on internal mods builds.
   if (Gpu.DeviceId >= Gpu.NV30)
   {
      Class3097.TestConfiguration               = new Object;
      Class3097.TestConfiguration.DisplayWidth  = 640;
      Class3097.TestConfiguration.DisplayHeight = 480;
      Class3097.TestConfiguration.Loops         = 1;
      Class3097.TestConfiguration.TimeoutMs     = 10000;
      Class3097.SkipModeset                     = true;
      Class3097.KeepRunning                     = true;

      Class3097.Run();

   }
   return OK;
}

//------------------------------------------------------------------------------
function MfgtestTemperatureThread()
{
   // Background thread: Turn the fan on/off to achieve the desired temperature.

   // Check that we have a thermal sensor and a controllable fan.
   if (0 == Master.TEMP_PRESENT)
   {
      Out.Printf(Out.PriHigh, "Exiting Temperature thread, no sensor detected.\n");
      return OK;
   }

   if ( (g_TempTarget > Master.TEMP_REMOTE_HIGH_LIMIT) ||
        (g_TempTarget > Master.TEMP_REMOTE_DEFAULT_HIGH_LIMIT) ||
        (g_TempTarget > Master.TEMP_REMOTE_MAX_HIGH_LIMIT) )
   {
      Out.Printf(Out.PriHigh, "Exiting Temperature thread, target temp too high.\n");
      return OK;
   }

   var originalFanSettings = new Array;
   var rc = Bus.GetFanSpeed (originalFanSettings);
   if (rc)
   {
      Out.Printf(Out.PriHigh, "Exiting Temperature thread, fan not software controlled.\n");
      return OK;
   }

   var tempFirst      = Master.TEMP_REMOTE;
   var tempLast       = null;
   var tempMin        = tempFirst;
   var tempMax        = tempFirst;
   var tempNumSamples = 0;
   var tempSum        = 0;

   while (g_TempTarget)
   {
      // Sample.
      tempLast = Master.TEMP_REMOTE;

      // Statistics.
      tempSum += tempLast;
      tempNumSamples++;
      if (tempLast < tempMin)
         tempMin = tempLast;
      if (tempLast > tempMax)
         tempMax = tempLast;

      // Control.
      var fanTarget = 100;  // 100%, max fan speed.
      if (tempLast < g_TempTarget)
         fanTarget = 0;     // 0%, fan off.

      rc = Bus.SetFanSpeed (fanTarget, originalFanSettings[1]);
      if (rc)
      {
         Out.Printf(Out.PriHigh, "Exiting Temperature thread, failed to set fan speed.\n");
         Bus.SetFanSpeed (originalFanSettings[0], originalFanSettings[1]);
         return OK;
      }

      // Wait.
      Sleep(1000);
   }

   // Restore original fan speed.
   Bus.SetFanSpeed (originalFanSettings[0], originalFanSettings[1]);

   // Report.
   Out.Printf(Out.PriHigh,
         "\nTemperature thread: %d,%d,%d,%d,%d,%d (degC first,last,min,max,mean,#samps)\n",
         tempFirst,
         tempLast,
         tempMin,
         tempMax,
         tempSum / tempNumSamples,
         tempNumSamples);

   return OK;
}

//------------------------------------------------------------------------------
// Run the GPU tests.
//
Log.Never("RunGpuTests");
function RunGpuTests()
{
   var rc = OK;
   var OS = GetOS();
   var stressThreadId = 0;
   var tempThreadId = 0;
   var wasMono = Vga.MONOCHROME;

   if (g_DoBgStress && (OperatingSystem == "Dos"))
   {
      // We must be in graphics mode before Class3097 starts.
      // @@@ For non-DOS platforms, this is probably broken, but we can't
      //     fix the problem until DisableUserInterface is nestable.
      Display.SetMode(640,480,32,60);

      // Set "monochrome" mode (i.e. lie to the DOS RM that we have our
      // user interface on the monochrome adaptor, so we won't
      // go back to VGA text mode in between tests).
      Vga.MONOCHROME = true;

      // Start the background 3d stress task.
      stressThreadId = Tasker.CreateThread("bg_stress", MfgtestStressThread);
      Sleep(250);
   }

   if (g_TempTarget && (OperatingSystem == "Dos"))
   {
      tempThreadId = Tasker.CreateThread("bg_temperature", MfgtestTemperatureThread);
   }


   // Run the tests.
   for (var Loop = 1; Loop <= g_Loops; ++Loop)
   {
      if (g_Loops > 1)
      {
         Out.Printf(Out.PriNormal, "\nBegin loop %d of %d:\n", Loop, g_Loops);
      }

      for (var i in g_GpuTests)
      {
         var Test = g_GpuTests[i];

         if (Test.Number > HIGHEST_TEST_NUMBER)
         {
            Out.Printf(Out.PriHigh,
                  "Test %d (%s) will never be run, its number is too high!\n",
                  Test.Number,
                  Test.Name);
         }

         // Check if we need to run this test on this operating system.
         if (Test.OperatingSystems & OS)
         {
            // Check if we need to run this test.
            if (g_TestMask.Enabled(Test.Number))
            {
               // Run the individual test.
               ErrorMap.Test = Test.Number;

               if (!g_CustomerGen)
               {
                  rc = eval(Test.Name + "()");
                  if (g_ShowMemUsed)
                  {
                     Out.Printf(Out.PriNormal,
                        "After loop.test %d.%d:  alloc'ed bytes:%d blocks:%d\n",
                        Loop,
                        Test.Number,
                        Pool.UsedBytes,
                        Pool.UsedBlocks
                     );
                  }

                  if (g_CsumReport && Test.Object && Test.Object.Golden)
                  {
                     var codes = Golden.Codes;
                     if (Test.Object.Golden.Codes)
                        codes = Test.Object.Golden.Codes;

                     var bins  = Golden.NumCodeBins;
                     if (Test.Object.Golden.NumCodeBins)
                        bins = Test.Object.Golden.NumCodeBins;

                     if (codes & Golden.CheckSums)
                     {
                        Out.Printf(Out.PriHigh,
                              "CsumReport test=%s bins=%d bad=%d diffs_argbsd=%d,%d,%d,%d,%d,%d\n",
                              Test.Name,
                              bins,
                              Test.Object.Golden.TotalBinMiscompares,
                              Test.Object.Golden.TotalCheckSumDiffs[0],
                              Test.Object.Golden.TotalCheckSumDiffs[1],
                              Test.Object.Golden.TotalCheckSumDiffs[2],
                              Test.Object.Golden.TotalCheckSumDiffs[3],
                              Test.Object.Golden.TotalCheckSumDiffs[4],
                              Test.Object.Golden.TotalCheckSumDiffs[5] );
                     }
                  }

                  if (rc != OK)
                  {
                     // If Class05f, glr_hwtest, or GLStress failed,
                     // check if this board has marginal memory or frame buffer interface.
                     if
                     (
                           (g_FbiCheck > 0)
                        && ((rc == RC.GOLDEN_VALUE_MISCOMPARE) || (rc == RC.GOLDEN_VALUE_MISCOMPARE_Z) || (rc == RC.GPU_STRESS_TEST_FAILED))
                        && (("Class05f.Run" == Test.Name) || ("glr_hwtest" == Test.Name) || ("GLStress.Run" == Test.Name))
                     )
                     {
                        CheckMemoryAndFBI(Test.Name);
                     }

                     g_FailedTests.push({Loop:Loop, Test:Test.Name, ErrorCode:ErrorMap.Code(rc)})
                     if (!g_RunOnError)
                     {
                        if (stressThreadId)
                        {
                           Class3097.KeepRunning = false;
                           Tasker.Join(stressThreadId);
                           stressThreadId = 0;
                           Vga.MONOCHROME = wasMono;
                        }
                        if (tempThreadId)
                        {
                           g_TempTarget = null;
                           Tasker.Join(tempThreadId);
                           tempThreadId = 0;
                        }
                        return Log.FirstError;
                     }
                  }
               }
               else
               {
                  // g_CustomerGen is true.

                  if (0 < Test.Name.indexOf(".Run"))
                  {
                     // Simple test, not a javascript function.
                     var tname = Test.Name.substring(0, Test.Name.indexOf(".Run"));
                     var L;
                     var skip = Golden.SkipCount;
                     if (undefined != Test.Object.Golden.SkipCount)
                        skip = Test.Object.Golden.SkipCount;
                     if (skip == 0)
                        skip = 1;

                     for (L = 0; L < Test.Object.TestConfiguration.Loops; L += skip)
                     {
                        Golden.MarkRecs(tname, L + skip - 1);
                     }
                  }
                  else
                  {
                     // Not a simple test.  Run the javascript function,
                     // and we will assume it is smart enough to check
                     // its CustomerGen property.
                     Test.Object.CustomerGen = g_CustomerGen;
                     eval(Test.Name + "()");
                  }

               } // run vs. g_CustomerGen
            } // test enabled
         } // correct OS
      } // for each test
   } // for each loop

   Out.Printf(Out.PriNormal, "GPU tests completed.\n");

   if (stressThreadId)
   {
      Class3097.KeepRunning = false;
      Tasker.Join(stressThreadId);
      stressThreadId = 0;
      Vga.MONOCHROME = wasMono;
   }
   if (tempThreadId)
   {
      g_TempTarget = null;
      Tasker.Join(tempThreadId);
      tempThreadId = 0;
   }

   if (g_RunOnError)
   {
      return Log.FirstError;
   }

   return OK;
}

//------------------------------------------------------------------------------
// Perform MCP test initialization.
//
Log.Never("FindMcp");
function FindMcp()
{
   CHECK_RC( Mcp.Initialize() );
   g_McpVersion = Mcp.GetVersion();

   if( g_McpVersion == Mcp.INVALID )
      return RC.INVALID_MCP_VERSION;

   // If is going to test Sata, detect both Ata/SAta controllers
   if( ( (g_McpVersion == Mcp.CK8)
         || (g_McpVersion == Mcp.CK8G)
         || (g_McpVersion == Mcp.MCP2S) )
       &&(g_IsSAta) )
   {
      g_AtaClassCode = [0x01018a, 0x010185];
   }
   // Otherwise only detect regular Ata controller
   else
   {
      g_AtaClassCode = [0x01018a];
   }

   return OK;
}

Log.Never("InitializeMcpTests");
Log.Never("Modem.Initialize");
function InitializeMcpTests()
{
   var rc = OK;
   CHECK_RC( FindMcp() );

   // Creation
   if ("Dos" == OperatingSystem)
   {
      CHECK_RC(Ata.CreateCrush11());
      CHECK_RC(AtaPi.CreateCrush11());
      CHECK_RC(Audio.CreateCrush11());
      CHECK_RC(Modem.CreateCrush11());

      if( g_McpVersion == Mcp.MCP1 )
      {
         CHECK_RC(Usb.CreateCrush11());
         CHECK_RC(Smbus.CreateCrush11());
      }
      // When both Usb1.1 and Usb2.0 exist,don't create.
      // It will be created in the Usb tests.
      else if( g_McpVersion == Mcp.MCP2 )
      {
         CHECK_RC(Fw.CreateCrush17());
         CHECK_RC(Smbus.CreateCrush17());
      }
      else if( ( g_McpVersion == Mcp.CK8 )
               ||( g_McpVersion == Mcp.CK8G )
               ||( g_McpVersion == Mcp.MCP2S ) )
      {
         CHECK_RC(Smbus.CreateCrush17());
      }
      else
      {
         return RC.INVALID_MCP_VERSION;
      }
   }
   CHECK_RC(LegacyEthernet.CreateCrush11());

   // Log all the tests.
   for (var i in g_McpTests)
   {
      Log.Always(g_McpTests[i].Name);
   }
   g_IsMcpInitialized = true;

   return OK;
}

//------------------------------------------------------------------------------
// Run the MCP tests.
//
Log.Never("RunMcpTests");
function RunMcpTests()
{
   #define TESTNUMBER_MCPBASE 1000

   var OS = GetOS();

   // Run the tests.
   for (var Loop = 1; Loop <= g_Loops; ++Loop)
   {
      if (g_Loops > 1)
      {
         Out.Printf(Out.PriNormal, "\nBegin loop %d of %d:\n", Loop, g_Loops);
      }

      for (i in g_McpTests)
      {
         var Test = g_McpTests[i];

         if (Test.Number > HIGHEST_TEST_NUMBER)
         {
            Out.Printf(Out.PriHigh,
                  "Test %d (%s) will never be run, its number is too high!\n",
                  Test.Number,
                  Test.Name);
         }

         // Check if we need to run this test on this operating system.
         if (Test.OperatingSystems & OS)
         {
            // Check if we need to run this test.
            if (g_TestMask.Enabled(Test.Number))
            {
               // Run the individual test.
               ErrorMap.Test = Test.Number * TESTNUMBER_MCPBASE;
               rc = eval(Test.Name + "()");
               if (rc != OK)
               {
                  g_FailedTests.push({Loop:Loop, Test:Test.Name, ErrorCode:ErrorMap.Code(rc)})
                  if (!g_RunOnError)
                  {
                     return Log.FirstError;
                  }
               }
            }
         } // correct OS
      } // for each test
   } // for each loop

   Out.Printf(Out.PriNormal, "MCP tests completed.\n");

   return OK;
}

//------------------------------------------------------------------------------
// Perform ACR test initialization.
//
Log.Never("InitializeAcrTests");
function InitializeAcrTests()
{
   var rc = OK;

   CHECK_RC( FindMcp() );

   // Creation
   CHECK_RC(Audio.CreateCrush11());
   CHECK_RC(Modem.CreateCrush11());
   CHECK_RC(Usb.CreateCrush11());

   if(g_IsMcp2)
   {
      CHECK_RC(Fw.CreateCrush17());
      CHECK_RC(Smbus.CreateCrush17());
   }
   else
      CHECK_RC(Smbus.CreateCrush11());


   // Log all the tests.
   for (var i in g_AcrTests)
   {
      Log.Always(g_AcrTests[i].Name);
   }

   return OK;
}

Log.Never("InitializeBayTests");
function InitializeBayTests()
{
   var rc = OK;

   CHECK_RC( FindMcp() );

   CHECK_RC(Audio.CreateCrush11());
   CHECK_RC(Audio.Initialize());
   Sleep(300);

   if(g_McpVersion == Mcp.MCP1)
      CHECK_RC(Smbus.CreateCrush11());
   else if( (g_McpVersion == Mcp.MCP2)
            || (g_McpVersion == Mcp.CK8)
            || (g_McpVersion == Mcp.CK8G) )
   { // Mcp2, CK8
      CHECK_RC(Smbus.CreateCrush17());
   }
   else
   {
      return RC.INVALID_MCP_VERSION;
   }
   CHECK_RC(Smbus.Initialize());

   CHECK_RC(Usb.CreateCrush11());
   CHECK_RC(Usb.Initialize());

   // Log all the tests.
   for (var i in g_BayTests)
   {
      Log.Always(g_BayTests[i].Name);
   }

   return OK;
}


//------------------------------------------------------------------------------
// Run the ACR tests.
//
Log.Never("RunAcrTests");
function RunAcrTests()
{
   #define TESTNUMBER_ACRBASE 1000000

   var OS = GetOS();

   // Run the tests.
   for (var Loop = 1; Loop <= g_Loops; ++Loop)
   {
      if (g_Loops > 1)
      {
         Out.Printf(Out.PriNormal, "\nBegin loop %d of %d:\n", Loop, g_Loops);
      }

      for (i in g_AcrTests)
      {
         var Test = g_AcrTests[i];

         if (Test.Number > HIGHEST_TEST_NUMBER)
         {
            Out.Printf(Out.PriHigh,
                  "Test %d (%s) will never be run, its number is too high!\n",
                  Test.Number,
                  Test.Name);
         }

         // Check if we need to run this test on this operating system.
         if (Test.OperatingSystems & OS)
         {
            // Check if we need to run this test.
            if (g_TestMask.Enabled(Test.Number))
            {
               // Run the individual test.
               ErrorMap.Test = Test.Number * TESTNUMBER_ACRBASE;
               rc = eval(Test.Name + "()");
               if (rc != OK)
               {
                  g_FailedTests.push({Loop:Loop, Test:Test.Name, ErrorCode:ErrorMap.Code(rc)})
                  if (!g_RunOnError)
                  {
                     return Log.FirstError;
                  }
               }
            }
         } // correct OS
      } // for each test
   } // for each loop

   Out.Printf(Out.PriNormal, "ACR tests completed.\n");

   return OK;
}

Log.Never("RunBayTests");
function RunBayTests()
{
   #define TESTNUMBER_BAYBASE 100000

   var OS = GetOS();

   // Run the tests.
   for (var Loop = 1; Loop <= g_Loops; ++Loop)
   {
      if (g_Loops > 1)
      {
         Out.Printf(Out.PriNormal, "\nBegin loop %d of %d:\n", Loop, g_Loops);
      }

      for (i in g_BayTests)
      {
         var Test = g_BayTests[i];

         if (Test.Number > HIGHEST_TEST_NUMBER)
         {
            Out.Printf(Out.PriHigh,
                  "Test %d (%s) will never be run, its number is too high!\n",
                  Test.Number,
                  Test.Name);
         }

         // Check if we need to run this test on this operating system.
         if (Test.OperatingSystems & OS)
         {
            // Check if we need to run this test.
            if (g_TestMask.Enabled(Test.Number))
            {
               // Run the individual test.
               ErrorMap.Test = Test.Number * TESTNUMBER_BAYBASE;
               rc = eval(Test.Name + "()");
               if (rc != OK)
               {
                  g_FailedTests.push({Loop:Loop, Test:Test.Name, ExitCode:rc})
                  if (!g_RunOnError)
                  {
                     return Log.FirstError;
                  }
               }
            }
         } // correct OS
      } // for each test
   } // for each loop

   Out.Printf(Out.PriNormal, "BAY tests completed.\n");

   return OK;
}

//------------------------------------------------------------------------------
// Perform last minute cleanup.
//
Log.Never("end");
function end()
{
   if (!g_ExecuteEnd)
   {
      return OK;
   }

   if (Gpu.IsInitialized)
   {
      // Allow RM to chenge the clocks
      Master.CLOCK_CHANGE_ALLOWED = true;

      // Allow RM to change GPU core voltage.
      Master.CORE_VOLTAGE_CHANGE_ALLOWED = true;

      // Restore the clocks.
      if (g_OriginalMemoryClock_Hz != null)
      {
         Dac.MEMORY_CLOCK = g_OriginalMemoryClock_Hz;
      }
      if (g_OriginalGraphicsClock_Hz != null)
      {
         Dac.GRAPHICS_CLOCK = g_OriginalGraphicsClock_Hz;
      }

      // Set the PowerMizer level to max performance.
      if (g_PowerMizerLevel != null)
         Master.POWER_MIZER = POWER_MIZER_MAX_PERF;

      // Force the RM into lowest voltage/clocks before exiting.
      Master.PERF_MODE = RM_PERF_MODE_ALWAYS_2D;
   }

   // Restore primary display head.
   if (g_OriginalDisplay != null)
   {
      // The boot head and/or display type are sometimes improperly detected
      // and we force the tests to run on a specified display type using the
      // -display command line argument. In this situation we will fail trying
      // to restore the display head to its original location. Since the failure
      // is harmless, ignore the return code and go on.
      Log.Next = false;
      if (OK == Display.Select(g_OriginalDisplay))
      {
         // Set a default mode to swing display back to original display head.
         Display.SetMode(640, 480, 32, 60);
      }

      if (!Vga.MONOCHROME)
         Vga.SetVesaMode(3);
   }

   if ((Golden.Store == Golden.Action) && (OK == Log.FirstError) && !Official)
   {
      if (g_CustomerGen)
      {
         var CustomerGoldenName =
               g_GoldenFileName.substring(0, g_GoldenFileName.indexOf(".")) +
               ".cus";
         Golden.WriteFile(CustomerGoldenName, true);
      }
      else
      {
         Golden.WriteFile(g_GoldenFileName, false);
      }
   }

   PrintErrors();

   if(g_IsTcoReboot)
   {
      // Set Timer Halt Bit
      CHECK_RC( SysCtrl.TcoTimerHalt() );

      // Clear Reboot bit
      CHECK_RC( SysCtrl.TcoTimerClrReboot() );

      // Stop thread
      g_IsTcoReboot = false;
      Tasker.Join(g_TcoThreadId);
   }

   return OK;
}

//------------------------------------------------------------------------------
// Parse command line arguments.
//
Log.Never("ParseArgv");
function ParseArgv()
{
   if (OperatingSystem == "Dos")
   {
      if (undefined == Registry.ResourceManager)
         Registry.ResourceManager = new Object;
   }

   // Parse the command line arguments.
   var i;
   var DisabledTestMask = false;
   for (i = 1; i < Argv.length; ++i)
   {
      switch (Argv[i])
      {
         case "-mclk":
         {
            g_MemoryClock_Hz = parseFloat(Argv[++i]) * 1e6;
            break;
         }

         case "-nvclk":
         {
            g_GraphicsClock_Hz = parseFloat(Argv[++i]) * 1e6;
            break;
         }

         case "-perfmode":
         {
            switch (parseInt(Argv[++i]))
            {
               case 1:
                  g_RmPerfMode = RM_PERF_MODE_ALWAYS_2D;
                  break;
               case 2:
                  g_RmPerfMode = RM_PERF_MODE_ALWAYS_3D_LOW_POWER;
                  break;
               case 3:
                  g_RmPerfMode = RM_PERF_MODE_ALWAYS_3D_PERFORMANCE;
                  break;
               default:
                  Out.Printf(Out.PriHigh, "-perfmode argument must be 1, 2, or 3(default).\n")
                  return RC.BAD_COMMAND_LINE_ARGUMENT;
            }
            break;
         }

         case "-gpu":
         case "-mfg":  // Support legacy switch until everyone get used to -gpu.
         {
            if (g_TestMode != TM_OQA)
            {
               Out.Printf(Out.PriHigh, "Test mode was specified more than once.\n")
               return RC.BAD_COMMAND_LINE_ARGUMENT;
            }
            g_TestMode = TM_GPU;
            break;
         }

         case "-engr":
         {
            if (Official)
            {
               Out.Printf(Out.PriHigh, "-engr can only be used in engineering build.\n")
               return RC.BAD_COMMAND_LINE_ARGUMENT;
            }
            if (g_TestMode != TM_OQA)
            {
               Out.Printf(Out.PriHigh, "Test mode was specified more than once.\n")
               return RC.BAD_COMMAND_LINE_ARGUMENT;
            }
            g_TestMode = TM_ENGR;
            break;
         }

         case "-mcp":
         {
            g_TestMode = TM_MCP;
            break;
         }

         case "-acr":
         {
            g_TestMode = TM_ACR;
            break;
         }

         case "-bay":
         {
            g_TestMode = TM_BAY;
            break;
         }

         case "-rmt_key":
         {
            g_ExpRemoteKey = parseInt(Argv[++i]);
            break;
         }

         case "-test":
         {
            var TestNum = parseInt(Argv[++i]);
            var Test = null;

            if( (g_TestMode == TM_GPU)||(g_TestMode == TM_ENGR) )
            {
               if(TestNum>g_GpuTests.length)
               {
                  Out.Printf(Out.PriHigh, "Invalid GpuTest Number %i.\n", TestNum);
                  return RC.MFGTEST_INVALID_TEST_NUMBER;
               }
               Test = FindTest(g_GpuTests, TestNum);
            }

            else if(g_TestMode == TM_MCP)
            {
               if(TestNum>g_McpTests.length)
               {
                  Out.Printf(Out.PriHigh, "Invalid McpTest Number %i.\n", TestNum);
                  return RC.MFGTEST_INVALID_TEST_NUMBER;
               }
               Test = FindTest(g_McpTests, TestNum);
            }

            else if(g_TestMode == TM_ACR)
            {
               if(TestNum>g_AcrTests.length)
               {
                  Out.Printf(Out.PriHigh, "Invalid AcrTest Number %i.\n", TestNum);
                  return RC.MFGTEST_INVALID_TEST_NUMBER;
               }
               Test = FindTest(g_AcrTests, TestNum);
            }

            else if(g_TestMode == TM_BAY)
            {
               if(TestNum>g_BayTests.length)
               {
                  Out.Printf(Out.PriHigh, "Invalid BayTest Number %i.\n", TestNum);
                  return RC.MFGTEST_INVALID_TEST_NUMBER;
               }
               Test = FindTest(g_BayTests, TestNum);
            }
            else
            {
               Out.Printf(Out.PriHigh, "Invalid Test Mode %i.\n", g_TestMode);
               return RC.MFGTEST_INVALID_TEST_MODE;
            }

            if(!Test)
            {
               Out.Printf(Out.PriHigh, "Cannot Find Test %i.\n", TestNum);
               return RC.MFGTEST_INVALID_TEST_NUMBER;
            }

            // Force OperatingSystems to OS_ALL.
            if( (  g_TestMode == TM_GPU)
                ||(g_TestMode == TM_ENGR)
                ||(g_TestMode == TM_MCP)
                ||(g_TestMode == TM_ACR)
                ||(g_TestMode == TM_BAY) )
            {
               Test.OperatingSystems = OS_ALL;
            }

            if (!DisabledTestMask)
            {
               g_TestMask.Disable();
               DisabledTestMask = true;
            }
            g_TestMask.Enable(TestNum);

            break;
         }

         case "-notest":
         {
            g_NoTest = true;
            break;
         }

         case "-skip":
         {
            var TestNum = parseInt(Argv[++i]);
            g_TestMask.Disable(TestNum);
            break;
         }

         case "-soak":
         {
            g_Soak_sec = parseInt(Argv[++i]);
            break;
         }

         case "-fbi_check":
         {
            g_FbiCheck = parseInt(Argv[++i]);
            if ((g_FbiCheck < 0) || (g_FbiCheck > 50))
            {
               Out.Printf(Out.PriHigh, "-fbi_check percent can be 0-50.\n")
               return RC.BAD_COMMAND_LINE_ARGUMENT;
            }

            break;
         }

         case "-run_on_error":
         {
            g_RunOnError = true;
            break;
         }

         case "-csum_report":
         {
            g_CsumReport = true;
            break;
         }

         case "-chipset":
         {
            switch (Argv[++i])
            {
               case "amd":
               {
                  g_Chipset = "UseAmdCompat";
                  break;
               }

               case "intel":
               {
                  g_Chipset = "UseIntelCompat";
                  break;
               }

               case "nvidia":
               {
                  g_Chipset = "UseNvidiaCompat";
                  break;
               }

               case "rcc":
               {
                  g_Chipset = "UseRccCompat";
                  break;
               }

               case "via":
               {
                  g_Chipset = "UseViaCompat";
                  break;
               }

               case "ali":
               {
                  g_Chipset = "UseAliCompat";
                  break;
               }

               case "sis":
               {
                  g_Chipset = "UseSisCompat";
                  break;
               }

               case "ati":
               {
                  g_Chipset = "UseAtiCompat";
                  break;
               }

               case "transmeta":
               {
                  g_Chipset = "UseTransmetaCompat";
                  break;
               }

               case "agp35":
               {
                  g_Chipset = "UseAGP35Compat";
                  break;
               }

               case "k8":
               {
                  g_Chipset = "UseK8Compat";
                  break;
               }

               default:
               {
                  Out.Printf(Out.PriHigh, "Unsupported chipset type.\n")
                  return RC.BAD_COMMAND_LINE_ARGUMENT;
               }
            }
            break;
         }

         case "-pci":
         {
            g_AgpRate = 0;
            break;
         }

         case "-agp1x":
         {
            g_AgpRate = 1;
            break;
         }

         case "-agp2x":
         {
            g_AgpRate = 2;
            break;
         }

         case "-agp4x":
         {
            g_AgpRate = 4;
            break;
         }

         case "-agp8x":
         {
            g_AgpRate = 8;
            break;
         }

         case "-sb":
         {
            g_SideBandEnabled = true;
            break;
         }

         case "-no_sb":
         {
            g_SideBandEnabled = false;
            break;
         }

         case "-fw":
         {
            g_FastWriteEnabled = true;
            break;
         }

         case "-no_fw":
         {
            g_FastWriteEnabled = false;
            break;
         }

         case "-no_ext_power":
         {
            g_DoExternalPowerCheck = false;
            break;
         }

         case "-ctx":
         {
            g_CtxSw = parseInt(Argv[++i]);
            break;
         }

         case "-nvclk_slowdown":
         {
            g_NVClkSlowdown = (parseInt(Argv[++i]) != 0);
            break;
         }

         case "-dyn_eng_ctrl":
         {
            g_DynEngCtrl = parseInt(Argv[++i]);
            break;
         }

         case "-power_mizer":
         {
            switch (parseInt(Argv[++i]))
            {
               case 1:
                  g_PowerMizerLevel = POWER_MIZER_MAX_BATT;
                  break;
               case 2:
                  g_PowerMizerLevel = POWER_MIZER_BALANCED;
                  break;
               case 3:
                  g_PowerMizerLevel = POWER_MIZER_MAX_PERF;
                  break;
               default:
                  Out.Printf(Out.PriHigh, "-power_mizer argument must be 1, 2, or 3.\n")
                  return RC.BAD_COMMAND_LINE_ARGUMENT;
            }
            break;
         }

         case "-dynamic_voltage":
         {
            g_DynamicVoltage = true;
            break;
         }

         case "-dynamic_clocks":
         {
            g_DynamicClocks = true;
            break;
         }

         case "-no_rc":
         {
            g_RobustChannels = 0;
            break;
         }

         case "-no_compress":
         {
            g_NoCompression = true;
            break;
         }

         case "-loops":
         {
            g_Loops = parseInt(Argv[++i]);
            break;
         }

         case "-glr_pp":
         {
            if (Official)
            {
               Out.Printf(Out.PriHigh, "Bad argument '%s'.\n", Argv[i]);
               return RC.BAD_COMMAND_LINE_ARGUMENT;
            }
            else
            {
               GLRandom.PowerPusherMs = parseInt(Argv[++i]);
            }
            break;
         }

         case "-rom":
         {
            g_RomVersions.push(Argv[++i]);
            break;
         }

         case "-revision":
         {
            g_Revision = parseInt(Argv[++i], 16);
            break;
         }

         case "-subsystem":
         {
            g_SubsytemId =
            {
               Vid : parseInt(Argv[++i], 16),
               Did : parseInt(Argv[++i], 16)
            }
            break;
         }

         case "-time":
         {
            g_RecordTime = true;
            break;
         }

         case "-verbose":
         {
            g_IsVerbose = true;
            break;
         }

         case "-display":
         {
            g_Display = parseInt(Argv[++i], 16);

            // Only one display can be selected.
            if ((g_Display & (g_Display - 1)) != 0)
            {
               Out.Printf(Out.PriHigh, "You can only select one active display.\n");
               return RC.INVALID_DISPLAY_MASK;
            }

            break;
         }

         case "-dd_scaler_mode":
         {
            switch (Argv[++i])
            {
               case "native":
                  g_DdScalerMode = Display.DdScalerNative;
                  break;

               case "scaled":
                  g_DdScalerMode = Display.DdScalerScaled;
                  break;

               case "centered":
                  g_DdScalerMode = Display.DdScalerCentered;
                  break;

               default:
               {
                  Out.Printf(Out.PriHigh, "-dd_scaler_mode can be: native, scaled or centered.\n");
                  return RC.BAD_COMMAND_LINE_ARGUMENT;
               }
            }

            break;
         }

         case "-tv":
         {
            Out.Printf(Out.PriHigh, "-tv is archaic.  Use -tv_in and/or -tv_out.\n");
            return RC.BAD_COMMAND_LINE_ARGUMENT;
         }

         case "-tv_out":
         {
            switch (Argv[++i])
            {
               case "auto":
               {
                  g_TVOutType = Display.TvTypeAuto;
                  break;
               }
               case "composite":
               {
                  g_TVOutType = Display.TvTypeComposite;
                  break;
               }
               case "svideo":
               {
                  g_TVOutType = Display.TvTypeSvideo;
                  break;
               }
               case "480i":
               {
                  g_HdtvMode  = Display.HD480i;
                  break;
               }
               case "480p":
               {
                  g_HdtvMode  = Display.HD480p;
                  break;
               }
               case "720p":
               {
                  g_HdtvMode  = Display.HD720p;
                  break;
               }
               case "1080i":
               {
                  g_HdtvMode  = Display.HD1080i;
                  break;
               }
               case "1080p":
               {
                  g_HdtvMode  = Display.HD1080p;
                  break;
               }
               default:
               {
                  Out.Printf(Out.PriHigh,
                     "-tv_out connector can be: auto, composite, svideo " +
                     "480i, 480p, 720p, 1080i, 1080p\n");
                  return RC.BAD_COMMAND_LINE_ARGUMENT;
               }
            }
            break;
         }

         case "-tv_in":
         {
            switch (Argv[++i])
            {
               case "auto":
                  Class04d.TvInputType = Display.TvTypeAuto;
                  break;

               case "composite":
                  Class04d.TvInputType = Display.TvTypeComposite;
                  break;

               case "svideo":
                  Class04d.TvInputType = Display.TvTypeSvideo;
                  break;

               case "composite_aux":
                  Class04d.TvInputType = Display.TvTypeComposite;
                  Class04d.UseAuxInputs= 1;
                  break;

               case "svideo_aux":
                  Class04d.TvInputType = Display.TvTypeSvideo;
                  Class04d.UseAuxInputs= 1;
                  break;

               default:
               {
                  Out.Printf(Out.PriHigh,
                     "-tv_in connector can be: auto, composite, svideo, " +
                     "composite_aux or svideo_aux.\n");
                  return RC.BAD_COMMAND_LINE_ARGUMENT;
               }
            }
            break;
         }

         case "-tv_encoder":
         {
            g_TvEncoder = Argv[++i];
            break;
         }

         case "-mode":
         {
            var w = parseInt(Argv[++i]);
            var h = parseInt(Argv[++i]);
            var d = parseInt(Argv[++i]);
            var r = parseInt(Argv[++i]);
            g_DisplayMode = {Width : w, Height : h, Depth : d, RefreshRate : r};
            break;
         }

         case "-matsinfo":
         {
            VgaTest.AutoStatus  = 1;
            WfMats.AutoStatus   = 1;
            Mats.AutoStatus     = 1;
            FastMats.AutoStatus = 1;
            break;
         }

         case "-memsize":
         {
            MemSize.SizeOverride = parseInt(Argv[++i]);
            break;
         }

         case "-id":
         {
            g_RecordId = true;
            break;
         }
         case "-notiled":
         {
            TestConfiguration.UseTiledSurface = false;
            break;
         }

         case "-screen_off":
         {
            TestConfiguration.DisableCrt = true;
            break;
         }

         case "-display_config":
         {
            // Disable all tests but GetDisplayConfig.
            var Test = FindTest(g_GpuTests, "GetDisplayConfig");
            g_TestMask.Disable();
            g_TestMask.Enable(Test.Number);
            Test.OperatingSystems = OS_ALL;
            g_TestMode = TM_GPU;
            g_LoadGoldenValues = false;
            break;
         }

         case "-check_display":
         {
            // Disable all tests but CheckDisplay.
            var Test = FindTest(g_GpuTests, "CheckDisplay");
            g_TestMask.Disable();
            g_TestMask.Enable(Test.Number);
            Test.OperatingSystems = OS_ALL;
            g_TestMode = TM_GPU;
            g_LoadGoldenValues = false;
            break;
         }

         case "-check_displays":
         {
            // Disable all tests but CheckDisplays.
            var Test = FindTest(g_GpuTests, "CheckDisplays");
            g_TestMask.Disable();
            g_TestMask.Enable(Test.Number);
            Test.OperatingSystems = OS_ALL;
            g_TestMode = TM_GPU;
            g_LoadGoldenValues = false;
            break;
         }

         case "-check_display_bar":
         {
            // Disable all tests but CheckDisplayBar.
            var Test = FindTest(g_GpuTests, "CheckDisplayBar");
            g_TestMask.Disable();
            g_TestMask.Enable(Test.Number);
            Test.OperatingSystems = OS_ALL;
            g_TestMode = TM_GPU;
            g_LoadGoldenValues = false;
            break;
         }

         case "-check_display_bars":
         {
            // Disable all tests but CheckDisplayBars.
            var Test = FindTest(g_GpuTests, "CheckDisplayBars");
            g_TestMask.Disable();
            g_TestMask.Enable(Test.Number);
            Test.OperatingSystems = OS_ALL;
            g_TestMode = TM_GPU;
            g_LoadGoldenValues = false;
            break;
         }

         case "-check_fp_gray":
         {
            // Disable all tests but CheckFpGray
            var Test = FindTest(g_GpuTests, "CheckFpGray");
            g_TestMask.Disable();
            g_TestMask.Enable(Test.Number);
            Test.OperatingSystems = OS_ALL;
            g_TestMode = TM_GPU;
            g_LoadGoldenValues = false;
            break;
         }

         case "-check_fp_stripes":
         {
            // Disable all tests but CheckFpStripes
            var Test = FindTest(g_GpuTests, "CheckFpStripes");
            g_TestMask.Disable();
            g_TestMask.Enable(Test.Number);
            Test.OperatingSystems = OS_ALL;
            g_TestMode = TM_GPU;
            g_LoadGoldenValues = false;
            break;
         }

         case "-stereo":
         {
            // Disable all tests but CheckStereo.
            var Test = FindTest(g_GpuTests, "StereoTest.Run");
            g_TestMask.Disable();
            g_TestMask.Enable(Test.Number);
            Test.OperatingSystems = OS_ALL;
            g_TestMode = TM_GPU;
            g_LoadGoldenValues = false;
            break;
         }

         case "-tvo":
         {
            // Disable all tests but CheckTvo.
            var Test = FindTest(g_GpuTests, "CheckTvo");
            g_TestMask.Disable();
            g_TestMask.Enable(Test.Number);
            Test.OperatingSystems = OS_ALL;
            g_TestMode = TM_GPU;
            g_LoadGoldenValues = false;
            break;
         }

         case "-framelock_skip_lock":
         {
            g_IsFramelockLock = false;
            break;
         }

         case "-framelock_auto":
         {
            g_IsFramelockAuto = true;
         }
         case "-framelock":
         {
            // Disable all tests but CheckTvo.
            var Test = FindTest(g_GpuTests, "CheckFrameLock");
            g_TestMask.Disable();
            g_TestMask.Enable(Test.Number);
            Test.OperatingSystems = OS_ALL;
            g_TestMode = TM_GPU;
            g_LoadGoldenValues = false;
            break;
         }

         case "-vivo":
         {
            if (!DisabledTestMask)
            {
               g_TestMask.Disable();
               DisabledTestMask = true;
            }

            var Test = FindTest(g_GpuTests, "Class04d.Run");
            g_TestMask.Enable(Test.Number);
            Test.OperatingSystems = OS_ALL;
            g_TestMode = TM_GPU;
            g_LoadGoldenValues = false;
            Class04d.DoVivo = true;
            break;
         }

         case "-progtv":
         {
            if (!DisabledTestMask)
            {
               g_TestMask.Disable();
               DisabledTestMask = true;
            }

            var Test = FindTest(g_GpuTests, "ProgramTvRom");
            g_TestMask.Enable(Test.Number);
            Test.OperatingSystems = OS_ALL;
            g_TestMode = TM_GPU;
            g_LoadGoldenValues = false;
            break;
         }

         case "-tuner":
         {
            if (!DisabledTestMask)
            {
               g_TestMask.Disable();
               DisabledTestMask = true;
            }

            var Test = FindTest(g_GpuTests, "Class04d.Run");
            g_TestMask.Enable(Test.Number);
            Test.OperatingSystems = OS_ALL;
            g_TestMode = TM_GPU;
            g_LoadGoldenValues = false;
            Class04d.DoTuner = true;
            break;
         }

         case "-rf":
         {
            Frequency = parseFloat(Argv[++i]);
            // SetRfFrequency is defined in progtv.js
            g_TunerRfFrequency = Math.floor(Frequency * 1000000);
            break;
         }

         case "-video_in":
         {
            if (!DisabledTestMask)
            {
               g_TestMask.Disable();
               DisabledTestMask = true;
            }

            var Test = FindTest(g_GpuTests, "Class04d.Run");
            g_TestMask.Enable(Test.Number);
            Test.OperatingSystems = OS_ALL;
            g_TestMode = TM_GPU;
            g_LoadGoldenValues = false;
            Class04d.DoDigitize = true;
            break;
         }

         case "-usb_ports":
         {
            g_UsbPorts = parseInt(Argv[++i]);
            break;
         }

         case "-usb_acr_port":
         {
            g_UsbAcrPort = parseInt(Argv[++i]);
            break;
         }

         case "-fw_devices":
         {
            g_FwDevices = parseInt(Argv[++i]);
            break;
         }

         case "-fw_loopback":
         {
            g_FwLoopback = true;
            break;
         }

         case "-fw_multi_phy_links":
         {
            g_FwMultiPhyLink = true;
            break;
         }

         case "-fw_ports":
         {
            g_FwPorts = parseInt(Argv[++i]);
            break;
         }

         case "-fw_skip_asyrd":
         {
            g_FwSkipAsyRd = true;
            break;
         }

         case "-audio_mic":
         {
            g_AciMic = true;
            break;
         }

         case "-audio_front_mic":
         {
            g_AciFrontMic = true;
            break;
         }

         case "-audio_cdin":
         {
            g_AciCdIn = true;
            break;
         }

         case "-audio_auxin":
         {
            g_AciAuxIn = true;
            break;
         }

         case "-audio_phone":
         {
            g_AciPhone = true;
            break;
         }

         case "-audio_videoin":
         {
            g_AciVideoIn = true;
            break;
         }
         case "-audio_spdifin":
         {
            g_AciSpdifIn = true;
            break;
         }

         case "-audio_all":
         {
            g_AciAll = true;
            break;
         }

         case "-audio_mbo_6729":
         {
            g_AciMbo6729 = true;
            break;
         }

         case "-audio_shuttle":
         {
            g_AciShuttle = true;
            break;
         }
         case "-audio_a7n8x":

         {
            g_AciA7N8X = true;
            break;
         }

         case "-nvmac_mac":
         {
            g_NvEnetMacLoopback = true;
            break;
         }

         case "-nvmac_phy":
         {
            g_NvEnetPhyLoopback = true;
            break;
         }

         case "-nvmac_10mbps":
         {
            g_NvEnetLinkSpeed = 10000000;
            break;
         }

         case "-nvmac_100mbps":
         {
            g_NvEnetLinkSpeed = 100000000;
            break;
         }

         case "-tcmac_mac":
         {
            g_TcomEnetMacLoopback = true;
            break;
         }

         case "-tcmac_phy":
         {
            g_TcomEnetPhyLoopback = true;
            break;
         }

         case "-tcmac_10mbps":
         {
            g_TcomEnetLinkSpeed = 10000000;
            break;
         }

         case "-tcmac_100mbps":
         {
            g_TcomEnetLinkSpeed = 100000000;
            break;
         }

         case "-sst_skip_ata":
         {
            g_StressDoAta = false;
            break;
         }

         case "-sst_skip_ethernet":
         {
            g_StressDoEth = false;
            break;
         }

         case "-sst_skip_aci":
         {
            g_StressDoAci = false;
            break;
         }

         case "-smb_skip_port1":
         {
            g_SmbDoPort1 = false;
            break;
         }

         case "-tco":
         {
            g_IsTcoReboot = true;
            break;
         }

         case "-arg":
         {
            // Pass to wrapper scripts.
            ++i;
            break;
         }

         case "-no_gold":
         {
            g_LoadGoldenValues = false;
            break;
         }

         case "-old_gold":
         {
            g_OldGold = true;
            break;
         }

         case "-show_gold":
         {
            g_ShowGold = true;
            g_TestMask.Disable();
            DisabledTestMask = true;
            g_TestMode = TM_GPU;
            g_LoadGoldenValues = true;
            break;
         }

         case "-simulate_dfp":
         {
            if (OperatingSystem != "Dos")
            {
               Out.Printf(Out.PriHigh, "-simulate_dfp is only supported under DOS.\n");
               return RC.BAD_COMMAND_LINE_ARGUMENT;
            }

            CHECK_RC(Import("edid.js"));
            if ((g_SimulatedDdEdid = GetEdid(Argv[++i])) == null)
            {
               Out.Printf(Out.PriHigh, "EDID not found for requested flat panel resolution.\n");
               return RC.BAD_COMMAND_LINE_ARGUMENT;
            }

            // Specify that 2 panels are attached.
            Registry.ResourceManager.DevicesConnected = 0x00030000;

            break;
         }

         case "-simulate_tv":
         {
            if (OperatingSystem != "Dos")
            {
               Out.Printf(Out.PriHigh, "-simulate_tv is only supported under DOS.\n");
               return RC.BAD_COMMAND_LINE_ARGUMENT;
            }

            Registry.ResourceManager.DevicesConnected = 0x00000100;
            break;
         }

         case "-tmds_crc":
         {
            g_DoTmdsCrc = true;
            break;
         }

         case "-no_drvslew":
         {
            g_AdjustDriveSlew = 0;
            break;
         }

         case "-no_recal_drvslew":
         {
            g_RecalDriveSlew = 0;
            break;
         }

         case "-serial_ports":
         {
            g_SerialPorts = parseInt(Argv[++i]);
            break;
         }
         case "-sata":
         {
            g_IsSAta = true;
            break;
         }
         case "-usb2_special":
         {
            g_IsSpecialUsb2 = true;
            break;
         }
         case "-stress_baseline":
         {
            // Disable all tests but StressBaseline.
            var Test = FindTest(g_GpuTests, "StressBaseline");
            g_TestMask.Disable();
            g_TestMask.Enable(Test.Number);
            Test.OperatingSystems = OS_ALL;
            g_TestMode = TM_GPU;
            g_LoadGoldenValues = false;
            break;
         }
         case "-mpeg_in_fb":
         {
            g_MpegInFb = true;
            break;
         }
         case "-no_require_fos":
         {
            g_RequireFOS = false;
            break;
         }

         case "-bgstress":
         {
            if (OperatingSystem == "Dos")
            {
               g_DoBgStress = true;
            }
            else
            {
               Out.Printf(Out.PriHigh, "Bad argument '%s'.\n", Argv[i]);
               return RC.BAD_COMMAND_LINE_ARGUMENT;
            }
            break;
         }


         case "-temp":
         {
            if (OperatingSystem == "Dos")
            {
               g_TempTarget = parseInt(Argv[++i]);
            }
            else
            {
               Out.Printf(Out.PriHigh, "Bad argument '%s'.\n", Argv[i]);
               return RC.BAD_COMMAND_LINE_ARGUMENT;
            }
            break;
         }

         case "-override":
         {
            g_OverrideScriptFile = Argv[++i];
            break;
         }

         case "-help":
         case "-h":
         case "-?":
         {
            g_ExecuteEnd = false;
            Out.Printf(Out.PriHigh, Argv[0] + " command line options:\n");
            Out.Printf(Out.PriHigh, "   -mclk MHz             Memory clock in MHz.\n");
            Out.Printf(Out.PriHigh, "   -nvclk MHz            NV (graphics) clock in MHz.\n");
            Out.Printf(Out.PriHigh, "   -perfmode number      Set performance mode for \"silent running\": 1, 2, or 3(default).\n");
            Out.Printf(Out.PriHigh, "   -gpu                  Run the gpu manufacture tests.\n");
            Out.Printf(Out.PriHigh, "   -mcp                  Run the mcp manufacture tests.\n");
            if (!Official)
               Out.Printf(Out.PriHigh, "   -engr                 Run the engr manufacture tests.\n");
            Out.Printf(Out.PriHigh, "   -acr                  Run the acr manufacture tests.\n");
            Out.Printf(Out.PriHigh, "   -bay                  Run the SoundBay manufacture tests.\n");
            Out.Printf(Out.PriHigh, "   -test number          Run the specified test(s).\n");
            Out.Printf(Out.PriHigh, "   -skip number          Skip the specified test(s).\n");
            Out.Printf(Out.PriHigh, "   -notest               Get ready to run tests, but don't actually run them.\n");
            Out.Printf(Out.PriHigh, "   -soak seconds         Soak the chip for given 'seconds'.\n");
            Out.Printf(Out.PriHigh, "   -fbi_check percent    Check for marginal memory and FBI.\n");
            Out.Printf(Out.PriHigh, "   -run_on_error         Continue running if error occurs.\n");
            Out.Printf(Out.PriHigh, "   -csum_report          Report checksum differences for quals.\n");
            Out.Printf(Out.PriHigh, "   -chipset type         Configure chipset as a generic 'type'.\n");
            Out.Printf(Out.PriHigh, "   -pci                  Use PCI transfer.\n");
            Out.Printf(Out.PriHigh, "   -agp1x                Use AGP 1x transfer.\n");
            Out.Printf(Out.PriHigh, "   -agp2x                Use AGP 2x transfer.\n");
            Out.Printf(Out.PriHigh, "   -agp4x                Use AGP 4x transfer.\n");
            Out.Printf(Out.PriHigh, "   -agp8x                Use AGP 8x transfer.\n");
            Out.Printf(Out.PriHigh, "   -sb                   Enable AGP side band.\n");
            Out.Printf(Out.PriHigh, "   -no_sb                Disable AGP side band.\n");
            Out.Printf(Out.PriHigh, "   -fw                   Enable AGP fast write.\n");
            Out.Printf(Out.PriHigh, "   -no_fw                Disable AGP fast write.\n");
            Out.Printf(Out.PriHigh, "   -no_ext_power         Do not check if external power is connected.\n");
            Out.Printf(Out.PriHigh, "   -ctx mode             Set context switching mode. 1=hybrid, 2=hw, 3=sw\n");
            Out.Printf(Out.PriHigh, "   -nvclk_slowdown state Enable (state = 1) or disable (state = 0) NV clock slowdown.\n");
            Out.Printf(Out.PriHigh, "   -dyn_eng_ctrl state   Enable (state = 1 or 2) or disable (state = 0) dynamic engine control.\n");
            Out.Printf(Out.PriHigh, "   -power_mizer level    Set PowerMizer 'level'.\n");
            Out.Printf(Out.PriHigh, "   -dynamic_clocks       Allow dynamic clock adjustment in RM.\n");
            Out.Printf(Out.PriHigh, "   -dynamic_voltage      Enable dynamic core voltage.\n");
            Out.Printf(Out.PriHigh, "   -no_rc                Disable robust channels.\n");
            Out.Printf(Out.PriHigh, "   -no_compress          Disable FB compression.\n");
            Out.Printf(Out.PriHigh, "   -loops count          Loop the tests 'count' times.\n");
            Out.Printf(Out.PriHigh, "   -glr_pp ms            Use PowerPusher in GLRandom for the specified interval.\n");
            Out.Printf(Out.PriHigh, "   -rom version          Check the rom 'version'.\n");
            Out.Printf(Out.PriHigh, "   -revision id          Check the NVIDIA chip revision.\n");
            Out.Printf(Out.PriHigh, "   -subsystem vid did    Check the subsystem vendor and device ID's.\n");
            Out.Printf(Out.PriHigh, "   -time                 Record duration of tests.\n");
            Out.Printf(Out.PriHigh, "   -verbose              Run the tests in verbose mode.\n");
            Out.Printf(Out.PriHigh, "   -display display      Run the tests on the specified 'display'.\n");
            Out.Printf(Out.PriHigh, "   -dd_scaler_mode mode  Digital display scaler mode; native, scaled or centered.\n");
            Out.Printf(Out.PriHigh, "   -tv_out connector     Type of TV-out connector; auto, composite or svideo,\n");
            Out.Printf(Out.PriHigh, "                            composite_aux or svideo_aux.\n");
            Out.Printf(Out.PriHigh, "   -tv_in connector      Type of TV-in connector; auto, composite or svideo.\n");
            Out.Printf(Out.PriHigh, "   -tv_encoder type      Type of TV encoder.\n");
            Out.Printf(Out.PriHigh, "   -mode w h d r         Set display mode 'w'idth, 'h'eight, 'd'epth, and 'r'efresh rate.\n");
            Out.Printf(Out.PriHigh, "   -matsinfo             If a mats-derived test fails, print out more info.\n");
            Out.Printf(Out.PriHigh, "   -memsize size         Test for explicit fb 'size', do not use boards.js.\n");
            Out.Printf(Out.PriHigh, "   -id                   Prompt user for test ID.\n");
            Out.Printf(Out.PriHigh, "   -notiled              Do not use tiled surfaces.\n");
            Out.Printf(Out.PriHigh, "   -screen_off           Disable output to the screen during tests.\n");
            Out.Printf(Out.PriHigh, "   -display_config       Get the display configuration.\n");
            Out.Printf(Out.PriHigh, "   -check_display        Run the CheckDisplay test.\n");
            Out.Printf(Out.PriHigh, "   -check_displays       Run the CheckDisplay test on all displays.\n");
            Out.Printf(Out.PriHigh, "   -check_display_bar    Run the CheckDisplayBar test.\n");
            Out.Printf(Out.PriHigh, "   -check_display_bars   Run the CheckDisplayBars test.\n");
            Out.Printf(Out.PriHigh, "   -check_fp_gray        Run the gray CheckDisplays test on flat panels\n");
            Out.Printf(Out.PriHigh, "   -check_fp_stripes     Run the stripe CheckDisplays test on flat panels\n");
            Out.Printf(Out.PriHigh, "   -stereo               Run the stereo test.\n");
            Out.Printf(Out.PriHigh, "   -tvo                  Run the internal TV encoder test.\n");
            Out.Printf(Out.PriHigh, "   -framelock            Run the Full FrameLock tests.\n");
            Out.Printf(Out.PriHigh, "   -framelock_auto       Run the Only the automated part of FrameLock tests.\n");
            Out.Printf(Out.PriHigh, "   -vivo                 Run the TV Video-in, Video-out test.\n");
            Out.Printf(Out.PriHigh, "   -progtv               Program non-volatile I2C on TV tuner cards\n");
            Out.Printf(Out.PriHigh, "   -tuner                Run the TV RF-in test\n");
            Out.Printf(Out.PriHigh, "   -rf                   Set TV tuner RF frequency.\n");
            Out.Printf(Out.PriHigh, "   -video_in             Digitize the TV-IN and display on the screen\n");
            Out.Printf(Out.PriHigh, "   -usb_ports number     Number of USB ports to be tested.\n");
            Out.Printf(Out.PriHigh, "   -usb_acr_port number  Number of USB port on Acr slot to be tested.\n");
            Out.Printf(Out.PriHigh, "   -fw_devices number    Number of devices attached.\n");
            Out.Printf(Out.PriHigh, "   -fw_loopback          Enable FireWire Loopback tests.\n");
            Out.Printf(Out.PriHigh, "   -fw_ports number      Number of FireWire ports to be tested, used with loopback enabled\n");
            Out.Printf(Out.PriHigh, "   -audio_mic            Loopback through the MIC channel.\n");
            Out.Printf(Out.PriHigh, "   -audio_front_mic      Loopback through the FRONT_MIC channel.\n");
            Out.Printf(Out.PriHigh, "   -audio_cdin           Loopback through the CD_IN channel.\n");
            Out.Printf(Out.PriHigh, "   -audio_auxin          Loopback through the AUX_IN channel.\n");
            Out.Printf(Out.PriHigh, "   -audio_phone          Loopback through the PHONE/MODEM_IN channel.\n");
            Out.Printf(Out.PriHigh, "   -audio_videoin        Loopback through the VIDEO_IN channel.\n");
            Out.Printf(Out.PriHigh, "   -audio_spdifin        Loopback between SpdifOut and SpdifIn.\n");
            Out.Printf(Out.PriHigh, "   -audio_all            Loopback through all ACI channels, including surround.\n");
            Out.Printf(Out.PriHigh, "   -nvmac_mac            Use internal MAC loopback for NVMAC tests.\n");
            Out.Printf(Out.PriHigh, "   -nvmac_phy            Use internal PHY loopback for NVMAC tests.\n");
            Out.Printf(Out.PriHigh, "   -nvmac_10mbps         Force link speed to 10Mbps for NVMAC tests.\n");
            Out.Printf(Out.PriHigh, "   -nvmac_100mbps        Force link speed to 100Mbps for NVMAC tests.\n");
            Out.Printf(Out.PriHigh, "   -tcmac_mac            Use internal MAC loopback for 3COM MAC tests.\n");
            Out.Printf(Out.PriHigh, "   -tcmac_phy            Use internal PHY loopback for 3COM MAC tests.\n");
            Out.Printf(Out.PriHigh, "   -tcmac_10mbps         Force link speed to 10Mbps for 3COM MAC tests.\n");
            Out.Printf(Out.PriHigh, "   -tcmac_100mbps        Force link speed to 100Mbps for 3COM MAC tests.\n");
            Out.Printf(Out.PriHigh, "   -sst_skip_ata         Skip ATA DMA in SystemStressTest.\n");
            Out.Printf(Out.PriHigh, "   -sst_skip_ethernet    Skip Ethernet DMA in SystemStressTest.\n");
            Out.Printf(Out.PriHigh, "   -sst_skip_aci         Skip ACI DMA in SystemStressTest.\n");
            Out.Printf(Out.PriHigh, "   -smb_skip_port1       Skip port1 in SMBus test if acr/cnr card is plugged in.\n");
            Out.Printf(Out.PriHigh, "   -no_gold              Do not load golden values.\n");
            Out.Printf(Out.PriHigh, "   -old_gold             Using old golden values.\n");
            Out.Printf(Out.PriHigh, "   -show_gold            Display contents of goldenXX.bin file.\n");
            Out.Printf(Out.PriHigh, "   -rmt_key number       Set the remote controller's expected key value.\n");
            Out.Printf(Out.PriHigh, "   -simulate_dfp type    Simulate a flat panel with the specified EDID.\n");
            Out.Printf(Out.PriHigh, "   -simulate_tv          Simulate a TV is attached .\n");
            Out.Printf(Out.PriHigh, "   -tmds_crc             Use TMDS/LVDS CRCs.\n");
            Out.Printf(Out.PriHigh, "   -no_drvslew           Do not adjust drive/slew and memory clock.\n");
            Out.Printf(Out.PriHigh, "   -no_recal_drvslew     Do not continuously recal the drive/slew.\n");
            Out.Printf(Out.PriHigh, "   -serial_ports         Total number of serial ports to be tested.\n");
            Out.Printf(Out.PriHigh, "   -stress_baseline      Find max nvclk for rm internal stress test, write bios patch file.\n");
            Out.Printf(Out.PriHigh, "   -mpeg_in_fb           Run MPEG tests out of framebuffer rather than AGP.\n");
            Out.Printf(Out.PriHigh, "   -no_require_fos       Allow glr_display to pass even if we can't FOS because we are using head 1.\n");
            Out.Printf(Out.PriHigh, "   -override file        Execute the script 'file' to override GPU settings.\n");
            Out.Printf(Out.PriHigh, "   -help or -h or -?     Print command line options.\n");
            Out.Printf(Out.PriHigh, "\n");
            return RC.EXIT_OK;
         }

         default:
         {
            Out.Printf(Out.PriHigh, "Bad argument '%s'.\n", Argv[i]);
            return RC.BAD_COMMAND_LINE_ARGUMENT;
         }
      }
   } // for each Argv

   return OK;
}

//------------------------------------------------------------------------------
// Set GPU test properties.
//
function SetGpuTestProperties()
{
   var Resolution = new Array;
   CHECK_RC(Display.GetMaxResolution(Display.Selected, Resolution));
   var MaxDisplayWidth  = Resolution[0];
   var MaxDisplayHeight = Resolution[1];

   //
   // MemSize properties.
   //
   if (undefined == MemSize.TestConfiguration)
   {
      MemSize.TestConfiguration = new Object;
   }
   MemSize.TestConfiguration.DisplayWidth       = Math.min(MaxDisplayWidth,  640);
   MemSize.TestConfiguration.DisplayHeight      = Math.min(MaxDisplayHeight, 480);
   MemSize.TestConfiguration.DisplayDepth       = 32;
   MemSize.TestConfiguration.DisplayRefreshRate = 60;
   MemSize.TestConfiguration.UseTiledSurface    = false;

   // MemSize does not require golden values.

   //
   // Mats properties.
   //
   Mats.Coverage = 10;  // Test 10% of memory.

   if (undefined == Mats.TestConfiguration)
   {
      Mats.TestConfiguration = new Object;
   }
   Mats.TestConfiguration.DisplayWidth       = Math.min(MaxDisplayWidth,  800);
   Mats.TestConfiguration.DisplayHeight      = Math.min(MaxDisplayHeight, 600);
   Mats.TestConfiguration.DisplayDepth       = 32;
   Mats.TestConfiguration.DisplayRefreshRate = 60;
   Mats.TestConfiguration.UseTiledSurface    = false;

   // Mats does not require golden values because it performs memory compares.

   //
   // FastMats properties.
   //
   if (undefined == FastMats.TestConfiguration)
   {
      FastMats.TestConfiguration = new Object;
   }
   FastMats.TestConfiguration.DisplayWidth       = Math.min(MaxDisplayWidth,  800);
   FastMats.TestConfiguration.DisplayHeight      = Math.min(MaxDisplayHeight, 600);
   FastMats.TestConfiguration.DisplayDepth       = 32;
   FastMats.TestConfiguration.DisplayRefreshRate = 60;
   FastMats.TestConfiguration.UseTiledSurface    = false;

   // FastMats does not require golden values because it performs memory compares.

   //
   // WfMats (whicked fast MATS) properties.
   //
   if (undefined == WfMats.TestConfiguration)
   {
      WfMats.TestConfiguration = new Object;
   }
   WfMats.TestConfiguration.DisplayWidth       = Math.min(MaxDisplayWidth, 1024);
   WfMats.TestConfiguration.DisplayHeight      = Math.min(MaxDisplayHeight, 768);
   WfMats.TestConfiguration.DisplayDepth       = 32;
   WfMats.TestConfiguration.DisplayRefreshRate = 60;
   WfMats.TestConfiguration.UseTiledSurface    = false;

   // WfMats does not require golden values because it performs memory compares.

   //
   // VgaTest properties.
   //
   if (undefined == VgaTest.TestConfiguration)
   {
      VgaTest.TestConfiguration = new Object;
   }
   VgaTest.WaitMs = 5000;

   // VgaTest does not require golden values because it performs memory compares.

   //
   // MemInterface properties.
   //
   if (undefined == MemInterface.TestConfiguration)
   {
      MemInterface.TestConfiguration = new Object;
   }
   MemInterface.TestConfiguration.DisplayWidth       = Math.min(MaxDisplayWidth, 1024);
   MemInterface.TestConfiguration.DisplayHeight      = Math.min(MaxDisplayHeight, 768);
   MemInterface.TestConfiguration.DisplayDepth       = 32;
   MemInterface.TestConfiguration.DisplayRefreshRate = 60;
   MemInterface.TestConfiguration.Loops              = 1000;

   if (undefined == MemInterface.Golden)
   {
      MemInterface.Golden = new Object;
   }
   MemInterface.Golden.Codes       = Golden.Crc;
   MemInterface.Golden.NumCodeBins = 1;

   if (Ikos)
   {
      MemInterface.TestConfiguration.Loops              = 20;
      MemInterface.TestConfiguration.RestartSkipCount   =    5;
      MemInterface.Golden.SkipCount = MemInterface.TestConfiguration.RestartSkipCount;
   }




   //
   // MemRefresh properties.
   //
   if (undefined == MemRefresh.TestConfiguration)
   {
      MemRefresh.TestConfiguration = new Object;
   }
   MemRefresh.TestConfiguration.DisplayWidth       = Math.min(MaxDisplayWidth, 1024);
   MemRefresh.TestConfiguration.DisplayHeight      = Math.min(MaxDisplayHeight, 768);
   MemRefresh.TestConfiguration.DisplayDepth       = 16;
   MemRefresh.TestConfiguration.DisplayRefreshRate = 60;
   MemRefresh.TestConfiguration.Loops              = 10;
   MemRefresh.TestConfiguration.RestartSkipCount   = 5;
   MemRefresh.TestConfiguration.UseTiledSurface    = false;

   if (undefined == MemRefresh.Golden)
   {
      MemRefresh.Golden = new Object;
   }
   MemRefresh.Golden.SkipCount = MemRefresh.TestConfiguration.RestartSkipCount;

   //
   // MemPatt properties.
   //
   if (undefined == MemPatt.TestConfiguration)
   {
      MemPatt.TestConfiguration = new Object;
   }
   MemPatt.TestConfiguration.DisplayWidth       = Math.min(MaxDisplayWidth, 1024);
   MemPatt.TestConfiguration.DisplayHeight      = Math.min(MaxDisplayHeight, 768);
   MemPatt.TestConfiguration.DisplayDepth       = 32;
   MemPatt.TestConfiguration.DisplayRefreshRate = 60;
   MemPatt.TestConfiguration.Loops              = 4000;

   if (undefined == MemPatt.Golden)
   {
      MemPatt.Golden = new Object;
   }
   MemPatt.Golden.Codes       = Golden.Crc;
   MemPatt.Golden.NumCodeBins = 1;
   MemPatt.DrawFreqA = 0;
   MemPatt.DrawFreqB = 1;
   MemPatt.PatternsA = [
                          [ 0, 0, 0, 0, 0, 0, 0, 0 ],
                       ];
   MemPatt.PatternsB = [
                          [ 0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa ],
                          [ 0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55 ],
                       ];

   if (Ikos)
   {
      MemPatt.TestConfiguration.Loops              = 20;
      MemPatt.TestConfiguration.RestartSkipCount   =    5;
      MemPatt.Golden.SkipCount = MemPatt.TestConfiguration.RestartSkipCount;
   }


   //
   // Class038 properties.
   //
   if (undefined == Class038.TestConfiguration)
   {
      Class038.TestConfiguration = new Object;
   }
   Class038.TestConfiguration.DisplayWidth       = Math.min(MaxDisplayWidth, 1024);
   Class038.TestConfiguration.DisplayHeight      = Math.min(MaxDisplayHeight, 768);
   Class038.TestConfiguration.DisplayDepth       = 16;
   Class038.TestConfiguration.DisplayRefreshRate = 60;
   Class038.TestConfiguration.Loops              = 1000;

   if (undefined == Class038.Golden)
   {
      Class038.Golden = new Object;
   }
   Class038.Golden.Codes       = Golden.Crc;
   Class038.Golden.NumCodeBins = 1;

   //
   // Class039 properties.
   //
   if (undefined == Class039.TestConfiguration)
   {
      Class039.TestConfiguration = new Object;
   }
   Class039.TestConfiguration.DisplayWidth       = Math.min(MaxDisplayWidth, 1024);
   Class039.TestConfiguration.DisplayHeight      = Math.min(MaxDisplayHeight, 768);
   Class039.TestConfiguration.DisplayDepth       = 32;
   Class039.TestConfiguration.DisplayRefreshRate = 60;
   Class039.TestConfiguration.Loops              = 100;
   Class039.TestConfiguration.UseTiledSurface    = false;
   if (("Mac" == OperatingSystem) || ("MacOSX" == OperatingSystem))
   {
      Class039.IsAgpOnly = true;
   }

   // Class039 does not require golden values because it performs memory compares.

   //
   // AgpTest properties.
   //
   if (undefined == AgpTest.TestConfiguration)
   {
      AgpTest.TestConfiguration = new Object;
   }
   AgpTest.TestConfiguration.DisplayWidth       = Math.min(MaxDisplayWidth,  640);
   AgpTest.TestConfiguration.DisplayHeight      = Math.min(MaxDisplayHeight, 480);
   AgpTest.TestConfiguration.DisplayDepth       =   32;
   AgpTest.TestConfiguration.DisplayRefreshRate =   60;
   AgpTest.TestConfiguration.Loops              = 1000;
   AgpTest.TestConfiguration.UseTiledSurface    = false;

   // AgpTest does not require golden values because it performs memory compares.

   //
   // Class046 properties.
   //
   if (undefined == Class046.TestConfiguration)
   {
      Class046.TestConfiguration = new Object;
   }
   Class046.TestConfiguration.DisplayWidth       = Math.min(MaxDisplayWidth,  640);
   Class046.TestConfiguration.DisplayHeight      = Math.min(MaxDisplayHeight, 480);
   Class046.TestConfiguration.DisplayDepth       = 32;
   Class046.TestConfiguration.DisplayRefreshRate = 60;
   Class046.TestConfiguration.Loops              = 20;
   Class046.TestConfiguration.DisableCrt         = false;

   if (undefined == Class046.Golden)
   {
      Class046.Golden = new Object;
   }
   Class046.Golden.Codes     = g_DoTmdsCrc ? Golden.TmdsCrc : Golden.DacCrc;
   Class046.Golden.SkipCount = 0;

   //
   // Class047 properties.
   //
   if (undefined == Class047.TestConfiguration)
   {
      Class047.TestConfiguration = new Object;
   }
   Class047.TestConfiguration.DisplayWidth       = Math.min(MaxDisplayWidth,  640);
   Class047.TestConfiguration.DisplayHeight      = Math.min(MaxDisplayHeight, 480);
   Class047.TestConfiguration.DisplayDepth       = 32;
   Class047.TestConfiguration.DisplayRefreshRate = 60;
   Class047.TestConfiguration.Loops              = 100;
   Class047.TestConfiguration.DisableCrt         = false;

   if (undefined == Class047.Golden)
   {
      Class047.Golden = new Object;
   }
   Class047.Golden.Codes     = g_DoTmdsCrc ? Golden.TmdsCrc : Golden.DacCrc;
   Class047.Golden.SkipCount = 0;

   //
   // Class04a properties.
   //
   if (undefined == Class04a.TestConfiguration)
   {
      Class04a.TestConfiguration = new Object;
   }
   Class04a.TestConfiguration.DisplayWidth       = Math.min(MaxDisplayWidth, 1024);
   Class04a.TestConfiguration.DisplayHeight      = Math.min(MaxDisplayHeight, 768);
   Class04a.TestConfiguration.DisplayDepth       = 32;
   Class04a.TestConfiguration.DisplayRefreshRate = 60;
   Class04a.TestConfiguration.Loops              = 200;

   if (undefined == Class04a.Golden)
   {
      Class04a.Golden = new Object;
   }
   Class04a.Golden.Codes       = Golden.Crc;
   Class04a.Golden.NumCodeBins = 1;

   //
   // Class04d properties.
   //
   if (undefined == Class04d.TestConfiguration)
   {
      Class04d.TestConfiguration = new Object;
   }
   Class04d.TestConfiguration.DisplayWidth       = Math.min(MaxDisplayWidth, 1024);
   Class04d.TestConfiguration.DisplayHeight      = Math.min(MaxDisplayHeight, 768);
   Class04d.TestConfiguration.DisplayDepth       = 32;
   Class04d.TestConfiguration.DisplayRefreshRate = 60;
   Class04d.TestConfiguration.Loops              = 1000;

   // Class04d does not require golden values because it is a user visual test

   //
   // Class05c properties.
   //
   if (undefined == Class05c.TestConfiguration)
   {
      Class05c.TestConfiguration = new Object;
   }
   Class05c.TestConfiguration.DisplayWidth       = Math.min(MaxDisplayWidth, 1024);
   Class05c.TestConfiguration.DisplayHeight      = Math.min(MaxDisplayHeight, 768);
   Class05c.TestConfiguration.DisplayDepth       = 32;
   Class05c.TestConfiguration.DisplayRefreshRate = 60;
   Class05c.TestConfiguration.Loops              = 1000;

   if (undefined == Class05c.Golden)
   {
      Class05c.Golden = new Object;
   }
   Class05c.Golden.Codes       = Golden.Crc;
   Class05c.Golden.NumCodeBins = 1;

   //
   // Class05d properties.
   //
   if (undefined == Class05d.TestConfiguration)
   {
      Class05d.TestConfiguration = new Object;
   }
   Class05d.TestConfiguration.DisplayWidth       = Math.min(MaxDisplayWidth, 1024);
   Class05d.TestConfiguration.DisplayHeight      = Math.min(MaxDisplayHeight, 768);
   Class05d.TestConfiguration.DisplayDepth       = 32;
   Class05d.TestConfiguration.DisplayRefreshRate = 60;
   Class05d.TestConfiguration.Loops              = 2000;
   Class05d.TestConfiguration.RestartSkipCount   = 200;

   if (undefined == Class05d.Golden)
   {
      Class05d.Golden = new Object;
   }
   Class05d.Golden.Codes        = Golden.Crc;
   Class05d.Golden.NumCodeBins  = 1;
   Class05d.Golden.SkipCount    = Class05d.TestConfiguration.RestartSkipCount;

   //
   // Old05d properties.
   //
   if (undefined == Old05d.TestConfiguration)
   {
      Old05d.TestConfiguration = new Object;
   }
   Old05d.TestConfiguration.DisplayWidth       = Math.min(MaxDisplayWidth, 1024);
   Old05d.TestConfiguration.DisplayHeight      = Math.min(MaxDisplayHeight, 768);
   Old05d.TestConfiguration.DisplayDepth       = 32;
   Old05d.TestConfiguration.DisplayRefreshRate = 60;
   Old05d.TestConfiguration.Loops              = 1000;

   if (undefined == Old05d.Golden)
   {
      Old05d.Golden = new Object;
   }
   Old05d.Golden.Codes       = Golden.Crc;
   Old05d.Golden.NumCodeBins = 1;

   //
   // Class05f properties.
   //
   if (undefined == Class05f.TestConfiguration)
   {
      Class05f.TestConfiguration = new Object;
   }
   Class05f.TestConfiguration.DisplayWidth       = Math.min(MaxDisplayWidth, 1024);
   Class05f.TestConfiguration.DisplayHeight      = Math.min(MaxDisplayHeight, 768);
   Class05f.TestConfiguration.DisplayDepth       = 32;
   Class05f.TestConfiguration.DisplayRefreshRate = 60;
   Class05f.TestConfiguration.Loops              = 500;

   if (undefined == Class05f.Golden)
   {
      Class05f.Golden = new Object;
   }
   Class05f.Golden.Codes       = Golden.Crc;
   Class05f.Golden.NumCodeBins = 1;

   //
   // Class061 properties.
   //
   if (undefined == Class061.TestConfiguration)
   {
      Class061.TestConfiguration = new Object;
   }
   Class061.TestConfiguration.DisplayWidth       = Math.min(MaxDisplayWidth, 1024);
   Class061.TestConfiguration.DisplayHeight      = Math.min(MaxDisplayHeight, 768);
   Class061.TestConfiguration.DisplayDepth       = 32;
   Class061.TestConfiguration.DisplayRefreshRate = 60;
   Class061.TestConfiguration.Loops              = 1000;

   if (undefined == Class061.Golden)
   {
      Class061.Golden = new Object;
   }
   Class061.Golden.Codes       = Golden.Crc;
   Class061.Golden.NumCodeBins = 1;

   //
   // Class064 properties.
   //
   if (undefined == Class064.TestConfiguration)
   {
      Class064.TestConfiguration = new Object;
   }
   Class064.TestConfiguration.DisplayWidth       = Math.min(MaxDisplayWidth, 1024);
   Class064.TestConfiguration.DisplayHeight      = Math.min(MaxDisplayHeight, 768);
   Class064.TestConfiguration.DisplayDepth       = 32;
   Class064.TestConfiguration.DisplayRefreshRate = 60;
   Class064.TestConfiguration.Loops              = 1000;

   if (undefined == Class064.Golden)
   {
      Class064.Golden = new Object;
   }
   Class064.Golden.Codes       = Golden.Crc;
   Class064.Golden.NumCodeBins = 1;

   //
   // Class077 properties.
   //
   if (undefined == Class077.TestConfiguration)
   {
      Class077.TestConfiguration = new Object;
   }
   Class077.TestConfiguration.DisplayWidth       = Math.min(MaxDisplayWidth, 1024);
   Class077.TestConfiguration.DisplayHeight      = Math.min(MaxDisplayHeight, 768);
   Class077.TestConfiguration.DisplayDepth       = 32;
   Class077.TestConfiguration.DisplayRefreshRate = 60;
   Class077.TestConfiguration.Loops              = 200;

   if (undefined == Class077.Golden)
   {
      Class077.Golden = new Object;
   }
   Class077.Golden.Codes       = Golden.Crc;
   Class077.Golden.NumCodeBins = 1;

   //
   // Class07c properties.
   //
   if (undefined == Class07c.TestConfiguration)
   {
      Class07c.TestConfiguration = new Object;
   }
   Class07c.TestConfiguration.DisplayWidth       = Math.min(MaxDisplayWidth,  640);
   Class07c.TestConfiguration.DisplayHeight      = Math.min(MaxDisplayHeight, 480);
   Class07c.TestConfiguration.DisplayDepth       = 32;
   Class07c.TestConfiguration.DisplayRefreshRate = 60;
   Class07c.TestConfiguration.Loops              = 5;
   Class07c.TestConfiguration.DisableCrt         = false;

   if (undefined == Class07c.Golden)
   {
      Class07c.Golden = new Object;
   }
   Class07c.Golden.Codes     = g_DoTmdsCrc ? Golden.TmdsCrc : Golden.DacCrc;
   Class07c.Golden.SkipCount = 0;

   //
   // Class07a properties.
   //
   if (undefined == Class07a.TestConfiguration)
   {
      Class07a.TestConfiguration = new Object;
   }
   Class07a.TestConfiguration.DisplayWidth       = Math.min(MaxDisplayWidth,  640);
   Class07a.TestConfiguration.DisplayHeight      = Math.min(MaxDisplayHeight, 480);
   Class07a.TestConfiguration.DisplayDepth       = 32;
   Class07a.TestConfiguration.DisplayRefreshRate = 60;
   Class07a.TestConfiguration.Loops              = 10;

   if (undefined == Class07a.Golden)
   {
      Class07a.Golden = new Object;
   }
   Class07a.Golden.Codes     = g_DoTmdsCrc ? Golden.TmdsCrc : Golden.DacCrc;
   Class07a.Golden.SkipCount = 0;

   //
   // Class089 properties.
   //
   if (undefined == Class089.TestConfiguration)
   {
      Class089.TestConfiguration = new Object;
   }
   Class089.TestConfiguration.DisplayWidth       = Math.min(MaxDisplayWidth, 1024);
   Class089.TestConfiguration.DisplayHeight      = Math.min(MaxDisplayHeight, 768);
   Class089.TestConfiguration.DisplayDepth       = 32;
   Class089.TestConfiguration.DisplayRefreshRate = 60;
   Class089.TestConfiguration.Loops              = 600;

   if (undefined == Class089.Golden)
   {
      Class089.Golden = new Object;
   }
   Class089.Golden.Codes       = Golden.Crc;
   Class089.Golden.NumCodeBins = 1;

   //
   // Class1774 properties.
   //
   if (undefined == Class1774.TestConfiguration)
   {
      Class1774.TestConfiguration = new Object;
   }
   Class1774.TestConfiguration.DisplayWidth       = Math.min(MaxDisplayWidth,  640);
   Class1774.TestConfiguration.DisplayHeight      = Math.min(MaxDisplayHeight, 480);
   Class1774.TestConfiguration.DisplayDepth       =  32;
   Class1774.TestConfiguration.DisplayRefreshRate =  60;
   Class1774.TestConfiguration.Loops              = 100; // **
   Class1774.TestConfiguration.DisableCrt         = false;

   // Apple refuses to use the -mpeg_in_fb flag, give them FB always
   if ((g_MpegInFb) || ("MacOSX" == OperatingSystem))
      Class1774.TestConfiguration.MemoryType = Memory.Pci;

   // ** Class1774 will play back the trace just once, but it will use
   // as many golden values as the tracefile has frames.
   // We plug in 100 here so that the g_CustomerGen mode works right.

   if (undefined == Class1774.Golden)
   {
      Class1774.Golden = new Object;
   }
   Class1774.Golden.Codes = g_DoTmdsCrc ? Golden.TmdsCrc : Golden.DacCrc;

   if(("MacOSX" == OperatingSystem) || ("Linux" == OperatingSystem))
   {
      // No Dac Crc on Mac OSX, doesn't work.
      Class1774.Golden.Codes  = Golden.Crc;
   }
   Class1774.Golden.SkipCount = 0;

   //
   // Class3174 properties.
   //
   if (undefined == Class3174.TestConfiguration)
   {
      Class3174.TestConfiguration = new Object;
   }
   Class3174.TestConfiguration.DisplayWidth       = Math.min(MaxDisplayWidth,  640);
   Class3174.TestConfiguration.DisplayHeight      = Math.min(MaxDisplayHeight, 480);
   Class3174.TestConfiguration.DisplayDepth       = 32;
   Class3174.TestConfiguration.DisplayRefreshRate = 60;
   Class3174.TestConfiguration.Loops              = 100; // **
   Class3174.TestConfiguration.DisableCrt         = false;

   // Apple refuses to use the -mpeg_in_fb flag, give them FB always.
   if ((g_MpegInFb) || ("MacOSX" == OperatingSystem))
      Class3174.TestConfiguration.MemoryType      = Memory.Pci;

   // ** Class3174 will play back the trace just once, but it will use
   // as many golden values as the tracefile has frames.
   // We plug in 100 here so that the g_CustomerGen mode works right.

   if (undefined == Class3174.Golden)
   {
      Class3174.Golden = new Object;
   }
   Class3174.Golden.Codes = g_DoTmdsCrc ? Golden.TmdsCrc : Golden.DacCrc;

   if(("MacOSX" == OperatingSystem) || ("Linux" == OperatingSystem))
   {
      // No Dac Crc on Mac OSX, doesn't work.
      Class3174.Golden.Codes = Golden.Crc;
   }
   Class3174.Golden.SkipCount = 0;

   //
   // GLRandom (glr_hwtest) properties.
   //
   if (undefined == GLRandom.TestConfiguration)
   {
      GLRandom.TestConfiguration = new Object;
   }
   GLRandom.TestConfiguration.DisplayWidth       = Math.min(MaxDisplayWidth, 1024);
   GLRandom.TestConfiguration.DisplayHeight      = Math.min(MaxDisplayHeight, 768);
   GLRandom.TestConfiguration.DisplayDepth       = 32;
   GLRandom.TestConfiguration.DisplayRefreshRate = 60;
   GLRandom.TestConfiguration.Loops              = 5000;

   if (undefined == GLRandom.Golden)
   {
      GLRandom.Golden = new Object;
   }
   if (("Mac" == OperatingSystem) || ("MacOSX" == OperatingSystem))
      GLRandom.FrameRetries = 40;
   else
      GLRandom.FrameRetries = 10;

   //
   // GLStress properties.
   //
   GLStress.LoopMs = 15000;

   if (undefined == GLStress.TestConfiguration)
   {
      GLStress.TestConfiguration = new Object;
   }
   GLStress.TestConfiguration.DisplayWidth       = Math.min(MaxDisplayWidth, 1024);
   GLStress.TestConfiguration.DisplayHeight      = Math.min(MaxDisplayHeight, 768);
   GLStress.TestConfiguration.DisplayDepth       = 32;
   GLStress.TestConfiguration.DisplayRefreshRate = 60;

   // GLStress does not require golden values because it performs memory compares.

   //
   // Run3DTrace properties.
   //
   if (undefined == Run3DTrace.TestConfiguration)
   {
      Run3DTrace.TestConfiguration = new Object;
   }
   Run3DTrace.TestConfiguration.DisplayWidth       = Math.min(MaxDisplayWidth,  640);
   Run3DTrace.TestConfiguration.DisplayHeight      = Math.min(MaxDisplayHeight, 480);
   Run3DTrace.TestConfiguration.DisplayDepth       = 32;
   Run3DTrace.TestConfiguration.DisplayRefreshRate = 60;
   Run3DTrace.TestConfiguration.UseTiledSurface    = false;

   // Run3DTrace does not require golden values.

   return OK;
}

//------------------------------------------------------------------------------
// Set test limits for this product.
//
function SetGpuTestLimits()
{
   Golden.Codes       = Golden.CheckSums;
   Golden.NumCodeBins = 97;

   // SumDiff format [alpha, red, green, blue, stencil, depth].
   switch (g_TestMode - 0)
   {
      case TM_OQA:
      {
         Golden.TestMode = Golden.Oqa;
         GLRandom.AllowedSoftErrors     = 2;
         break;
      } // TM_OQA

      case TM_GPU:
      {
         Golden.TestMode = Golden.Mfg;
         GLRandom.AllowedSoftErrors     = 1;
         break;
      } // TM_GPU

      case TM_ENGR:
      {
         Golden.TestMode = Golden.Mfg;
         Golden.AllowedBadBinsOneCheck   = 0;
         Golden.AllowedErrorBinsOneCheck = 0;
         Golden.AllowedSumDiffOneCheck   = [0, 0, 0, 0, 0, 0];
         Golden.AllowedBinDiff           = [0, 0, 0, 0, 0, 0];
         GLRandom.FrameRetries           = 0;
         break;
      } // TM_ENGR

      case TM_MCP:
      case TM_ACR:
      case TM_BAY:
         break;

      default:
      {
         Out.Printf(Out.PriHigh, "Invalid test mode.\n");
         return RC.SOFTWARE_ERROR;
      }
   } // g_TestMode

   if (g_RunOnError)
      GLRandom.ContinueOnFrameError = true;

   return OK;
}

//------------------------------------------------------------------------------
// Set the memory and core clock rates (guard bands).
//
function SetGpuClocks()
{
   // Set the RM performance mode before directly modifying the clocks.
   Master.PERF_MODE = g_RmPerfMode;

   // Set the PowerMizer level if required.
   if (g_PowerMizerLevel != null)
      Master.POWER_MIZER = g_PowerMizerLevel;

   // Set the memory clock.
   if (g_MemoryClock_Hz != null)
   {
      g_OriginalMemoryClock_Hz = Dac.MEMORY_CLOCK;
      Dac.MEMORY_CLOCK = g_MemoryClock_Hz;
   }

   // Set the graphics clock.
   if (g_GraphicsClock_Hz != null)
   {
      g_OriginalGraphicsClock_Hz = Dac.GRAPHICS_CLOCK;
      Dac.GRAPHICS_CLOCK = g_GraphicsClock_Hz;
   }

   // Store the chosen GPU core voltage for reporting later.
   // This is only correct when CORE_VOLTAGE_CHANGE_ALLOWED is true.
   // This only works on voltage-changing VBIOSs.
   if (Master.CORE_VOLTAGE_MV > 0)
      g_GpuCoreVoltage = 0.001 * Master.CORE_VOLTAGE_MV;

   // Force RM to leave GPU core voltage alone unless the user specified otherwise.
   Master.CORE_VOLTAGE_CHANGE_ALLOWED = g_DynamicVoltage;

   // Force RM to not touch MODS clock settings.
   Master.CLOCK_CHANGE_ALLOWED = g_DynamicClocks;

   // Store the test clocks to report them later.
   g_TestMemoryClock_Hz   = Dac.MEMORY_CLOCK;
   g_TestGraphicsClock_Hz = Dac.GRAPHICS_CLOCK;

   return OK;
}

//------------------------------------------------------------------------------
// Check if the frame buffer interface or memory is marginal.
//
function CheckMemoryAndFBI(TestName)
{
   // Reduce the mclk and see if the test passes.
   MemoryClock_Hz = Dac.MEMORY_CLOCK;
   Dac.MEMORY_CLOCK = (1.0 - (g_FbiCheck / 100.0)) * MemoryClock_Hz;
   Out.Printf(Out.PriNormal, "Retrying %s at %5.1f MHz memory clock.\n", TestName, Dac.MEMORY_CLOCK / 1e6);
   var rc = eval(TestName + "()");

   // If the test passed, this means we did have marginal memory or FBI, return
   // set the first error to indicate the correct failure.
   if (OK == rc)
   {
      Log.ResetFirstError();
      rc = RC.MARGINAL_MEMORY_OR_FBI;
   }

   // Restore the original mclk.
   Dac.MEMORY_CLOCK = MemoryClock_Hz;

   return rc;
}

//------------------------------------------------------------------------------
// Map OS string to OS_*.
//
Log.Never("GetOS");
function GetOS()
{
   var OS = 0;

   switch (OperatingSystem)
   {
      case "Dos": {OS = OS_DOS; break;}
      case "Win": {OS = OS_WIN; break;}
      case "MacOSX":
      case "Mac": {OS = OS_MAC; break;}
      case "Xos": {OS = OS_XOS; break;}
      case "VxWorks": {OS = OS_VXWORKS; break;}
      case "Linux": {OS = OS_LINUX; break;}
      default:
      {
         Out.Printf(Out.PriHigh, "Invalid operating system.\n");
         break;
      }
   }

   return OS;
}

//------------------------------------------------------------------------------
// Load golden values.
//
var g_GoldenFileName = "";
function LoadGoldenValuesHelper()
{
   var rc = OK;

   if ("" == g_GoldenFileName)
      g_GoldenFileName = "golden" + Out.Sprintf("%02x", Gpu.DeviceId) + ".bin";

   Golden.Name   = Out.Sprintf("%s_NV%02X", OperatingSystem, Gpu.DeviceId);
   Golden.Action = Golden.Check;

   Log.Next = false;
   rc = Golden.ReadFile(g_GoldenFileName, g_ShowGold ? Out.LevNormal : Out.LevLow);

   if (g_OldGold && (rc == RC.GOLDEN_FILE_MISMATCH))
   {
      rc = OK;
      Out.Printf(Out.PriHigh, "Ignoring golden file mismatch\n");
   }
   return rc;
}
function LoadGoldenValues()
{
   return LoadGoldenValuesHelper();
}

//------------------------------------------------------------------------------
// Get the FCODE version
//
Log.Never("GetFcodeVersion");
Log.Never("ReadRegistryString");
function GetFcodeVersion()
{
   var FcodeVersion = new Array;
   if ("MacOSX" == OperatingSystem)
   {
      if ( ReadRegistryString("NVDA",  "rom-revision", FcodeVersion) )
         return null;
   }
   else
   {
      if ( ReadRegistryString("NVDA,NVMac",  "rom-revision", FcodeVersion)
           && ReadRegistryString("NVDA,Parent", "rom-revision", FcodeVersion) )
         return null;
   }

   return FcodeVersion[0];
}

//------------------------------------------------------------------------------
// Check if the ROM version matches the expected value(s).
//
function CheckRomVersion()
{
   var rc = OK;

   if (0 == g_RomVersions.length)
   {
      Out.Printf(Out.PriHigh, "Skipping CheckRomVersion test.\n");
      return OK;
   }

   CHECK_RC(DisableUserInterface(640, 480, 32, 60));

   var RomVersion = (("Mac" == OperatingSystem) || ("MacOSX" == OperatingSystem)) ?
                         GetFcodeVersion() : BiosVersion;

   rc = RC.INCORRECT_ROM_VERSION;
   for (var i in g_RomVersions)
   {
      if (RomVersion == g_RomVersions[i])
      {
         rc = OK;
         break;
      }
   }

   if (rc != OK)
   {
      Out.Printf(Out.PriHigh, "ROM version = %s; expected %s.\n", RomVersion, g_RomVersions);
   }

   EnableUserInterface();

   return rc;
}

//------------------------------------------------------------------------------
// Check if the chip revision matches the expected value.
//
function CheckRevision()
{
   if (null == g_Revision)
   {
      Out.Printf(Out.PriHigh, "Skipping CheckRevision test.\n");
      return OK;
   }

   if (g_Revision != Bus.Revision)
   {
      Out.Printf(Out.PriHigh, "Revision = %#04x; expected %#04x.\n", Bus.Revision, g_Revision);
      return RC.INCORRECT_CHIP_REVISION;
   }

   return OK;
}

//------------------------------------------------------------------------------
// Check if the subsystem vendor and device ID's matches the expected value(s).
//
function CheckSubsystemIds()
{
   if (null == g_SubsytemId)
   {
      Out.Printf(Out.PriHigh, "Skipping CheckSubsystemIds test.\n");
      return OK;
   }

   if
   (
         (g_SubsytemId.Vid != Bus.SubsystemVendorId)
      || (g_SubsytemId.Did != Bus.SubsystemDeviceId)
   )
   {
      Out.Printf(Out.PriHigh, "Subsystem VID = %#06x; expected %#06x.\n", Bus.SubsystemVendorId, g_SubsytemId.Vid);
      Out.Printf(Out.PriHigh, "Subsystem DID = %#06x; expected %#06x.\n", Bus.SubsystemDeviceId, g_SubsytemId.Did);
      return RC.INCORRECT_SUBSYSTEM_ID;
   }

   return OK;
}

//------------------------------------------------------------------------------
// Check if the TV encoder matches the expected value.
//
function CheckTvEncoderType()
{
   if (null == g_TvEncoder)
   {
      Out.Printf(Out.PriHigh, "Skipping CheckTvEncoderType test.\n");
      return OK;
   }

   if (g_TvEncoder != TvEncoder)
   {
      Out.Printf(Out.PriHigh, "TV encoder = %s; expected %s.\n", TvEncoder, g_TvEncoder);
      return RC.INCORRECT_TV_ENCODER;
   }

   return OK;
}


//------------------------------------------------------------------------------
// Check if the thermal sensor is present and not totally broken.
//
function CheckThermalSanity()
{
   if (0 == Master.TEMP_PRESENT)
   {
      Out.Printf(Out.PriHigh, "Skipping CheckThermalSanity test, no sensor detected.\n");
      return OK;
   }

   if ( (0 == Master.TEMP_LOCAL_HIGH_LIMIT) ||
        (0 == Master.TEMP_LOCAL_DEFAULT_HIGH_LIMIT) ||
        (0 == Master.TEMP_LOCAL_MAX_HIGH_LIMIT) ||
        (0 == Master.TEMP_REMOTE_HIGH_LIMIT) ||
        (0 == Master.TEMP_REMOTE_DEFAULT_HIGH_LIMIT) ||
        (0 == Master.TEMP_REMOTE_MAX_HIGH_LIMIT) )

   {
      Out.Printf(Out.PriHigh, "Failing CheckThermalSanity!\n");

      if (0 == Master.TEMP_LOCAL_HIGH_LIMIT)
         Out.Printf(Out.PriHigh, " Local high limit:     %d\n",
                    Master.TEMP_LOCAL_HIGH_LIMIT);

      if (0 == Master.TEMP_LOCAL_DEFAULT_HIGH_LIMIT)
         Out.Printf(Out.PriHigh, " Local default limit:  %d\n",
                    Master.TEMP_LOCAL_DEFAULT_HIGH_LIMIT);

      if (0 == Master.TEMP_LOCAL_MAX_HIGH_LIMIT)
         Out.Printf(Out.PriHigh, " Local max limit:      %d\n",
                    Master.TEMP_LOCAL_MAX_HIGH_LIMIT);

      if (0 == Master.TEMP_REMOTE_HIGH_LIMIT)
         Out.Printf(Out.PriHigh, " Remote high limit:    %d\n",
                    Master.TEMP_REMOTE_HIGH_LIMIT);

      if (0 == Master.TEMP_REMOTE_DEFAULT_HIGH_LIMIT)
         Out.Printf(Out.PriHigh, " Remote default limit: %d\n",
                    Master.TEMP_REMOTE_DEFAULT_HIGH_LIMIT);

      if (0 == Master.TEMP_REMOTE_MAX_HIGH_LIMIT)
         Out.Printf(Out.PriHigh, " Remote max limit:     %d\n",
                    Master.TEMP_REMOTE_MAX_HIGH_LIMIT);

      Out.Printf(Out.PriHigh, " Do you have the correct BIOS?\n");
      return RC.SOFTWARE_ERROR;
   }

   if ( (Master.TEMP_LOCAL >= Master.TEMP_LOCAL_HIGH_LIMIT) ||
        (Master.TEMP_REMOTE >= Master.TEMP_REMOTE_HIGH_LIMIT) )
   {
      Out.Printf(Out.PriHigh, "Failing CheckThermalSanity!\n");
      Out.Printf(Out.PriHigh, " Chip reports overheating.\n");
      Out.Printf(Out.PriHigh, " Local: %4d, Limit: %4d\n",
                 Master.TEMP_LOCAL, Master.TEMP_LOCAL_HIGH_LIMIT);
      Out.Printf(Out.PriHigh, " Remote:%4d, Limit: %4d\n",
                 Master.TEMP_REMOTE, Master.TEMP_REMOTE_HIGH_LIMIT);
      Out.Printf(Out.PriHigh, " Thermal diode is broken or heatsink is removed.\n");

      return RC.THERMAL_OVERTEMP;
   }

   return OK;
}

//------------------------------------------------------------------------------
// JavaScript wrapper for the PowerPusher test
//
function JsPowerPusher()
{
   var rc;

   // Run PowerPusher
   rc = PowerPusher.Run();
   if (OK != rc)
      return rc;

   // Is there a thermal sensor?
   if (0 == Master.TEMP_PRESENT)
   {
      // No thermal sensor
      Out.Printf(Out.PriLow, "JsPowerPusher: no thermal sensor detected, returning OK.\n");
      return OK;
   }

   // Has PowerPusher caused the chip to overheat?
   if ( (Master.TEMP_LOCAL >= Master.TEMP_LOCAL_HIGH_LIMIT) ||
        (Master.TEMP_REMOTE >= Master.TEMP_REMOTE_HIGH_LIMIT) )
   {
      Out.Printf(Out.PriHigh, "Failing JsPowerPusher!\n");
      Out.Printf(Out.PriHigh, " Chip reports overheating after running PowerPusher.\n");
      Out.Printf(Out.PriHigh, " Local: %4d, Limit: %4d\n",
                 Master.TEMP_LOCAL, Master.TEMP_LOCAL_HIGH_LIMIT);
      Out.Printf(Out.PriHigh, " Remote:%4d, Limit: %4d\n",
                 Master.TEMP_REMOTE, Master.TEMP_REMOTE_HIGH_LIMIT);

      return RC.THERMAL_OVERTEMP;
   }

   return OK;
}

//------------------------------------------------------------------------------
// Run MATS doing byte read/writes instead of dword read/writes.
//
function ByteTest()
{
   Mats.DataWidth = 8;
   Mats.End       = 1;  // Only test first megabyte.
   Mats.TestType  = Mats.LinearMats;

   var rc = Mats.Run();

   // Restore original state.
   Mats.DataWidth = 32;
   Mats.End       =  0;
   Mats.TestType  = Mats.BoxMats;

   return rc;
}

//------------------------------------------------------------------------------
// Run March test
//
function MarchTest()
{
   Mats.TestType  = Mats.March;

   var rc = Mats.Run();

   // Restore original state.
   Mats.TestType  = Mats.BoxMats;

   return rc;
}

//------------------------------------------------------------------------------
// Get the display configuration.
//
function GetDisplayConfig()
{
   var rc;

   var Configs = new Array;
   CHECK_RC(Display.GetConfigs(Configs));

   //
   // Configs =
   //    [
   //       [displays config 0, [head assignment 0, head assignment 1, ...]],
   //       [displays config 1, [head assignment 0, head assignment 1, ...]],
   //       ...
   //    ]
   //
   for (var i in Configs)
   {
      Out.Printf(Out.PriNormal, "%#010x : ", Configs[i][0]);
      for (var h in Configs[i][1])
      {
         Out.Printf(Out.PriNormal, "%d ", Configs[i][1][h]);
      }
      Out.Printf(Out.PriNormal, "\n");
   }

   return OK;
}

//------------------------------------------------------------------------------
// Program the I2C ROM on cards with tv-tuner capability
//
function ProgramTvRom()
{
   var rc;
   return ProgramTvMemory();
}

//------------------------------------------------------------------------------
// Run the specified display test on each display config.
//
function RunDisplayTests(DisplayTest)
{
   var rc = OK;

   var OriginalDisplay = Display.Selected;
   Out.Printf(Out.PriDebug, "Original display = %#010x.\n", OriginalDisplay);

   var Configs = new Array;
   CHECK_RC(Display.GetConfigs(Configs));

   Cleanup:
   for (var i in Configs)
   {
      CHECK_RC_CLEANUP(Display.Select(Configs[i][0]));
      Log.Next = false;
      CHECK_RC_CLEANUP(DisplayTest());
   }

   // Go back to original display.
   Display.Select(OriginalDisplay);
   Display.SetMode(640, 480, 32, 60);

   // Go back to text mode for user display.
   if ("Dos" == OperatingSystem)
   {
      if (!Vga.MONOCHROME)
      {
         Vga.SetVesaMode(3);
      }
   }

   return rc;
}

//------------------------------------------------------------------------------
// Display a white-red-green-blue diagonal stripe image on the selected display(s).
//
function CheckDisplay()
{
   var rc = OK;

   var Width       = (g_DisplayMode == null) ? 1024 : g_DisplayMode.Width;
   var Height      = (g_DisplayMode == null) ?  768 : g_DisplayMode.Height;
   var Depth       = (g_DisplayMode == null) ?   32 : g_DisplayMode.Depth;
   var RefreshRate = (g_DisplayMode == null) ?   60 : g_DisplayMode.RefreshRate;

   var DisplayInfo = new Array;
   var DisplayIdx  = 0;

   var Pitch = new Array;
   var i;
   for (i = 0; i < 24; ++i)
   {
      if (Display.Selected & (1 << i))
      {
         var Resolution = new Array(Width, Height);

         // If we are diplaying on a TV, run at TV's max resolution...unless
         // the user supplied an explicit resolution.
         if(
              ((1<<i) & 0x00FF00) &&
              (g_DisplayMode == null)
           )
         {
            CHECK_RC(Display.GetNativeResolution(1 << i, Resolution));
         }

         CHECK_RC(Display.SetMode(1 << i, Resolution[0], Resolution[1], Depth,
            RefreshRate, Display.FosTapsNone, Display.ColorCompressionNone,
            g_DdScalerMode));

         Display.GetPitch(1 << i, Pitch);

         // Store the data for this road as an object.
         DisplayInfo[DisplayIdx++] = { Display: 1<<i,
                                       Width:   Resolution[0],
                                       Height:  Resolution[1],
                                       Depth:   Depth,
                                       Refresh: RefreshRate,
                                       Pitch:   Pitch[0] };
      }
   }

   for (i = 0; i < DisplayIdx; ++i)
   {
      // Set the DAC start address for this screen
      var ScanoutOffset = i ? DisplayInfo[0].Pitch * DisplayInfo[0].Height : 0;
      Display.SetImageOffset(DisplayInfo[i].Display, ScanoutOffset);

      // Fill the primary surface with a white-red-green-blue diagonal stripe image.
      Memory.FillRgb(Bus.FbBaseAddress + ScanoutOffset,
                     DisplayInfo[i].Width,
                     DisplayInfo[i].Height,
                     DisplayInfo[i].Depth,
                     DisplayInfo[i].Pitch );

      // Add description messages.
      var Scale = Math.floor(DisplayInfo[i].Width / 512);

      PutCharsHorizCentered(DisplayInfo[i].Width, DisplayInfo[i].Height / 8 * 1.5, "white diagonal stripes", Scale, 0xFFFFFF, 0, DisplayInfo[i].Display);
      PutCharsHorizCentered(DisplayInfo[i].Width, DisplayInfo[i].Height / 8 * 3, "red diagonal stripes",   Scale, 0xFFFFFF, 0, DisplayInfo[i].Display);
      PutCharsHorizCentered(DisplayInfo[i].Width, DisplayInfo[i].Height / 8 * 5, "green diagonal stripes", Scale, 0xFFFFFF, 0, DisplayInfo[i].Display);
      PutCharsHorizCentered(DisplayInfo[i].Width, DisplayInfo[i].Height / 8 * 7, "blue diagonal stripes",  Scale, 0xFFFFFF, 0, DisplayInfo[i].Display);

      var TvString = "";
      if( DisplayInfo[i].Display & 0x00FF00 )
      {
         var Result = new Array;
         CHECK_RC( Display.GetTvOutType( DisplayInfo[i].Display, Result));
         TvString = Out.Sprintf(" (%s)", Result[0]);
      }

      var Message = Out.Sprintf("%dx%dx%d %dHz %#010x%s: Is image OK (y/n)?",
                                DisplayInfo[i].Width,
                                DisplayInfo[i].Height,
                                DisplayInfo[i].Depth,
                                DisplayInfo[i].Refresh,
                                DisplayInfo[i].Display,
                                TvString );

      // put the prompt a fraction of the way down the screen so that it shows
      // even on TVs that crop the top of the screen
      PutCharsHorizCentered(DisplayInfo[i].Width,
                            DisplayInfo[i].Height / 12,
                            Message,
                            Scale,
                            0xFFFFFF,
                            0,
                            DisplayInfo[i].Display );
   }

   // Check if image is OK.
   FlushInputBuffer();
   var Answer;
   do
   {
      Answer = GetChar().toLowerCase();
   } while ((Answer != "n") && (Answer != "y"));

   if ("n" == Answer)
   {
      return RC.BAD_DAC;
   }

   return OK;
}

function CheckDisplays()
{
   Log.Next = false;
   return RunDisplayTests(CheckDisplay);
}

//------------------------------------------------------------------------------
// Display a white-red-green-blue bars image on the selected displays(s).
//
function CheckDisplayBar()
{
   var rc = OK;

   var Width       = (g_DisplayMode == null) ? 1024 : g_DisplayMode.Width;
   var Height      = (g_DisplayMode == null) ?  768 : g_DisplayMode.Height;
   var Depth       = (g_DisplayMode == null) ?   32 : g_DisplayMode.Depth;
   var RefreshRate = (g_DisplayMode == null) ?   60 : g_DisplayMode.RefreshRate;

   // If we are diplaying on a TV, run at TV's max resolution.
   var i;
   for (i = 8; i < 16; ++i)
   {
      if (Display.Selected & (1 << i))
      {
         var Resolution = new Array;
         CHECK_RC(Display.GetMaxResolution(1 << i, Resolution));
         Width  = Resolution[0];
         Height = Resolution[1];
         break;
      }
   }

   // Set the modes.
   for (i = 0; i < 24; ++i)
   {
      if (Display.Selected & (1 << i))
      {
         CHECK_RC(Display.SetMode(1 << i, Width, Height, Depth, RefreshRate,
            Display.FosTapsNone, Display.ColorCompressionNone, g_DdScalerMode));
      }
   }

   // Get the pitch on the first display.
   var Pitch  = new Array;
   for (i = 0; i < 24; ++i)
   {
      if (Display.Selected & (1 << i))
      {
         Display.GetPitch(1 << i, Pitch);
         break;
      }
   }

   // Fill the primary surface with a white-red-green-blue bar image.
   Memory.FillRgbBars(Bus.FbBaseAddress, Width, Height, Depth, Pitch[0]);

   // Add description messages.
   var Scale = Math.floor(Width / 512);
   PutCharsHorizCentered(Width, Height / 8 * 1, "white bar", Scale);
   PutCharsHorizCentered(Width, Height / 8 * 3, "red bar",   Scale);
   PutCharsHorizCentered(Width, Height / 8 * 5, "green bar", Scale);
   PutCharsHorizCentered(Width, Height / 8 * 7, "blue bar",  Scale);
   var Message = Out.Sprintf("%dx%dx%d %dHz %#010x: Is image OK (y/n)?",
                     Width,
                     Height,
                     Depth,
                     RefreshRate,
                     Display.Selected);
   PutChars(0, 0, Message, Scale);

   // Check if image is OK.
   FlushInputBuffer();
   var Answer;
   do
   {
      Answer = GetChar().toLowerCase();
   } while ((Answer != "n") && (Answer != "y"));

   if ("n" == Answer)
   {
      return RC.BAD_DAC;
   }

   return OK;
}

function CheckDisplayBars()
{
   Log.Next = false;
   return RunDisplayTests(CheckDisplayBar);
}

//------------------------------------------------------------------------------
// Run the specified display test on each digital (flat panel) display.
//
function RunFpDisplayTests(DisplayTest)
{
   var rc;
   var Width, Height, Depth, RefreshRate;

   var OriginalDisplay = Display.Selected;
   Out.Printf(Out.PriDebug, "Original display = %#010x.\n", OriginalDisplay);

   var Detected = new Array;
   CHECK_RC(Display.GetDetected(Detected));

   Cleanup:
   for (var i = 16; i < 24; ++i)
   {
      if ((Detected[0] & (1 << i)) == 0)
      {
         continue;
      }

      CHECK_RC_CLEANUP(Display.Select(1 << i));

      // Run the test at the panel's native resolution if the user did not specify the mode.
      if (null == g_DisplayMode)
      {
         var Resolution = new Array;
         Display.SetMode(640, 480, 32, 60);  // Make sure we can talk to the panel.
         CHECK_RC_CLEANUP(Display.GetMaxResolution(1 << i, Resolution));
         Width       = Resolution[0];
         Height      = Resolution[1];
         Depth       = 32;
         RefreshRate = 60;
      }
      else
      {
         Width       = g_DisplayMode.Width;
         Height      = g_DisplayMode.Height;
         Depth       = g_DisplayMode.Depth;
         RefreshRate = g_DisplayMode.RefreshRate;
      }

      // Set the mode.
      CHECK_RC(Display.SetMode(1 << i, Width, Height, Depth, RefreshRate,
         Display.FosTapsNone, Display.ColorCompressionNone, g_DdScalerMode));

      // Get the pitch.
      var Pitch = new Array;
      Display.GetPitch(1 << i, Pitch);

      // Run the display test.
      Log.Next = false;
      CHECK_RC_CLEANUP(DisplayTest(Width, Height, Depth, RefreshRate, Pitch[0]));
   }

   // Go back to original display.
   Display.Select(OriginalDisplay);
   Display.SetMode(640, 480, 32, 60);

   // Go back to text mode for user display.
   if ("Dos" == OperatingSystem)
   {
      if (!Vga.MONOCHROME)
      {
         Vga.SetVesaMode(3);
      }
   }

   return rc;
}

//------------------------------------------------------------------------------
// Display a "special" gray image on the selected display.
//
function CheckDisplayGray
(
   Width,
   Height,
   Depth,
   RefreshRate,
   Pitch
)
{
   var rc;

   CHECK_RC(Memory.ReadTga("gray1280.tga", Bus.FbBaseAddress, Width, Height, Depth, Pitch));

   // Put a prompt on the screen
   var Scale = Math.floor(Width / 512);
   var Message = Out.Sprintf("%dx%dx%d %dHz, %5.1f MHz, %#010x: Is image OK (y/n)?",
                     Width,
                     Height,
                     Depth,
                     RefreshRate,
                     Dac.GetPixelClock(Display.Selected) / 1e6,
                     Display.Selected);
   PutChars(0, 0, Message, Scale);

   // Check if image is OK.
   FlushInputBuffer();
   var Answer;
   do
   {
      Answer = GetChar().toLowerCase();
   } while ((Answer != "n") && (Answer != "y"));

   if ("n" == Answer)
   {
      return RC.BAD_DAC;
   }

   return OK;
}

function CheckFpGray()
{
   Log.Next = false;
   return RunFpDisplayTests(CheckDisplayGray);
}

//------------------------------------------------------------------------------
// Display a "special" stripe image on the given head and display type.
//
function CheckDisplayStripes
(
   Width,
   Height,
   Depth,
   RefreshRate,
   Pitch
)
{
   var rc;

   // Fill the primary surface with a stripe image.
   Memory.FillStripes(Bus.FbBaseAddress, Width, Height, Depth, Pitch);

   // Add description messages.
   var Scale = Math.floor(Width / 512);
   PutCharsHorizCentered(Width, Height / 12 *  1, "red even",   Scale);
   PutCharsHorizCentered(Width, Height / 12 *  3, "green even", Scale);
   PutCharsHorizCentered(Width, Height / 12 *  5, "blue even",  Scale);
   PutCharsHorizCentered(Width, Height / 12 *  7, "red odd",    Scale);
   PutCharsHorizCentered(Width, Height / 12 *  9, "green odd",  Scale);
   PutCharsHorizCentered(Width, Height / 12 * 11, "blue odd",   Scale);

   var Message = Out.Sprintf("%dx%dx%d %dHz, %5.1f MHz, %#010x: Is image OK (y/n)?",
                     Width,
                     Height,
                     Depth,
                     RefreshRate,
                     Dac.GetPixelClock(Display.Selected) / 1e6,
                     Display.Selected);
   PutChars(0, 0, Message, Scale);

   // Check if image is OK.
   FlushInputBuffer();
   var Answer;
   do
   {
      Answer = GetChar().toLowerCase();
   } while ((Answer != "n") && (Answer != "y"));

   if ("n" == Answer)
   {
      return RC.BAD_DAC;
   }

   return OK;
}

function CheckFpStripes()
{
   Log.Next = false;
   return RunFpDisplayTests(CheckDisplayStripes);
}

//------------------------------------------------------------------------------
// Test the internal TV encoder.
//
function CheckTvo()
{
   var rc;

   // Check that we have an NVIDIA TV encoder.
   if (TvEncoder.search("NVIDIA") == -1)
   {
      Out.Printf(Out.PriHigh, "You can only run the TVO test on an NVIDIA TV encoder.\n");
      return RC.INCORRECT_TV_ENCODER;
   }

   // Check that a TV is connected.
   var Detected = new Array;
   CHECK_RC(Display.GetDetected(Detected));
   if ((Detected[0] & 0x0000FF00) == 0)
   {
      Out.Printf(Out.PriHigh, "TV is not connected.\n");
      return RC.INVALID_DISPLAY_MASK;
   }

   var OriginalDisplay = Display.Selected;
   Out.Printf(Out.PriDebug, "Original display = %#010x.\n", OriginalDisplay);

   var Configs = new Array;
   CHECK_RC(Display.GetConfigs(Configs));

   var Modes =
      [
         {Width: 640, Height:480, Depth:32},
         {Width: 800, Height:600, Depth:32},
         {Width:1024, Height:768, Depth:32},
      ];

   Cleanup:
   for (var i in Configs)
   {
      // Only run the test on TV's.
      if ((Configs[i][0] & 0x0000FF00) == 0)
         continue;

      CHECK_RC_CLEANUP(Display.Select(Configs[i][0]));

      // Run the test at each of the specified resolutions.
      for (var j in Modes)
      {
         var Mode = Modes[j];

         CHECK_RC_CLEANUP(Display.SetMode(Mode.Width, Mode.Height, Mode.Depth, 60));

         var Pitch = new Array;
         CHECK_RC_CLEANUP(Display.GetPitch(Display.Selected & 0x0000FF00, Pitch));
         Memory.FillRgb(Bus.FbBaseAddress, Mode.Width, Mode.Height, Mode.Depth, Pitch[0]);

         // Measure the TVO CRCs two times and check if they match.
         var Crcs1 = new Array;
         CHECK_RC_CLEANUP(Dac.GetTvCrcValues(Display.Selected & 0x0000FF00, Crcs1));

         var Crcs2 = new Array;
         CHECK_RC_CLEANUP(Dac.GetTvCrcValues(Display.Selected & 0x0000FF00, Crcs2));

         // The CRC are not synced to any event and can come in at any order,
         // so we need to check all combinations.
         var TvoIsGood = false;
         for (var displacement = 0; displacement < Crcs1.length; ++displacement )
         {
            for (var component = 0; component < Crcs1.length; component++)
            {
               if( Crcs2[(component + displacement) % Crcs1.length] != Crcs1[component] )
               {
                  break;
               }
            }

            // Did all components match?
            if (component == Crcs1.length)
            {
               TvoIsGood = true;
               break;
            }
         }
         if (!TvoIsGood)
         {
            Out.Printf(Out.PriLow, "TVO CRCs = [%s], [%s]\n", Crcs1, Crcs2);
            rc = RC.TVO_IS_BAD;
            break Cleanup;
         }

      } // for each mode

      // WAR for bug 78877, switch from VGA+TV to LVDS+TV fails first time.
      Display.Select(OriginalDisplay);
      Display.SetMode(640, 480, 32, 60);

   } // for each display config

   // Go back to original display.
   Display.Select(OriginalDisplay);
   Display.SetMode(640, 480, 32, 60);

   // Go back to text mode for user display.
   if ("Dos" == OperatingSystem)
   {
      if (!Vga.MONOCHROME)
      {
         Vga.SetVesaMode(3);
      }
   }

   return rc;
}

function CheckFrameLock()
{
   var rc;

   if (undefined == FrameLockTest.TestConfiguration)
      FrameLockTest.TestConfiguration = new Object;

   FrameLockTest.TestConfiguration.DisplayWidth       = 640;
   FrameLockTest.TestConfiguration.DisplayHeight      = 480;
   FrameLockTest.TestConfiguration.DisplayDepth       =  32;
   FrameLockTest.TestConfiguration.DisplayRefreshRate =  60;

   FrameLockTest.IsCheckLock = g_IsFramelockLock;
   FrameLockTest.LostLockPercent = 0.10;
   FrameLockTest.LostLockFrame = 3;

   CHECK_RC( FrameLockTest.Initialize() );

   Cleanup:
   {
      CHECK_RC_CLEANUP( FrameLockTest.ResetTest() );
      CHECK_RC_CLEANUP( FrameLockTest.LoopbackTest() );
      CHECK_RC_CLEANUP( FrameLockTest.SyncTest(false, g_IsFramelockAuto ) );
      CHECK_RC_CLEANUP( FrameLockTest.SyncTest(true,  g_IsFramelockAuto ) );
   }

   FrameLockTest.Restore();

   return rc;
}


//------------------------------------------------------------------------------
// Quick SBIOS test.
//
function RunSBiosTest()
{
   /*
   BUS:0 DEV:1 FUNC:0
   VenID: 0x10DE
   DevID: 0x01B2
   PCI Config Space register offset: 0x4C (MCP_LEG_CFG_19_LDT_ISO_EN)

   Default value: 0x00000000
   Prod value: 0x0000FDDE
   */

   var DeviceId = Pci.Read16(0, 1, 0, 2);

   if( DeviceId != 0x01B2 )
   {
      Out.Printf(Out.PriHigh, "*** ERROR: Invalid DeviceId for ISA Bridge ***\n");
      return RC.INVALID_DEVICE_ID;
   }

   var Cfg19 =  Pci.Read32(0, 1, 0, 0x4c);
   Out.Printf(Out.PriLow, "ISA Bridge DeviceId = 0x%04x, Cfg19 = 0x%08x.\n",
      DeviceId, Cfg19);

   /*
   if( Cfg19 == 0 )
   {
      Out.Printf(Out.PriHigh, "*** ERROR: RunSBiosTest(): ISA Bridge Invalid CFG19 = 0x%0x ***\n", Cfg19);
      return RC.SBIOS_TEST_FAIL;
   }
   */

   // Check for this specific value for now.
   if( Cfg19 != 0x0000FDDE )
   {
      Out.Printf(Out.PriHigh, "*** ERROR: RunSBiosTest(): ISA Bridge Invalid CFG19 = 0x%0x ***\n", Cfg19);
      return RC.SBIOS_TEST_FAIL;
   }

   return OK;
}

//------------------------------------------------------------------------------
function RunSBiosTest()
{
   /*
   BUS:0 DEV:1 FUNC:0
   VenID: 0x10DE
   DevID: 0x01B2
   PCI Config Space register offset: 0x4C (MCP_LEG_CFG_19_LDT_ISO_EN)

   Default value: 0x00000000
   Prod value: 0x0000FDDE
   */

   var DeviceId = Pci.Read16(0, 1, 0, 2);

   if( DeviceId != 0x01B2 )
   {
      Out.Printf(Out.PriHigh, "*** ERROR: Invalid DeviceId for ISA Bridge ***\n");
      return RC.INVALID_DEVICE_ID;
   }

   var Cfg19 =  Pci.Read32(0, 1, 0, 0x4c);
   Out.Printf(Out.PriLow, "ISA Bridge DeviceId = 0x%04x, Cfg19 = 0x%08x.\n",
      DeviceId, Cfg19);

   /*
   if( Cfg19 == 0 )
   {
      Out.Printf(Out.PriHigh, "*** ERROR: RunSBiosTest(): ISA Bridge Invalid CFG19 = 0x%0x ***\n", Cfg19);
      return -1;
   }
   */

   // Check for this specific value for now.
   if( Cfg19 != 0x0000FDDE )
   {
      Out.Printf(Out.PriHigh, "*** ERROR: RunSBiosTest(): ISA Bridge Invalid CFG19 = 0x%0x ***\n", Cfg19);
      return RC.SBIOS_TEST_FAIL;
   }

   return OK;
}

//------------------------------------------------------------------------------
// Run GPU traces.
//
function RunGpuTraces()
{
   var rc = OK;
   CHECK_RC(Import("trace.js"));
   return TraceNV2A();
}

//------------------------------------------------------------------------------
// Shmoo GPU clock up to find the max freq that the resman internal stress test
// can run at, and patch the vbios to record this baseline information.
// The resource manager can use this information later to track changes in
// the board's performance.
//
function StressBaseline()
{
   // Allow RM to set appropriate voltages.
   Master.CORE_VOLTAGE_CHANGE_ALLOWED = true;

   // Go to graphics mode.
   CHECK_RC( Display.SetMode(640,480,32,60) );

   // Go to max 3d perf mode, to find our nvclk shmoo starting point.
   Master.PERF_MODE = RM_PERF_MODE_ALWAYS_3D_PERFORMANCE;
   var initialNvclk = Dac.GRAPHICS_CLOCK * 0.9;

   // Go to low-power 3d perf mode, to set a conservative mem clock.
   Master.PERF_MODE = RM_PERF_MODE_ALWAYS_3D_LOW_POWER;
   Class3097.RecoveryNvclkHz = Dac.GRAPHICS_CLOCK;

   // Set up to call the RM stress test.
   Class3097.TestConfiguration = new Object;
   Class3097.TestConfiguration.DisplayWidth        = 640;
   Class3097.TestConfiguration.DisplayHeight       = 480;
   Class3097.TestConfiguration.DisplayDepth        =  32;
   Class3097.TestConfiguration.DisplayRefreshRate  =  60;
   Class3097.TestConfiguration.DisableCrt          = true;
   Class3097.TestConfiguration.Loops               =   1;
   Class3097.TestConfiguration.UseTiledSurface     = false;
   Class3097.IgnoreSingleBitErrors                 = false;
   Class3097.SkipModeset                           = true;
   Log.Never("Class3097.Run");

   var SilentRunningStress = new Object;
   SilentRunningStress.Nvclk   = 0;
   SilentRunningStress.Errors  = 0;
   SilentRunningStress.Temp    = 0;

   // Shmoo nvclk up until failure (1+ bad pixels, twice, or a hang).
   Dac.GRAPHICS_CLOCK = initialNvclk;
   initialNvclk = Dac.GRAPHICS_CLOCK;
   var nvHz = initialNvclk;
   var rc   = 0;

   Out.Printf(Out.PriHigh, "Find max nvclk for resman internal stress test.\n");

   while (true)
   {
      // Run the test 3 times.
      // We have failed if any run hangs, or we get too many bad pixels.
      var numRuns = 3;
      var accumBadPixels = 0;
      var run;
      for (run = 0; run < numRuns; run++)
      {
         // Mystery WAR:
         // If we don't do the modesets here, the shmoo will stop for too many
         // bad pixels 20mhz or more earlier.
         Vga.SetVesaMode(3);
         Display.SetMode(640,480,32,60);

         rc = Class3097.Run();
         if ((rc != 0) && (rc != RC.STRESS_TEST_FAILED))
            break;
         accumBadPixels += Class3097.Errors;
      }
      if ((rc != 0) && (rc != RC.STRESS_TEST_FAILED))
         break;

      if (accumBadPixels/numRuns > 1.0)
         break;

      // This nvclk did not fail.  Record the data.
      SilentRunningStress.Nvclk  = nvHz / (10*1000);     // vbios stores clocks div 10k
      SilentRunningStress.Errors = accumBadPixels / numRuns;
      SilentRunningStress.Temp   = Master.TEMP_REMOTE;

      Out.Printf(Out.PriHigh, "  OK at nvclk %.1f  temp %d deg C\n",
         nvHz / 1e6,
         SilentRunningStress.Temp);

      // Set next nvclk that is at least 2mhz greater than current.
      var targetHz = nvHz + 2 * 1000*1000;
      var probeHz  = targetHz;
      while (nvHz < targetHz)
      {
         Dac.GRAPHICS_CLOCK = probeHz;
         nvHz               = Dac.GRAPHICS_CLOCK;
         probeHz += 0.5 * 1000*1000;
      }
   }

   // Restore safe clocks.
   Master.PERF_MODE = RM_PERF_MODE_ALWAYS_2D;

   if (0 == SilentRunningStress.Nvclk)
   {
      if (!rc)
         rc = RC.STRESS_TEST_FAILED;
      return rc;
   }

   // Write out a vbios patch specification.

   var oldOutFile = Out.FileName;
   Out.Open("baseline.txt", Out.Truncate);

   Out.Printf(Out.PriHigh, "SILENTRUNNINGSTRESSINFO_10=\n");
   Out.Printf(Out.PriHigh, " db 07h\n");        // size
   Out.Printf(Out.PriHigh, " db 11h\n");        // version
   Out.Printf(Out.PriHigh, " dd %xh\n",         // nvclk divided by 10000
         SilentRunningStress.Nvclk);
   Out.Printf(Out.PriHigh, " db %xh\n",         // errs
         SilentRunningStress.Errors);
   Out.Printf(Out.PriHigh, "END\n");

   Out.Printf(Out.PriHigh, "BOARDINFO_10=\n");
   Out.Printf(Out.PriHigh, " db 11h\n");        // version
   Out.Printf(Out.PriHigh, " db 00h\n");        // Core Speedometer setting during production (placeholder)
   Out.Printf(Out.PriHigh, " db 00h\n");        // IO Speedometer setting during production  (placeholder)
   Out.Printf(Out.PriHigh, " db %xh\n",         // temperature (degrees C)
         SilentRunningStress.Temp);
   Out.Printf(Out.PriHigh, " db 00h\n");        // Pad drive strength during production (placeholder)
   Out.Printf(Out.PriHigh, " db 00h\n");        // Calibration Resistor setting  (placeholder)
   Out.Printf(Out.PriHigh, "\n");

   Out.Open(oldOutFile, Out.Append);

   Out.Printf(Out.PriHigh,
         "Baseline stresstest performance: %d errors at %.1f Mhz nvclk, %d deg C.\n",
         SilentRunningStress.Errors,
         SilentRunningStress.Nvclk / 100,
         SilentRunningStress.Temp);
   return 0;
}


//------------------------------------------------------------------------------
// MCP tests.
//
function RunSystemMemoryTest()
{
   var rc = OK;

   //
   // Initialize the GPU hardware because SysMats uses GPU functionality.
   //
   if (!g_IsGpuInitialized)
   {
      CHECK_RC(InitializeGpuTests());
   }

   // Run the system memory test.
   return SysMats.Run();
}

function RunAtaTest()
{
   var rc = OK;

   CHECK_RC( Ata.Initialize(g_AtaClassCode) );

   // Ata test locations for all the drives
   AtaTest.Cylinder   = 0x1000;
   AtaTest.Head       = 0x0;
   AtaTest.Sector     = 0x10;
   AtaTest.NumSectors = 1;

   //RunRegular Ata Test when Controller A Secondary Master
   Out.Printf(Out.PriNormal, "\tAtaTest for ATA.\n");
   CHECK_RC( Ata.SetDevice(0) );

   AtaTest.Channel    = Ata.Secondary;
   AtaTest.Drive      = Ata.Master;
   //AtaTest.IsInterrupt=true;
   CHECK_RC( AtaTests(true) );

   if( ( (g_McpVersion == Mcp.CK8)
         || (g_McpVersion == Mcp.CK8G) )
       &&(g_IsSAta) )
   {
      // Set to secondary ATA controller
      Out.Printf(Out.PriNormal, "\n\tAtaTest for SATA\n");
      CHECK_RC( Ata.SetDevice(1) );

      AtaTest.Channel    = Ata.Primary;
      AtaTest.Drive      = Ata.Master;
      CHECK_RC( AtaTests(false) );
   }
   return OK;
}

function AtaTests(IsAtaMDma)
{
   CHECK_RC(AtaTest.Initialize());

   Cleanup:
   {
      CHECK_RC_CLEANUP(AtaTest.Diag20());
      CHECK_RC_CLEANUP(AtaTest.Diag21());
      CHECK_RC_CLEANUP(AtaTest.Diag22());

      // PIO mode.
      CHECK_RC_CLEANUP(AtaTest.Diag25(0x8));
      CHECK_RC_CLEANUP(AtaTest.Diag25(0x9));
      CHECK_RC_CLEANUP(AtaTest.Diag25(0xa));
      CHECK_RC_CLEANUP(AtaTest.Diag25(0xb));
      CHECK_RC_CLEANUP(AtaTest.Diag25(0xc));
      CHECK_RC_CLEANUP(AtaTest.Diag26(0x8));
      CHECK_RC_CLEANUP(AtaTest.Diag26(0x9));
      CHECK_RC_CLEANUP(AtaTest.Diag26(0xa));
      CHECK_RC_CLEANUP(AtaTest.Diag26(0xb));
      CHECK_RC_CLEANUP(AtaTest.Diag26(0xc));
      CHECK_RC_CLEANUP(AtaTest.Diag27(0x8));
      CHECK_RC_CLEANUP(AtaTest.Diag27(0x9));
      CHECK_RC_CLEANUP(AtaTest.Diag27(0xa));
      CHECK_RC_CLEANUP(AtaTest.Diag27(0xb));
      CHECK_RC_CLEANUP(AtaTest.Diag27(0xc));

      // DMA read.
      if(IsAtaMDma)
      {
         CHECK_RC_CLEANUP(AtaTest.Diag30());

         CHECK_RC_CLEANUP(AtaTest.Diag31(16));
         CHECK_RC_CLEANUP(AtaTest.Diag31(64));
         CHECK_RC_CLEANUP(AtaTest.Diag31(128));
         CHECK_RC_CLEANUP(AtaTest.Diag36(32));
         CHECK_RC_CLEANUP(AtaTest.Diag36(256));

         CHECK_RC_CLEANUP(AtaTest.Diag35());
         CHECK_RC_CLEANUP(AtaTest.Diag37(0x20));
         CHECK_RC_CLEANUP(AtaTest.Diag37(0x21));
         CHECK_RC_CLEANUP(AtaTest.Diag37(0x22));
      }

      CHECK_RC_CLEANUP(AtaTest.Diag37(0x40));
      CHECK_RC_CLEANUP(AtaTest.Diag37(0x41));
      CHECK_RC_CLEANUP(AtaTest.Diag37(0x42));

      // ATA100 DMA read.
      if( "Dos" == OperatingSystem )
      {
         CHECK_RC_CLEANUP(AtaTest.Diag37(0x43));
         CHECK_RC_CLEANUP(AtaTest.Diag37(0x44));
         CHECK_RC_CLEANUP(AtaTest.Diag37(0x45));

         // ATA133 DMA read.
         if( (g_McpVersion == Mcp.MCP2)
             || (g_McpVersion == Mcp.CK8)
             || (g_McpVersion == Mcp.CK8G)
             || (g_McpVersion == Mcp.MCP2S) )
            CHECK_RC_CLEANUP(AtaTest.Diag37(0x46));
      }

      // DMA write.
      if(IsAtaMDma)
      {
         CHECK_RC_CLEANUP(AtaTest.Diag40());
         CHECK_RC_CLEANUP(AtaTest.Diag41(16));
         CHECK_RC_CLEANUP(AtaTest.Diag41(64));
         CHECK_RC_CLEANUP(AtaTest.Diag41(128));
         CHECK_RC_CLEANUP(AtaTest.Diag46(32));
         CHECK_RC_CLEANUP(AtaTest.Diag46(256));

         CHECK_RC_CLEANUP(AtaTest.Diag45());
         CHECK_RC_CLEANUP(AtaTest.Diag47(0x20));
         CHECK_RC_CLEANUP(AtaTest.Diag47(0x21));
         CHECK_RC_CLEANUP(AtaTest.Diag47(0x22));
      }

      CHECK_RC_CLEANUP(AtaTest.Diag47(0x40));
      CHECK_RC_CLEANUP(AtaTest.Diag47(0x41));
      CHECK_RC_CLEANUP(AtaTest.Diag47(0x42));


      // ATA100 DMA write.
      if( "Dos" == OperatingSystem )
      {
         CHECK_RC_CLEANUP(AtaTest.Diag47(0x43));
         CHECK_RC_CLEANUP(AtaTest.Diag47(0x44));
         CHECK_RC_CLEANUP(AtaTest.Diag47(0x45));

         // ATA133 DMA write.
         if( (g_McpVersion == Mcp.MCP2)
             || (g_McpVersion == Mcp.CK8)
             || (g_McpVersion == Mcp.CK8G)
             || (g_McpVersion == Mcp.MCP2S) )
            CHECK_RC_CLEANUP(AtaTest.Diag47(0x46));
      }
   }

   AtaTest.UnInitialize();

   return rc;
}

function RunAtapiTest()
{
   var rc = OK;

   CHECK_RC( AtaPi.Initialize() );

   AtaPiTest.Channel   = Ata.Secondary;
   AtaPiTest.Drive     = Ata.Slave;   // 0x10
   AtaPiTest.NumBlocks = 3;

   //
   // CD-ROM/DVD-ROM Tests.
   //
   CHECK_RC(AtaPiTest.Initialize());
   CHECK_RC(AtaPiTest.GetVerifyValue());
   CHECK_RC(AtaPiTest.Diag10());
   CHECK_RC(AtaPiTest.Diag20(0x8));
   CHECK_RC(AtaPiTest.Diag20(0x9));
   CHECK_RC(AtaPiTest.Diag20(0xa));
   CHECK_RC(AtaPiTest.Diag20(0xb));
   CHECK_RC(AtaPiTest.Diag20(0xc));
   CHECK_RC(AtaPiTest.Diag30(0x20));
   CHECK_RC(AtaPiTest.Diag30(0x21));
   CHECK_RC(AtaPiTest.Diag30(0x22));
   CHECK_RC(AtaPiTest.Diag30(0x40));
   CHECK_RC(AtaPiTest.Diag30(0x41));
   //CHECK_RC(AtaPiTest.Diag30(0x42));
   //CHECK_RC(AtaPiTest.Diag30(0x43));
   //CHECK_RC(AtaPiTest.Diag10());

   return OK;
}

function RunFloppyTest()
{
   var TestFile = "a:/floppy.txt";

   //
   // Create a test file.
   //
   var f = new File(TestFile);
   f.open("text", "write");
   if (!f.isOpen)
   {
      Out.Printf(Out.PriHigh, "Failed to open floppy test file.\n");
      return RC.CANNOT_OPEN_FILE;
   }

   var TestString = "Floppy test line ";
   var NumLines   = 100;

   var i;
   for (var i = 0; i < NumLines; ++i)
   {
      f.writeln(TestString, i);
   }
   f.close();
   Out.Printf(Out.PriLow, "Wrote test file.\n");

   //
   // Read the test file.
   //
   f.open("text", "read");
   if (!f.isOpen)
   {
      Out.Printf(Out.PriHigh, "Failed to open floppy test file.\n");
      return RC.CANNOT_OPEN_FILE;
   }
   var Lines = f.readAll();
   f.close();
   Out.Printf(Out.PriLow, "Read test file.\n");

   //
   // Check results.
   //
   if (Lines.length != NumLines)
   {
      return RC.FLOPPY_TEST_FAILED;
   }
   for (i = 0; i < NumLines; ++i)
   {
      if ((TestString + i) != Lines[i])
      {
         return RC.FLOPPY_TEST_FAILED;
      }
   }

   return OK;
}

function RunNvEthernetTest()
{
   var rc = OK;

   CHECK_RC( NvEthernet.Initialize() );

   // Check register access before loading the driver.
   if (OK != (rc = NvEthernetTest.MemRegTest()))
   {
      NvEthernet.Restore();
      return rc;
   }

   // Load driver.
   if (OK != (rc = NvEthernet.Open()))
   {
      NvEthernet.Restore();
      return rc;
   }

   // Print MAC address.
   Out.Printf(Out.PriLow, "NVidia MAC Physical Node Address: %s\n", NvEthernet.NodeAddress);

   Cleanup:
   {
      // if loopback is specified without a link speed, choose 100Mbps
      if ( (g_NvEnetMacLoopback || g_NvEnetPhyLoopback)
           && (g_NvEnetLinkSpeed == null) )
         g_NvEnetLinkSpeed = 100000000;

      // override link speed if necessary
      if (g_NvEnetLinkSpeed != null)
      {
         CHECK_RC_CLEANUP(NvEthernet.SetLinkSpeed(g_NvEnetLinkSpeed));
      }

      if (g_NvEnetMacLoopback)
      {
         // also enable PHY loopback to set up link parameters
         CHECK_RC_CLEANUP(NvEthernet.SetPhyLoopback(1));
         CHECK_RC_CLEANUP(NvEthernet.SetMacLoopback(1));
      }
      else if (g_NvEnetPhyLoopback)
      {
         CHECK_RC_CLEANUP(NvEthernet.SetPhyLoopback(1));
      }
      else
      {
         CHECK_RC_CLEANUP(NvEthernet.WaitForLink(1000));
      }
      Out.Printf(Out.PriLow, "Phy Ctrl 0x%04x, Status 0x%04x\n", NvEthernet.PhyRegRd(0), NvEthernet.PhyRegRd(1));

      //
      // Ethernet Tests.
      //

      CHECK_RC_CLEANUP(NvEthernetTest.InterruptTest());
      CHECK_RC_CLEANUP(NvEthernetTest.LoopbackTest());
      CHECK_RC_CLEANUP(NvEthernetTest.BurstTest());
   }

   // Unload driver.
   NvEthernet.Close();

   // Deinitialize device.
   NvEthernet.Restore();

   return rc;
}

function RunTcomEthernetTest()
{
   var rc = OK;

   CHECK_RC( TcomEthernet.Initialize() );

   // Check register access before loading the driver.
   Cleanup:
   {
      CHECK_RC_CLEANUP(TcomEthernetTest.MemRegTest());
      CHECK_RC_CLEANUP(TcomEthernetTest.IoRegTest());
      CHECK_RC_CLEANUP(TcomEthernetTest.EepromTest());
   }

   // If any of the basic tests failed, exit here.
   if (OK != rc)
   {
      TcomEthernet.Restore();
      return rc;
   }

   // Load driver.
   if (OK != (rc = TcomEthernet.Open()))
   {
      TcomEthernet.Restore();
      return rc;
   }

   Out.Printf(Out.PriLow, "3COM MAC Physical Node Address: %s\n", TcomEthernet.NodeAddress);

   Cleanup:
   {
      // if loopback is specified without a link speed, choose 100Mbps
      if ( (g_TcomEnetMacLoopback || g_TcomEnetPhyLoopback)
           && (g_TcomEnetLinkSpeed == null) )
         g_TcomEnetLinkSpeed = 100000000;

      // override link speed if necessary
      if (g_TcomEnetLinkSpeed != null)
         CHECK_RC_CLEANUP(TcomEthernet.SetLinkSpeed(g_TcomEnetLinkSpeed));

      if (g_TcomEnetMacLoopback)
      {
         // also enable PHY loopback to set up link parameters
         CHECK_RC_CLEANUP(TcomEthernet.SetPhyLoopback(1));
         CHECK_RC_CLEANUP(TcomEthernet.SetMacLoopback(1));
      }
      else if (g_TcomEnetPhyLoopback)
      {
         CHECK_RC_CLEANUP(TcomEthernet.SetPhyLoopback(1));
      }
      else
         CHECK_RC_CLEANUP(TcomEthernet.WaitForLink(5000));

      // Wait for some PHYs that advertise a valid link before it is reliable.
      Sleep(1000);

      //
      // Ethernet Tests.
      //

      CHECK_RC_CLEANUP(TcomEthernetTest.InterruptTest());
      CHECK_RC_CLEANUP(TcomEthernetTest.LoopbackTest());
      CHECK_RC_CLEANUP(TcomEthernetTest.BurstTest());
   }

   // Unload driver.
   TcomEthernet.Close();

   // Deinitialize device.
   TcomEthernet.Restore();

   return rc;
}

function RunHpnaPhyTest()
{
   var rc = OK;

   CHECK_RC( NvEthernet.Initialize() );

   Cleanup:
   {
      CHECK_RC_CLEANUP(NvEthernet.Open());

      var PhyCtrl, PhyStat, PhyId;

      PhyCtrl = NvEthernet.PhyRegRd(0);
      if( PhyCtrl!=0x0400 )
      {
         Out.Printf(Out.PriHigh, "*** ERROR: PhyReg(0) = 0x%04x MisMatch expected 0x0400\n",
                    PhyCtrl);
         CHECK_RC_CLEANUP(RC.HPNA_PHY_TEST_FAIL);
      }

      PhyStat = NvEthernet.PhyRegRd(1);
      if( PhyStat!=0x2841 )
      {
         Out.Printf(Out.PriHigh, "*** ERROR: PhyReg(1) = 0x%04x MisMatch expected 0x2841\n",
                    PhyStat);
         CHECK_RC_CLEANUP(RC.HPNA_PHY_TEST_FAIL);
      }

      PhyId = NvEthernet.PhyRegRd(2);
      if( PhyId!=0x32 )
      {
         Out.Printf(Out.PriHigh, "*** ERROR: PhyReg(2) = 0x%04x MisMatch expected 0x0032\n",
                    PhyId);
         CHECK_RC_CLEANUP(RC.HPNA_PHY_TEST_FAIL);
      }

      PhyId = NvEthernet.PhyRegRd(3);
      if( ( PhyId!=0xcc10 ) && ( PhyId!=0xcc13 ) )
      {
         Out.Printf(Out.PriHigh, "*** ERROR: PhyReg(3) = 0x%04x MisMatch expected 0xcc10 or 0xcc13\n",
                    PhyId);
         CHECK_RC_CLEANUP(RC.HPNA_PHY_TEST_FAIL);
      }
      Out.Printf(Out.PriHigh, "$$$ HpnaPhyTest() PASS $$$\n");
   }

   NvEthernet.Close();
   NvEthernet.Restore();

   return rc;
}

function RunAciTest()
{
   var rc = OK;

   CHECK_RC( Audio.Initialize() );

   AudioTest.Loops = 100;
   AudioTest.IsRight=true;
   AudioTest.IsLeft=true;
   AudioTest.IsMonoRecRight=false;

   //
   // Audio Tests.
   //
   CHECK_RC(AudioTest.Initialize());
   CHECK_RC(AudioTest.OutTest(0));
   CHECK_RC(AudioTest.Diag11());
   CHECK_RC(AudioTest.Diag13());

   // for MSI MBO6729 ONLY
   if(g_AciMbo6729)
   {
      Out.Printf(Out.PriNormal, "... Audio Test for Mbo6729 ...\n");

      Out.Printf(Out.PriNormal, "\tLineOut --> LineIn\n");
      CHECK_RC(AudioTest.LoopBack(AudioTest.LineIn));
      Out.Printf(Out.PriNormal, "\tSurr --> CdIn\n");
      CHECK_RC(AudioTest.LoopBackSurr(AudioTest.CdIn));

      // Using the L/R shorted cable to test Left/Right -> Mic
      // Play and Record from Right channel
      AudioTest.IsLeft=true;
      AudioTest.IsRight=false;
      Out.Printf(Out.PriNormal, "\tSurr (L) --> Mic\n");
      CHECK_RC(AudioTest.LoopBackSurr(AudioTest.Mic));
      // Play and Record from Right channel
      AudioTest.IsRight=true;
      AudioTest.IsLeft=false;
      AudioTest.IsMonoRecRight=true;
      Out.Printf(Out.PriNormal, "\tSurr (R) --> Mic\n");
      CHECK_RC(AudioTest.LoopBackSurr(AudioTest.Mic));

      // Reset the AudioTest to Normal mode.
      AudioTest.IsMonoRecRight=false;
      AudioTest.IsRight=true;
      AudioTest.IsLeft=true;
   }
   // for shuttle ONLY
   if(g_AciShuttle)
   {
      Out.Printf(Out.PriNormal, "... Audio Test for Shuttle ...\n");

      Out.Printf(Out.PriNormal, "\tLineOut --> LineIn\n");
      CHECK_RC(AudioTest.LoopBack(AudioTest.LineIn));
      Out.Printf(Out.PriNormal, "\tSurr --> CdIn\n");
      CHECK_RC(AudioTest.LoopBackSurr(AudioTest.CdIn));

      // Using the L/R shorted cable to test Left/Right -> Mic
      // Play and Record from Right channel
      AudioTest.IsLeft=true;
      AudioTest.IsRight=false;
      Out.Printf(Out.PriNormal, "\tSurr (L) --> Mic2\n");
      CHECK_RC(AudioTest.LoopBackSurr(AudioTest.Mic2));
      // Play and Record from Right channel
      AudioTest.IsRight=true;
      AudioTest.IsLeft=false;
      AudioTest.IsMonoRecRight=true;
      Out.Printf(Out.PriNormal, "\tSurr (R) --> Mic2\n");
      CHECK_RC(AudioTest.LoopBackSurr(AudioTest.Mic2));

      // Reset the AudioTest to Normal mode.
      AudioTest.IsMonoRecRight=false;
      AudioTest.IsRight=true;
      AudioTest.IsLeft=true;
   }
   else if(g_AciA7N8X)
   {
      Out.Printf(Out.PriNormal, "... Audio Test for Asus A7N8X-LA ...\n");

      Out.Printf(Out.PriNormal, "\tLineOut --> LineIn\n");
      CHECK_RC(AudioTest.LoopBack(AudioTest.LineIn));
      Out.Printf(Out.PriNormal, "\tLineOut --> Mic\n");
      CHECK_RC(AudioTest.LoopBack(AudioTest.Mic));
      Out.Printf(Out.PriNormal, "\tLineOut --> FrontMic/Mic2\n");
      CHECK_RC(AudioTest.LoopBack(AudioTest.Mic2));
      Out.Printf(Out.PriNormal, "\tLineOut --> AuxIn\n");
      CHECK_RC(AudioTest.LoopBack(AudioTest.AuxIn));

      Out.Printf(Out.PriNormal, "\tSpdif --> D/A --> CdIn\n");
      CHECK_RC(AudioTest.LoopBackSp(AudioTest.CdIn));
   }
   // use all channels
   else if(g_AciAll)
   {
      Out.Printf(Out.PriNormal, "... Audio Test All channel mode ...\n");

      Out.Printf(Out.PriNormal, "\tLineOut --> LineIn\n");
      CHECK_RC(AudioTest.LoopBack(AudioTest.LineIn));
      Out.Printf(Out.PriNormal, "\tLineOut --> Mic\n");
      CHECK_RC(AudioTest.LoopBack(AudioTest.Mic));
      Out.Printf(Out.PriNormal, "\tLineOut --> Phone\n");
      CHECK_RC(AudioTest.LoopBack(AudioTest.Phone));
      Out.Printf(Out.PriNormal, "\tSurr --> CdIn\n");
      CHECK_RC(AudioTest.LoopBackSurr(AudioTest.CdIn));
      Out.Printf(Out.PriNormal, "\tSurr --> AuxIn\n");
      CHECK_RC(AudioTest.LoopBackSurr(AudioTest.AuxIn));
   }
   // default mode
   else
   {
      Out.Printf(Out.PriNormal, "... Audio Test default mode ...\n");

      Out.Printf(Out.PriNormal, "\tLineOut --> LineIn\n");
      CHECK_RC(AudioTest.LoopBack(AudioTest.LineIn));

      if(g_AciMic)
      {
         Out.Printf(Out.PriNormal, "\tLineOut --> Mic\n");
         CHECK_RC(AudioTest.LoopBack(AudioTest.Mic));
      }

      if(g_AciFrontMic)
      {
         Out.Printf(Out.PriNormal, "\tLineOut --> FrontMic\n");
         CHECK_RC(AudioTest.LoopBack(AudioTest.Mic2));
      }

      if(g_AciCdIn)
      {
         Out.Printf(Out.PriNormal, "\tLineOut --> CdIn\n");
         CHECK_RC(AudioTest.LoopBack(AudioTest.CdIn));
      }

      if(g_AciAuxIn)
      {
         Out.Printf(Out.PriNormal, "\tLineOut --> AuxIn\n");
         CHECK_RC(AudioTest.LoopBack(AudioTest.AuxIn));
      }

      if(g_AciVideoIn)
      {
         Out.Printf(Out.PriNormal, "\tLineOut --> VideoIn\n");
         CHECK_RC(AudioTest.LoopBack(AudioTest.VideoIn));
      }

      if(g_AciPhone)
      {
         Out.Printf(Out.PriNormal, "\tLineOut --> Phone\n");
         CHECK_RC(AudioTest.LoopBack(AudioTest.Phone));
      }
   }

   // SpdifOut->SpdifIn for ALC650 Codec Only
   if(g_AciSpdifIn)
   {
      Out.Printf(Out.PriNormal, "\tSpdifOut --> SpdifIn\n");
      CHECK_RC(AudioTest.LoopBackSp(AudioTest.SpdifIn));
   }

   return OK;
}

function RunAcrAciTest()
{
   var rc = OK;

   CHECK_RC( Audio.Initialize() );

   //
   // Audio Tests.
   //
   CHECK_RC(AudioTest.Initialize());

   // Use the AuxIn Channel at source.
   CHECK_RC(AudioTest.LoopBackSurr(AudioTest.AuxIn));

   // Use the CdIn channel as source.
   CHECK_RC(AudioTest.LoopBackSurr(AudioTest.CdIn));

   return OK;
}

function RunAciSpdifTest()
{
   var rc = OK;

   CHECK_RC( Audio.Initialize() );
   Sleep(1000);

   // Audio Tests.
   AudioTest.Loops = 100;
   CHECK_RC(AudioTest.Initialize());

   // Sp channel loop test
   CHECK_RC( AudioTest.Diag13() );
   // Use the In channel as source.
   CHECK_RC( AudioTest.LoopBackSp(AudioTest.LineIn) );

   return OK;
}

function RunBayAciTest1()
{
   Out.Printf(Out.PriNormal,"\tLineOut (Acr) -> LineIn (Acr)\n");
   CHECK_RC( AudioTest.LoopBack(AudioTest.LineIn) );

   Out.Printf(Out.PriNormal,"\tLineOut (Acr) -> Mic (Acr)\n");
   CHECK_RC( AudioTest.LoopBack(AudioTest.Mic) );

   Out.Printf(Out.PriNormal,"\tLineOut (Acr) -> TAD (Acr)\n");
   CHECK_RC( AudioTest.LoopBack(AudioTest.Phone) );

   Out.Printf(Out.PriNormal,"\tCenter (Acr) -> CdIn(Acr)\n");
   CHECK_RC( AudioTest.LoopBackSurr(AudioTest.CdIn) );

   Out.Printf(Out.PriNormal,"\tRear (Acr) -> AuxIn(Bay)\n");
   CHECK_RC( AudioTest.LoopBackSurr(AudioTest.AuxIn) );

   Out.Printf(Out.PriNormal,"\tSpdifOut (Acr) -> SpdifIn (Bay)\n");
   CHECK_RC( AudioTest.LoopBackSp(AudioTest.VideoIn) );

   return OK;
}

function RunBayAciTest2()
{
   Out.Printf(Out.PriNormal,"\tHeadPhone (Bay) -> Mic2 (Bay)\n");
   CHECK_RC( AudioTest.LoopBack(AudioTest.Mic2) );

   Out.Printf(Out.PriNormal,"\tOpticalOut (Bay) -> OpticalIn (Bay)\n");
   CHECK_RC( AudioTest.LoopBackSp(AudioTest.VideoIn) );
   return OK;
}

function RunBayAciTest3()
{
   Out.Printf(Out.PriNormal,"\tRear (Acr) -> AuxIn(Bay)\n");
   CHECK_RC( AudioTest.LoopBackSurr(AudioTest.AuxIn) );

   Out.Printf(Out.PriNormal,"\tSpdifOut (Bay) -> SpdifIn (Bay)\n");
   CHECK_RC( AudioTest.LoopBackSp(AudioTest.VideoIn) );
   return OK;
}

function RunBayRemoteTest()
{
   RemoteTest.ExpKey = g_ExpRemoteKey;
   CHECK_RC( RemoteTest.DLightTest() );
   CHECK_RC( RemoteTest.PeakHoldTest() );
   CHECK_RC( RemoteTest.TestKey() );
   return OK;
}

function RunBayUsbTest()
{
   var rc = OK;

   // run test for Usb1.1
   CHECK_RC( UsbTest.Initialize() );
   CHECK_RC( UsbTest.Run(g_UsbAcrPort) );


   return OK;
}

function RunMciTest()
{
   var rc = OK;

   CHECK_RC( Modem.Initialize() );

   ModemTest.Loops = 100;

   //
   // Modem Tests.
   //
   CHECK_RC(ModemTest.Initialize());
   CHECK_RC(ModemTest.Diag11());
   // Loopback doesn't work
   //CHECK_RC(ModemTest.LoopBack(4));

   return OK;
}

function RunAcrMciTest()
{
   var rc = OK;

   CHECK_RC( Modem.Initialize() );

   ModemTest.Loops = 100;

   //
   // Modem Tests.
   //
   CHECK_RC(ModemTest.Initialize());
   CHECK_RC(ModemTest.Diag11());

   return OK;
}

function RunSmbusTest()
{
   var rc = OK;

   if( g_McpVersion != Mcp.MCP1)
   {
      Out.Printf(Out.PriHigh, "*** ERROR: Smbus1 Test CANNOT run on McpVersion %i ***\n", g_McpVersion);
      return RC.TEST_CANNOT_RUN;
   }
   CHECK_RC( Smbus.Initialize() );

   CHECK_RC(Smbus.HstSlvSnp(0, 0x20, 0xaa, 0x9876));

   if(g_SmbDoPort1)
      CHECK_RC(Smbus.HstSlvSnp(1, 0x32, 0xba, 0x1234));

   return OK;
}

function RunSmbus2Test()
{
   var rc = OK;

   if( (g_McpVersion != Mcp.MCP2)
       && (g_McpVersion != Mcp.CK8)
       && (g_McpVersion != Mcp.CK8G)
       && (g_McpVersion != Mcp.MCP2S) )
   {
      Out.Printf(Out.PriHigh, "*** ERROR: Smbus2 Test CANNOT run On McpVersion %i ***\n", g_McpVersion);
      return RC.TEST_CANNOT_RUN;
   }

   CHECK_RC( Smbus.Initialize() );
   CHECK_RC(Smbus.HstSlvSnp(0, 0x8, 0x20, 0xabcd));

   if(g_SmbDoPort1)
      CHECK_RC(Smbus.HstSlvSnp(1, 0x8, 0x30, 0x1234));

   return OK;
}

function RunApuTest()
{
   var rc = OK;

   //
   // Audio Trace
   //
   ApuTrace.BaseDir = ".\\aputrace\\"
   for (var i = 0; i < 65; ++i)
   {
      ApuTrace.TraceName = "Tst" + i;
      CHECK_RC(Apu.Create() );
      rc = ApuTrace.Run();
      Apu.Destroy();

      CHECK_RC(rc);
   }

   return OK;
}

function RunApuMats()
{
   var rc = OK;

   //
   // Audio SRAM Test
   //
   CHECK_RC(Apu.Create() );
   rc = ApuMats.Run();
   Apu.Destroy();

   return rc;
}


function RunUsbTest()
{
   var rc = OK;
   var Dev0 = [0,2,0];
   var Dev1 = [0,3,0];
   var PortMapOff = 0x50;

   if( (g_McpVersion == Mcp.MCP2)
       || (g_McpVersion == Mcp.CK8)
       || (g_McpVersion == Mcp.CK8G)
       || (g_McpVersion == Mcp.MCP2S) )
   {
      Usb.Free();
      Usb.CreateCrush11();
      Dev1 = [0,2,1];
   }

   var PortMapA = Pci.Read32(Dev0[0], Dev0[1], Dev0[2], PortMapOff);
   var PortMapB = Pci.Read32(Dev1[0], Dev1[1], Dev1[2], PortMapOff);

   var Ports = g_UsbPorts;
   UsbTest.IsInfo=false;


   // Check port mapping register in Pci Cfg for Mcp1
   if( g_McpVersion == Mcp.MCP1 )
   {
      if( (PortMapA|PortMapB)!= 0x3f )
      {
         Out.Printf(Out.PriHigh, "\n*** : BIOS DIDN'T initialize USB PortMap correctly ***\n");
         Pci.Write32(Dev0[0], Dev0[1], Dev0[2], PortMapOff, 0x7);
         Pci.Write32(Dev1[0], Dev1[1], Dev1[2], PortMapOff, 0x38);

         PortMapA = Pci.Read32(Dev0[0], Dev0[1], Dev0[2], PortMapOff);
         PortMapB = Pci.Read32(Dev1[0], Dev1[1], Dev1[2], PortMapOff);

         if( (PortMapA|PortMapB)!= 0x3f )
         {
            Out.Printf(Out.PriHigh, "*** ERROR: BIOS SET PortMap Wrong: PortMapA 0x%04x, PortMapB 0x%04x.\n",
                          PortMapA, PortMapB);
            return RC.USB_INVALID_PORT_MAP;
         }
      }
   }

   // Check port mapping register in Pci Cfg for Mcp2 and Ck8
   else if ( (g_McpVersion == Mcp.MCP2)
             || (g_McpVersion == Mcp.CK8)
             || (g_McpVersion == Mcp.CK8G) )
   {
      // Some hack to work around bios, Match port mapping in OHCI to EHCI
      if( (Pci.Read32(0,2,2,0x7c)&0x80000000)!=0 )
      {
         if( (PortMapA!=0x13)||(PortMapB!=0x2c) )
         {
            Out.Printf(Out.PriHigh, "\n*** WARNING: BIOS DIDN'T initialize USB PortMap correctly ***\n");
            Out.Printf(Out.PriHigh, "*** Mods did the Correction ***\n\n");
            Pci.Write32(Dev0[0], Dev0[1], Dev0[2], PortMapOff, 0x13);
            Pci.Write32(Dev1[0], Dev1[1], Dev1[2], PortMapOff, 0x2c);
            PortMapA = Pci.Read32(Dev0[0], Dev0[1], Dev0[2], PortMapOff);
            PortMapB = Pci.Read32(Dev1[0], Dev1[1], Dev1[2], PortMapOff);
            if( (PortMapA!=0x13)||(PortMapB!=0x2c) )
            {
               Out.Printf(Out.PriHigh, "*** ERROR: PortMap Wrong: PortMapA 0x%04x, PortMapB 0x%04x.\n",
                             PortMapA, PortMapB);
               return RC.USB_INVALID_PORT_MAP;
            }
         }
      }
      else
      {
         if( (PortMapA|PortMapB)!= 0x3f )
         {
            Out.Printf(Out.PriHigh, "*** ERROR: BIOS SET PortMap Wrong: PortMapA 0x%04x, PortMapB 0x%04x.\n",
                          PortMapA, PortMapB);
            return RC.USB_INVALID_PORT_MAP;
         }
      }
   }

   // For Mcp2S, no port mapping exist, since it is hardwired in hw.
   else if(g_McpVersion == Mcp.MCP2S)
   {
      if( (PortMapA!=0xf)||(PortMapB!=0xf) )
         Out.Printf(Out.PriHigh, "*** ERROR: OHCI is not fully enabled PortMapA 0x%x, PortMapB 0x%x\n",
                    PortMapA, PortMapB);
      return RC.USB_INVALID_PORT_MAP;
   }

   CHECK_RC( Usb.Initialize() );
   CHECK_RC( UsbTest.Initialize() );

   // Usb Test.
   for (var Port = 0; Port < Ports; ++Port)
   {
      CHECK_RC(UsbTest.Run(Port));
   }

   return OK;
}

function RunUsb2Test()
{
   var rc = OK;

   if( (g_McpVersion != Mcp.MCP2)
       && (g_McpVersion != Mcp.CK8)
       && (g_McpVersion != Mcp.CK8G)
       && (g_McpVersion != Mcp.MCP2S) )
   {
      Out.Printf(Out.PriHigh, "*** ERROR: Usb2Test CANNOT run on McpVersion %i ***\n", g_McpVersion);
      return RC.TEST_CANNOT_RUN;
   }
   Usb.Free();

   Usb.CreateCrush17();
   CHECK_RC( Usb.Initialize() );
   Sleep(100);

   // Usb Test.
   if(g_IsSpecialUsb2)
   {
      for (var Port = 0; Port < g_UsbPorts; ++Port)
      {
         CHECK_RC( Usb.ResetPort(Port) );
      }
   }

   for (var Port = 0; Port < g_UsbPorts; ++Port)
   {
      CHECK_RC(UsbTest.Run(Port));
   }

   return OK;
}

function RunAcrUsbTest()
{
    var rc = OK;

    if( (g_McpVersion == Mcp.MCP2)
        || (g_McpVersion == Mcp.CK8)
        || (g_McpVersion == Mcp.CK8G)
        || (g_McpVersion == Mcp.MCP2S) )
    {
        Usb.Free();
        Usb.CreateCrush11();
    }
    CHECK_RC( Usb.Initialize() );
    CHECK_RC( UsbTest.Initialize() );
    CHECK_RC( UsbTest.Run(g_UsbAcrPort) );

    return OK;
}

function RunAcrUsb2Test()
{
   var rc = OK;

   if( (g_McpVersion != Mcp.MCP2)
       && (g_McpVersion != Mcp.CK8)
       && (g_McpVersion != Mcp.CK8G)
       && (g_McpVersion != Mcp.MCP2S) )
   {
      Out.Printf(Out.PriHigh, "*** ERROR: Usb2Test CANNOT run on McpVersion %i ***\n", g_McpVersion);
      return RC.TEST_CANNOT_RUN;
   }

   Usb.Free();
   Usb.CreateCrush17();
   CHECK_RC( Usb.Initialize() );
   Sleep(300);

   CHECK_RC( UsbTest.Run(g_UsbAcrPort) );

   return OK;
}

function RunAcrEEPROMTest()
{
   var rc = OK;

   CHECK_RC( Smbus.Initialize() );

   var Port = 1;       // At Smbus Port1
   var Address = 0x58; // EEPROM Slave address on Acr card.
   var Offset = 0;
   var OrigLen = 0;
   var Len = 6; // 6 Bytes
   var Original = [];
   var WrArray = [0x41, 0x43, 0x51, 0x00, 0x00, 0x01]; // Means "ACR00"
   var RdBack1 = [];
   var RdBack2 = [];

   // Read the Origianl value in the EEPROM
   CHECK_RC( Smbus.ReadBlk(Port, Address, Offset, Len, Original) );
   //ArrayData.Print(Original);

   // Write the Write data, Read back and compare
   Cleanup:
   {
      CHECK_RC_CLEANUP( Smbus.WriteBlk(Port, Address, Offset, WrArray) );
      CHECK_RC_CLEANUP( Smbus.ReadBlk(Port, Address, Offset, Len, RdBack1) );
      //ArrayData.Print(RdBack1);
      CHECK_RC_CLEANUP( ArrayData.Compare(RdBack1, WrArray) );
   }

   // Write the Original data, Read back to make sure that it is good.
   CHECK_RC( Smbus.WriteBlk(Port, Address, Offset, Original) );
   CHECK_RC( Smbus.ReadBlk(Port, Address, Offset, Len, RdBack2) );
   //ArrayData.Print(RdBack2);
   CHECK_RC( ArrayData.Compare(RdBack2, Original) );

   return OK;
}

function RunFwTest()
{
   var rc = OK;

   // FireWire hardware is only present in MCP2 and in NV18 GPUs.
   if (((Gpu.IsInitialized && (Gpu.DeviceId == Gpu.NV18)) || (g_McpVersion==Mcp.MCP2)))
   {
      CHECK_RC( Fw.Initialize() );

      // work around to enable FW A03 fix, should been done in BIOS
      if(g_FwDeviceId == 0x6e)
      {
         var Bar1 = Pci.Read32(0,0xd,0,0x14);
         if( (Memory.Read32(Bar1)&0x80000000)==0 )
         {
            Out.Printf(Out.PriHigh, "\n*** WARNING: BIOS DIDN'T INITIALIZE FIREWIRE correctly ***\n");
            Out.Printf(Out.PriHigh, "*** Mods did the Correction ***\n\n");
            Memory.Write32(Bar1, Memory.Read32(Bar1)|0x80000000);
            if( (Memory.Read32(Bar1)&0x80000000)==0 )
            {
               Out.Print(Out.PriHigh, "*** ERROR: Cannot enable Fw Fix ***\n");
               return RC.FW_FIX_CANNOT_ENABLED;
            }
         }
      }

      CHECK_RC( Fw.SetDevice(Fw.GetDevice(g_FwDeviceId)) );

      // In loopback mode OHCIA Ports -- OHCIB Ports
      if(g_FwLoopback)
      {
         // Special OHCI multi-connection supported in AgerePhy
         if(g_FwMultiPhyLink)
         {
            Out.Printf(Out.PriNormal, "\tFwTest MultiPhyLink LOOPBACK mode.\n");
            // Only 2 OHCIs (One NV one Pci) are involved under the MultiLink mode,
            // reset g_FwDevices=1.
            g_FwDevices = 1;
            CHECK_RC( Fw.EnableOnePhyPort(0) );
            Sleep(1500);
         }
         // Normal OHCI single connection for all other phys
         else
         {
            Out.Printf(Out.PriNormal, "\tFwTest Normal LOOPBACK mode.\n");
            g_FwDevices = g_FwPorts;
         }
      }
      else
      {
         Out.Printf(Out.PriNormal, "\tFwTest Normal DEVICE mode.\n");
      }

      FwTest.IsInfo=false;
      FwTest.RdDWord = 3; // for AsyRdTest Only.
      FwTest.Loops = 10;
      FwTest.Irq = 0x9; // This is not used, no effect
      FwTest.ExpDevice = (g_FwDevices+1);
      FwTest.Device1= Fw.GetDevice(g_FwDeviceId);// Set the OHCI to be tested
      CHECK_RC( FwTest.Initialize() );
      CHECK_RC( FwTest.SelfIdTest() );

      if(!g_FwLoopback)
      {
         CHECK_RC( FwTest.PingTest() );
         //CHECK_RC( FwTest.RemoteAccessTest() );
         CHECK_RC( FwTest.ForceRootTest() );

         if(!g_FwSkipAsyRd)
            CHECK_RC( FwTest.AsyRdTest() );
      }
      CHECK_RC( FwTest.StoreValueTest() );

      // Mcp2
      if(g_FwDeviceId==0x006e)
      {
         CHECK_RC( FwTest.AsyStreamTest() );
         CHECK_RC( FwTest.IsoXmitTest() );
      }

      // Normal Loopback
      if( (g_FwLoopback)&&(!g_FwMultiPhyLink) )
      {
         // Offset that make sure the Device1!=Device2, and continues increment
         // the OHCI count by one, after Device1 is skipped.
         var Ext = 0;

         // g_FwPorts are the Number of Nv OHCI ports to be tested, also the
         // number of PCI OHCIs needed, excluding Nv OHCI.
         for(var i = 0; i<g_FwPorts; i++)
         {
            // Properly set the loopback OHCIs, needed for AMI BIOS
            if(FwTest.Device1==i)
               Ext = 1;
            FwTest.Device2 = i+Ext;
            FwTest.InitProperties();

            Out.Printf(Out.PriHigh, "... FW Loopback Test To PCI OHCI Device1 = %x, Device2 = %x ...\n",
                       FwTest.Device1, FwTest.Device2);

            CHECK_RC( FwTest.IsoLoopbackTest() );
            CHECK_RC( FwTest.AsyLoopbackTest() );
         }
      }

      // Multi Link Loopback
      if( (g_FwLoopback)&&(g_FwMultiPhyLink) )
      {
         // Properly set the loopback OHCIs, needed for AMI BIOS
         if(FwTest.Device1==0)
            FwTest.Device2 = 1;
         else
            FwTest.Device2 = 0;

         FwTest.InitProperties();
         Out.Printf(Out.PriHigh, "... FW Loopback Test Multi_Link OHCI Device1 = %x, Device2 = %x ...\n",
                    FwTest.Device1, FwTest.Device2);

         for(var i = 0; i<g_FwPorts; i++)
         {
            Out.Printf(Out.PriHigh, "......Test Port %i ......\n",i);
            // Need to switch port
            if(i!=0)
            {
               CHECK_RC( Fw.EnableOnePhyPort(i) );
               Sleep(1000);
               CHECK_RC( FwTest.Initialize() );
            }
            CHECK_RC( FwTest.IsoLoopbackTest() );
            CHECK_RC( FwTest.AsyLoopbackTest() );
         }  // End Test all ports

         // Re-enable all ports
         CHECK_RC( Fw.EnableAllPhyPort() );
      }

   }
   // End of test for NV18 and Mcp2
   else
   {
      Out.Printf(Out.PriNormal, "No firewire HW is present, FwTest not run.\n");
   }

   return OK;
}


function RunSerialTest()
{
   var rc = OK;

   for(var i = 0; i<g_SerialPorts; i++)
   {
      SerialTest.Port = (i+1);
      CHECK_RC(SerialTest.Initialize());

      // Serail port test.

      // Internal loopback.
      CHECK_RC(SerialTest.Diag10());
      CHECK_RC(SerialTest.Diag11());
      CHECK_RC(SerialTest.Diag15(0xff));

      // External loopback.
      CHECK_RC(SerialTest.Diag20());
      CHECK_RC(SerialTest.Diag21());
      CHECK_RC(SerialTest.Diag25(0xff));
   }

   return OK;
}

function RunParallelTest()
{
   var rc = OK;

   ParallelTest.Port = 1;
   CHECK_RC(ParallelTest.Initialize());

   // Parallel port test.

   // External loopback.
   CHECK_RC(ParallelTest.Diag10());
   CHECK_RC(ParallelTest.Diag11());
   CHECK_RC(ParallelTest.Diag20(0xff));

   return OK;
}

function RunRtcTest()
{
   var rc = OK;

   CHECK_RC(RtcTest.Initialize());

   Cleanup:
   {
      // Ram Test.
      CHECK_RC_CLEANUP( RtcTest.Diag10() );
      CHECK_RC_CLEANUP( RtcTest.Diag11() );
      CHECK_RC_CLEANUP( RtcTest.Diag12(50) );

      // Set Time Test
      /*
      CHECK_RC_CLEANUP( RtcTest.Diag20(5) );
      CHECK_RC_CLEANUP( RtcTest.Diag21(5) );
      CHECK_RC_CLEANUP( RtcTest.Diag22(5) );
      CHECK_RC_CLEANUP( RtcTest.Diag23(5) );
      */

      // Time Update test
      CHECK_RC_CLEANUP( RtcTest.Diag25(0) );
      /*
      for (var i = 0; i<3; i++)
      {
         CHECK_RC_CLEANUP( RtcTest.Diag25(i) );
         CHECK_RC_CLEANUP( RtcTest.Diag26(i) );
         CHECK_RC_CLEANUP( RtcTest.Diag27(i) );
         CHECK_RC_CLEANUP( RtcTest.Diag28(i) );
      }
      */

      // Alarm Test
      CHECK_RC_CLEANUP( RtcTest.Diag30(1, 1) );
      /*
      CHECK_RC_CLEANUP( RtcTest.Diag30(1, 0) );
      CHECK_RC_CLEANUP( RtcTest.Diag30(0, 1) );
      CHECK_RC_CLEANUP( RtcTest.Diag30(0, 0) );
      CHECK_RC_CLEANUP( RtcTest.Diag31(1, 1) );
      CHECK_RC_CLEANUP( RtcTest.Diag31(1, 0) );
      CHECK_RC_CLEANUP( RtcTest.Diag31(0, 1) );
      CHECK_RC_CLEANUP( RtcTest.Diag31(0, 0) );
      CHECK_RC_CLEANUP( RtcTest.Diag32(1, 1) );
      CHECK_RC_CLEANUP( RtcTest.Diag32(1, 0) );
      CHECK_RC_CLEANUP( RtcTest.Diag32(0, 1) );
      CHECK_RC_CLEANUP( RtcTest.Diag32(0, 0) );
      CHECK_RC_CLEANUP( RtcTest.Diag33(1, 1) );
      CHECK_RC_CLEANUP( RtcTest.Diag33(1, 0) );
      CHECK_RC_CLEANUP( RtcTest.Diag33(0, 1) );
      CHECK_RC_CLEANUP( RtcTest.Diag33(0, 0) );
      */

      // The Interrupt and Accuracy test need to hook up Isr, only run under dos.
      if ( OperatingSystem == "Dos")
      {
         // Interrupt Test.
         CHECK_RC_CLEANUP( RtcTest.Diag40() );
         // Accuracy Test
         for ( var i = 1; i<4; i++) // min i = 0, max i = 0xf
         {
            CHECK_RC_CLEANUP( RtcTest.Diag50(i*2) );
         }
      }
   }

   RtcTest.Restore();
   return rc;
}

function RunPs2Test()
{
   // Ps2 Mouse Loopback tests
   var rc = OK;

   CHECK_RC(Ps2Test.Initialize());
   CHECK_RC(Ps2Test.Diag20());
   CHECK_RC(Ps2Test.Diag10());
   CHECK_RC(Ps2Test.Diag11());

   return OK;
}

function RunTimerTest()
{
   // Timer Accuracy tests
   var rc = OK;

   CHECK_RC(TimerTest.Initialize());

   Cleanup:
   {
      CHECK_RC_CLEANUP(TimerTest.Diag10());
      CHECK_RC_CLEANUP(TimerTest.Diag11());
      CHECK_RC_CLEANUP(TimerTest.Diag15());
      CHECK_RC_CLEANUP(TimerTest.Diag16());
      CHECK_RC_CLEANUP(TimerTest.Diag20());
      CHECK_RC_CLEANUP(TimerTest.Diag21());
      CHECK_RC_CLEANUP(TimerTest.Diag25());
      CHECK_RC_CLEANUP(TimerTest.Diag26());
   }

   TimerTest.Restore();

   return rc;
}

function RunGameTest()
{
   // Game Port Test
   var rc = OK;
   CHECK_RC(GameTest.Initialize());

   // X, Y postion test (Resister test)
   CHECK_RC(GameTest.Diag(0));
   CHECK_RC(GameTest.Diag(1));
   CHECK_RC(GameTest.Diag(2));
   CHECK_RC(GameTest.Diag(3));

   // Button Test
   CHECK_RC(GameTest.Diag(4));
   CHECK_RC(GameTest.Diag(5));
   CHECK_RC(GameTest.Diag(6));
   CHECK_RC(GameTest.Diag(7));
   return OK;
}

function RunSystemStressTest()
{
   // Run the system stress test.
   // Concurrently run the ACI/CODEC, MAC/PHY, ATA/hard drive,
   // system memory stress test, and frame buffer stress test.
   var rc = OK;

   if (!g_IsMcpInitialized)
      CHECK_RC(InitializeMcpTests());

   if (!g_IsGpuInitialized)
      CHECK_RC(InitializeGpuTests());

   if( g_StressDoAta )
   {
      SystemStressTest.DoAta = true;
      CHECK_RC(Ata.Initialize(g_AtaClassCode));
   }
   else
      SystemStressTest.DoAta = false;

   // LegancyEthernet only works for Mcp1.
   if( g_StressDoEth )
   {
      SystemStressTest.DoEthernet = true;
      CHECK_RC(LegacyEthernet.Initialize());
   }
   else
      SystemStressTest.DoEthernet = false;

   if( g_StressDoAci )
   {
      SystemStressTest.DoAci = true;
      CHECK_RC(Audio.Initialize());
   }
   else
      SystemStressTest.DoAci = false;

   return SystemStressTest.Run();
}

//------------------------------------------------------------------------------
// Perform any product specific overrides.
//
function DoGpuOverrides()
{
   var rc;
   var i;

   // Only run the OQA tests if TM_OQA mode.
   if (TM_OQA == g_TestMode)
   {
      g_TestMask.Disable();
      g_TestMask.Enable( 1);  // CheckRomVersion
      g_TestMask.Enable(30);  // CheckRevision
      g_TestMask.Enable(25);  // CheckSubsystemIds
      g_TestMask.Enable(40);  // CheckTvEncoderType
      g_TestMask.Enable(17);  // MemSize
      g_TestMask.Enable( 3);  // Mats
      g_TestMask.Enable( 5);  // Class039
      g_TestMask.Enable(10);  // Class05f
      g_TestMask.Enable(21);  // Class07a
      g_TestMask.Enable(16);  // glr_hwtest
   }
   else if (TM_GPU == g_TestMode)
   {
      // Increase the loop counts for the full manufacture tests.
      Class039.TestConfiguration.Loops =   250;
      Class05f.TestConfiguration.Loops =  1000;
      Class07a.TestConfiguration.Loops =    35;
      GLRandom.TestConfiguration.Loops = 15000;

      // Only run I2CTest on NV30 and later GPUs.
      if (Gpu.DeviceId >= Gpu.NV35)
         FindTest(g_GpuTests, "I2CTest.Run").OperatingSystems |= OS_DOS;
   }
   else if (TM_ENGR == g_TestMode)
   {
      FindTest(g_GpuTests, "MemInterface.Run").OperatingSystems |= OS_DOS | OS_MAC | OS_LINUX;
      FindTest(g_GpuTests, "MemPatt.Run"     ).OperatingSystems |= OS_DOS | OS_MAC | OS_LINUX;
      FindTest(g_GpuTests, "Class05c.Run"    ).OperatingSystems |= OS_DOS | OS_MAC | OS_LINUX;
      FindTest(g_GpuTests, "Old05d.Run"      ).OperatingSystems |= OS_DOS | OS_MAC | OS_LINUX;
      FindTest(g_GpuTests, "Class061.Run"    ).OperatingSystems |= OS_DOS | OS_MAC | OS_LINUX;
      FindTest(g_GpuTests, "Class064.Run"    ).OperatingSystems |= OS_DOS | OS_MAC | OS_LINUX;
      FindTest(g_GpuTests, "Class038.Run"    ).OperatingSystems |= OS_DOS | OS_MAC | OS_LINUX;
      FindTest(g_GpuTests, "WfMats.Run"      ).OperatingSystems |= OS_DOS | OS_MAC | OS_LINUX;

      // GLStress uses GL color logic op XOR, which is emulated in software
      // on pre-NV20 chips, except for NV11/15/17/18 Quadro boards.
      if (Gpu.DeviceId >= Gpu.NV20)
         FindTest(g_GpuTests, "GLStress.Run" ).OperatingSystems |= OS_DOS | OS_MAC | OS_LINUX;

      // Only run I2CTest on NV30 and later GPUs.
      if (Gpu.DeviceId >= Gpu.NV35)
         FindTest(g_GpuTests, "I2CTest.Run").OperatingSystems |= OS_DOS;

      Mats.Coverage = 100; // Test 100% of memory.
      Class039.TestConfiguration.Loops =  1000;
      AgpTest.TestConfiguration.Loops  =  5000;
      Class07c.TestConfiguration.Loops =    25;
      Class04a.TestConfiguration.Loops =  1000;
      Class05f.TestConfiguration.Loops =  5000;
      Class077.TestConfiguration.Loops =  1000;
      Class07a.TestConfiguration.Loops =   100;
      GLRandom.TestConfiguration.Loops = 30000;
   }

   if (Ikos)
   {
      // We're running on a very slow hardware simulator.
      // Store image files.
      Golden.DumpTga = Golden.OnStore | Golden.OnError;

      // Global settings; low resolutions, just one frame per test.
      TestConfiguration.DisplayWidth  = 640;
      TestConfiguration.DisplayHeight = 480;
      TestConfiguration.Loops         = Golden.SkipCount;
      TestConfiguration.TimeoutMs     = 30000;

      // Check a small fraction of the memory.
      Mats.Coverage = 10;
      Mats.End = 1;
      FastMats.End = 1;
      WfMats.End = 1;

      // Each test: low resolution, one frame per test.
      for (i in g_GpuTests)
      {
         var obj = g_GpuTests[i].Object;
         if (obj == null)
            continue;

         var obj_loops = TestConfiguration.Loops;

         if ((obj.Golden           != undefined) &&
             (obj.Golden.SkipCount != undefined))
         {
            obj_loops = obj.Golden.SkipCount;

            if (obj.TestConfiguration == undefined)
               obj.TestConfiguration = new Object;
         }

         if (obj.TestConfiguration != undefined)
         {
            obj.TestConfiguration.DisplayWidth  = 640;
            obj.TestConfiguration.DisplayHeight = 480;
            obj.TestConfiguration.Loops         = 2 * obj_loops;  // 2 frames
         }
      }
   }

   // If displaying on a TV, run all tests in 640x480 mode.
   if (Display.IsTv(Display.Selected))
   {
      for (i in g_GpuTests)
      {
         var obj = g_GpuTests[i].Object;
         if
         (
               (obj != null)
            && (obj.TestConfiguration != undefined)
         )
         {
            obj.TestConfiguration.DisplayWidth  = 640;
            obj.TestConfiguration.DisplayHeight = 480;
         }
      }
   }

   switch (Gpu.DeviceId - 0)
   {
      case Gpu.NV5:
      {
         // Do not run Class038, Class07a, Class089, and Class07c  test on NV5.
         FindTest(g_GpuTests, "Class038.Run").OperatingSystems = OS_NONE;
         FindTest(g_GpuTests, "Class07c.Run").OperatingSystems = OS_NONE;
         FindTest(g_GpuTests, "Class07a.Run").OperatingSystems = OS_NONE;
         FindTest(g_GpuTests, "Class089.Run").OperatingSystems = OS_NONE;
         break;
      }

      case Gpu.NV10:
      {
         // Run the Class046 instead of Class07c cursor test on NV10.
         // Class07c is only available on NV11 and higher.
         FindTest(g_GpuTests, "Class046.Run").OperatingSystems = OS_DOS;
         FindTest(g_GpuTests, "Class07c.Run").OperatingSystems = OS_NONE;
         break;
      }

      case Gpu.NV15:
      {
         if (0x0151 == Gpu.PciDeviceId)
         {
            // Do not run MemRefresh on Ti products.
            FindTest(g_GpuTests, "MemRefresh.Run").OperatingSystems = OS_NONE;
         }

         // The PllTest is not supported on NV15.
         FindTest(g_GpuTests, "PllTest.Run").OperatingSystems = OS_NONE;
         break;
      }

      case Gpu.NV17:
      case Gpu.NV18:
      case Gpu.NV1F:
      case Gpu.NV30:
      case Gpu.NV35:
      {
         // Run Class1774 on NV17, NV18, NV1F, NV30, and NV35.
         FindTest(g_GpuTests, "Class1774.Run").OperatingSystems = (OS_DOS | OS_MAC);
         break;
      }

      case Gpu.NV20:
      {
         if
         (
               (0x0201 == Gpu.PciDeviceId)
            || (0x0202 == Gpu.PciDeviceId)
         )
         {
            // Do not run MemRefresh on Ti products.
            FindTest(g_GpuTests, "MemRefresh.Run").OperatingSystems = OS_NONE;
         }

         // The PllTest is not supported on NV20.
         FindTest(g_GpuTests, "PllTest.Run").OperatingSystems = OS_NONE;
         break;
      }

      case Gpu.NV25:
      {
         // Differentiate golden values between A2 and A3 silicon revisions.
         if (Bus.Revision == 0xA2)
         {
            Golden.Name += "_A2";
         }
         break;
      }

      case Gpu.NV2A:
      {
         Class038.TestConfiguration.Loops =  1000;
         Class04a.TestConfiguration.Loops =   500;
         Class05f.TestConfiguration.Loops =   500;
         Class07a.TestConfiguration.Loops =   100;
         GLRandom.TestConfiguration.Loops = 20000;
         break;
      }

      case Gpu.NV31:
      case Gpu.NV34:
      case Gpu.NV36:
      {
         // Run Class3174 on NV31, NV34, and NV36.
         FindTest(g_GpuTests, "Class3174.Run" ).OperatingSystems = (OS_DOS | OS_MAC);
         break;
      }

      default:
      {
         Out.Printf(Out.PriLow, "No overrides for NV%x.\n", Gpu.DeviceId);
         break;
      }
   }

   // Import the override script file if necessary.
   if (g_OverrideScriptFile != null)
   {
      CHECK_RC(Import(g_OverrideScriptFile));
   }

   return OK;
}

//------------------------------------------------------------------------------
// Warning Message stub for disabled tests
//
function WarningMessage()
{
   Out.Printf(Out.PriHigh, "Warning, nonexistant test number.\n");
   Out.Printf(Out.PriHigh, " No test performed.\n");

   return OK;
}


//------------------------------------------------------------------------------
// Print the system information.
//
function PrintSystemInfo()
{
   var Chipsets =
   {
      "10DE" :
      {
         "Vendor" : "NVIDIA",
         "01A4"   : "nForce",
         "01A5"   : "nForce",
         "01A6"   : "nForce",
         "01E0"   : "nForce2",
         "01E1"   : "nForce2",
         "00D1"   : "CK8",
      },
      "8086" :
      {
         "Vendor" : "Intel",
         "7180"   : "440LX",
         "7190"   : "440BX",
         "71A0"   : "440GX",
         "1130"   : "815",
         "2500"   : "820",
         "2501"   : "820",
         "3575"   : "830M",
         "1A20"   : "840",
         "1A21"   : "840",
         "1A22"   : "840",
         "1A30"   : "845",
         "2560"   : "845G",
         "2530"   : "850",
         "2531"   : "860",
         "255D"   : "Granite Bay",
         "2550"   : "Placer",
         "2570"   : "Springdale",
         "2578"   : "Canterwood",
         "3340"   : "Odem",
         "3580"   : "Presswood",
      },
      "1022" :
      {
         "Vendor" : "AMD",
         "7006"   : "751",
         "700E"   : "761",
         "700C"   : "762",
         "7454"   : "8151",
      },
      "1106" :
      {
         "Vendor" : "VIA",
         "0691"   : "8371/82C694",
         "0391"   : "8371/82C694",
         "0305"   : "KT133",
         "3099"   : "KT266/KT333",
         "3128"   : "P4X266A",
         "3148"   : "P4M266",
         "3168"   : "P4X333/P4X400",
         "3189"   : "KT400",
      },
      "1166" :
      {
         "Vendor" : "ServerWorks",
         "0005"   : "6585HE",
         "0009"   : "6585HE",
      },
      // Micron has two PCI device IDs
      "1344" :
      {
         "Vendor" : "Micron",
         "3120"   : "SAMDDR",
         "3200"   : "SCIDDR",
      },
      "1042" :
      {
         "Vendor" : "Micron",
         "3120"   : "SAMDDR",
         "3200"   : "SCIDDR",
      },
      "10B9" :
      {
         "Vendor" : "ALi",
         "1621"   : "M1621",
         "1631"   : "M1631",
         "1647"   : "M1647",
         "1651"   : "M1651",
         "1671"   : "M1671",
      },
      "1039" :
      {
         "Vendor" : "SiS",
         "0630"   : "630",
         "0633"   : "633",
         "0635"   : "635",
         "0645"   : "645",
         "0646"   : "645DX",
         "0648"   : "648",
         "0650"   : "650",
         "0651"   : "651",
         "0655"   : "655",
         "0730"   : "730",
         "0733"   : "733",
         "0735"   : "735",
         "0745"   : "745",
      },
      "1002" :
      {
         "Vendor" : "ATI",
         "CAB2"   : "IGP340M",
      },
      "1279" :
      {
         "Vendor" : "Transmeta",
         "0060"   : "Astro",
         "0395"   : "TM5800 Crusoe",
      },
   };

   // Print the command line arguments.
   Out.Printf(Out.PriHigh, "\nCommand Line : ");
   for (i = 0; i < Argv.length; ++i)
   {
      Out.Printf(Out.PriHigh, "%s ", Argv[i]);
   }
   Out.Printf(Out.PriHigh, "\n");

   if (g_RecordId)
   {
      Out.Printf(Out.PriHigh, "\nTest ID : %s\n", g_TestId);
   }

   var Matches;
   Out.Printf(Out.PriHigh,"\nVersion\n");
   Out.Printf(Out.PriHigh,"MODS        : %s\n", Version);
   Out.Printf(Out.PriHigh,"mfgtest.js  : %s\n", ((Matches = g_MfgtestId.match(/#(\d+)/))  != undefined) ? Matches[1] : "unknown");
   Out.Printf(Out.PriHigh,"glrandom.js : %s\n", ((Matches = g_GlrandomId.match(/#(\d+)/)) != undefined) ? Matches[1] : "unknown");
   Out.Printf(Out.PriHigh,"boards.js   : %s\n", ((Matches = g_BoardsJsId.match(/#(\d+)/)) != undefined) ? Matches[1] : "unknown");
   Out.Printf(Out.PriHigh,"progtv.js   : %s\n", ((Matches = g_ProgtvId.match(/#(\d+)/))   != undefined) ? Matches[1] : "unknown");

   if (g_IsGpuInitialized)
   {
      // For DOS only, make sure we are always back to text mode to display the results.
      if ("Dos" == OperatingSystem)
      {
         if (!Vga.MONOCHROME)
            Vga.SetVesaMode(3);
      }

      var RomVersion = (("Mac" == OperatingSystem) || ("MacOSX" == OperatingSystem)) ?
                                      GetFcodeVersion() : BiosVersion;

      Out.Printf(Out.PriHigh, "\nGPU Configuration\n");
      Out.Printf(Out.PriHigh, "DID            : %#06x (%s)\n", Gpu.PciDeviceId, Gpu.DeviceName);
      Out.Printf(Out.PriHigh, "Revision       : %#04x\n",      Bus.Revision);
      Out.Printf(Out.PriHigh, "Subsystem VID  : %#06x\n",      Bus.SubsystemVendorId);
      Out.Printf(Out.PriHigh, "Subsystem DID  : %#06x\n",      Bus.SubsystemDeviceId);
      Out.Printf(Out.PriHigh, "Display        : %#010x\n",     Display.Selected);
      Out.Printf(Out.PriHigh, "Memory Size    : %u MB\n",      FrameBuffer.STATIC_RAMAMOUNT / (1024*1024));
      Out.Printf(Out.PriHigh, "Memory Type    : %s\n",         FrameBuffer.TYPE_DDR ? "DDR" : "SDR");
      Out.Printf(Out.PriHigh, "ROM Version    : %s\n",         RomVersion);
      Out.Printf(Out.PriHigh, "TV Encoder     : %s\n",         TvEncoder);
      Out.Printf(Out.PriHigh, "NV Base        : %#010x\n",     Bus.NvBaseAddress);
      Out.Printf(Out.PriHigh, "FB Base        : %#010x\n",     Bus.FbBaseAddress);
      Out.Printf(Out.PriHigh, "IRQ            : %u\n",         Bus.Irq);
      if ((g_TestGraphicsClock_Hz != 0) && (g_TestMemoryClock_Hz != 0))
      {
         Out.Printf(Out.PriHigh, "Graphics Clock : %5.1f MHz\n", g_TestGraphicsClock_Hz / 1e6);
         Out.Printf(Out.PriHigh, "Memory Clock   : %5.1f MHz\n", g_TestMemoryClock_Hz / 1e6);
      }
      if (null != g_GpuCoreVoltage)
      {
         Out.Printf(Out.PriHigh, "Core Voltage   : %4.2f V\n", g_GpuCoreVoltage);
      }

      if (Bus.IsAgpEnabled)
      {
         Out.Printf(Out.PriHigh, "\nAGP Configuration\n");
         Out.Printf(Out.PriHigh, "Revision       : %s\n", Bus.IsAgp3 ? "3.0" : "2.0");
         Out.Printf(Out.PriHigh, "Request Depth  : %u\n", Bus.AgpRequestDepth);
         Out.Printf(Out.PriHigh, "Sideband       : %s\n", Bus.IsAgpSbaEnabled ? "Enabled" : "Disabled");
         Out.Printf(Out.PriHigh, "Fastwrite      : %s\n", Bus.IsAgpFwEnabled ? "Enabled" : "Disabled");
         Out.Printf(Out.PriHigh, "Data Rate      : %u\n", Bus.AgpDataRate);
      }
      else
      {
         Out.Printf(Out.PriHigh, "\nAGP is disabled.\n");
      }

      if (OperatingSystem == "Dos")
      {
         // Chipset info.
         var Vid = Out.Sprintf("%04X", Pci.Read16(0, 0, 0, 0));
         var Did = Out.Sprintf("%04X", Pci.Read16(0, 0, 0, 2));
         var VidName = "unknown";
         var DidName = "unknown";
         if (Chipsets[Vid] != undefined)
         {
            VidName = Chipsets[Vid].Vendor;
            if (Chipsets[Vid][Did] != undefined)
            {
               DidName = Chipsets[Vid][Did]
            }
         }

         Out.Printf(Out.PriHigh, "\nChipset\n");
         Out.Printf(Out.PriHigh, "VID : %s (%s)\n", Vid, VidName);
         Out.Printf(Out.PriHigh, "DID : %s (%s)\n", Did, DidName);

         // CPU info.
         Out.Printf(Out.PriHigh, "\nCPU\n");
         Out.Printf(Out.PriHigh, "%s", Cpu.Id);
         Out.Printf(Out.PriHigh, "Speed     : %d MHz\n\n", Cpu.Speed);
      }
   }

   Out.Flush();

   return OK;
}

//------------------------------------------------------------------------------
// Print the errors.
//
function PrintErrors()
{
   if (g_FailedTests.length > 1)
   {
      Out.Printf(Out.PriHigh, "\nFailures :\n");
      for (var i in g_FailedTests)
      {
         Out.Printf(Out.PriHigh, "\tLoop = %d\t\tTest = %s\t\tError Code = %d\n",
            g_FailedTests[i].Loop, g_FailedTests[i].Test, g_FailedTests[i].ErrorCode);
      }
   }
   else
   {
      if (g_FailedTests[0] != undefined)
         Out.Printf(Out.PriHigh, "\n%s test failed.\n", g_FailedTests[0].Test);

      Out.Printf(Out.PriHigh, "\nError Code = %d\n", Log.FirstError);
   }

   if (OK == Log.FirstError)
   {
      Out.Printf(Out.PriHigh,
                 "\n"
                 + "#######     ####     ######    ######  \n"
                 + "########   ######   ########  ######## \n"
                 + "##    ##  ##    ##  ##     #  ##     # \n"
                 + "##    ##  ##    ##   ###       ###     \n"
                 + "########  ########    ####      ####   \n"
                 + "#######   ########      ###       ###  \n"
                 + "##        ##    ##  #     ##  #     ## \n"
                 + "##        ##    ##  ########  ######## \n"
                 + "##        ##    ##   ######    ######  \n"
                 + "\n"
      );
   }
   else
   {
      Out.Printf(Out.PriHigh,
                 "\n"
                 + "#######     ####    ########  ###      \n"
                 + "#######    ######   ########  ###      \n"
                 + "##        ##    ##     ##     ###      \n"
                 + "##        ##    ##     ##     ###      \n"
                 + "#######   ########     ##     ###      \n"
                 + "#######   ########     ##     ###      \n"
                 + "##        ##    ##     ##     ###      \n"
                 + "##        ##    ##  ########  ######## \n"
                 + "##        ##    ##  ########  ######## \n"
                 + "\n"
      );
   }

   return OK;
}

//------------------------------------------------------------------------------
// TCO support.
//
function InitializeTco()
{
   var rc= OK;
   CHECK_RC( FindMcp() );

   if(g_McpVersion==Mcp.MCP1)
   {
      SysCtrl.CreateCrush11();
   }
   else if( (g_McpVersion==Mcp.MCP2)
            ||(g_McpVersion==Mcp.CK8)
            ||(g_McpVersion == Mcp.CK8G)
            ||(g_McpVersion == Mcp.MCP2S) )
   {
      SysCtrl.CreateCrush17();
   }
   else
   {
      Out.Printf(Out.PriHigh, "*** ERROR: Invlid McpChip ***\n");
      return RC.INVALID_MCP_VERSION;
   }

   CHECK_RC( SysCtrl.Initialize() );

   //Set Timer Halt Bit
   SysCtrl.TcoTimerHalt();

   // Set Timer Value
   SysCtrl.TcoTimerSetValue(0x05);

   // Set reboot bit
   SysCtrl.TcoTimerSetReboot();

   // Clear Timer Halt Bit
   SysCtrl.TcoTimerRun();

   return OK;
}

function TcoThread()
{
   Out.Printf(Out.PriNormal, "TcoThread\n");
   var NumReset = 0;

   while(g_IsTcoReboot)
   {
      Out.Printf(Out.PriLow, "+++++ g_IsTcoReboot True, Reset %i +++++\n", NumReset);

      // Reset around 5 sec
      SysCtrl.TcoTimerResetValue(0x05);

      Sleep(3000);
      NumReset++;
   }

   Out.Printf(Out.PriNormal, "+++++ g_IsTcoReboot %i, Reset %i times +++++\n", g_IsTcoReboot, NumReset);
   return OK;
}

//------------------------------------------------------------------------------
// main (if begin() is defined, begin() will be called before main()).
//
function main()
{
   var rc = OK;

   // Parse the command line arguments.
   CHECK_RC(ParseArgv());

   // Set output level.
   if (g_IsVerbose)
   {
      Out.ScreenLevel = Out.LevLow;
      Out.FileLevel   = Out.LevLow;
   }

   Log.RecordTime = g_RecordTime;

   if (g_RecordId)
   {
      Out.Printf(Out.ScreenOnly, "\nTest ID? ");
      g_TestId = In.readln();
   }

   if(g_IsTcoReboot)
   {
      CHECK_RC( InitializeTco() );

      // Set Thread to Run.
      g_TcoThreadId = Tasker.CreateThread( "TcoThread", TcoThread );
      Out.Printf(Out.PriLow, "+++ TcoThreadId 0x%x +++\n", g_TcoThreadId);
   }

   if (TM_MCP == g_TestMode)
   {
      CHECK_RC(InitializeMcpTests());
      PrintSystemInfo();
      if (!g_NoTest)
         CHECK_RC(RunMcpTests());
   }
   else if (TM_ACR == g_TestMode)
   {
      CHECK_RC(InitializeMcpTests());
      PrintSystemInfo();
      if (!g_NoTest)
         CHECK_RC(RunAcrTests());
   }
   else if (TM_BAY == g_TestMode)
   {
      CHECK_RC(InitializeBayTests());
      PrintSystemInfo();
      if (!g_NoTest)
         CHECK_RC(RunBayTests());
   }
   else
   {
      CHECK_RC(InitializeGpuTests());
      PrintSystemInfo();
      if (!g_NoTest)
         CHECK_RC(RunGpuTests());
   }

   return OK;
}

